@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,57 +12,59 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "entry.h"
 
 #include <typeinfo>
 
+#include <QDebug>
+
 #include "file.h"
 
 // FIXME: Check if using those constants in the program is really necessary
 // or can be replace by config files
-const QLatin1String Entry::ftAbstract = QLatin1String("abstract");
-const QLatin1String Entry::ftAddress = QLatin1String("address");
-const QLatin1String Entry::ftAuthor = QLatin1String("author");
-const QLatin1String Entry::ftBookTitle = QLatin1String("booktitle");
-const QLatin1String Entry::ftChapter = QLatin1String("chapter");
-const QLatin1String Entry::ftColor = QLatin1String("x-color");
-const QLatin1String Entry::ftComment = QLatin1String("comment");
-const QLatin1String Entry::ftCrossRef = QLatin1String("crossref");
-const QLatin1String Entry::ftDOI = QLatin1String("doi");
-const QLatin1String Entry::ftEditor = QLatin1String("editor");
-const QLatin1String Entry::ftFile = QLatin1String("file");
-const QLatin1String Entry::ftISSN = QLatin1String("issn");
-const QLatin1String Entry::ftISBN = QLatin1String("isbn");
-const QLatin1String Entry::ftJournal = QLatin1String("journal");
-const QLatin1String Entry::ftKeywords = QLatin1String("keywords");
-const QLatin1String Entry::ftLocalFile = QLatin1String("localfile");
-const QLatin1String Entry::ftLocation = QLatin1String("location");
-const QLatin1String Entry::ftMonth = QLatin1String("month");
-const QLatin1String Entry::ftNote = QLatin1String("note");
-const QLatin1String Entry::ftNumber = QLatin1String("number");
-const QLatin1String Entry::ftPages = QLatin1String("pages");
-const QLatin1String Entry::ftPublisher = QLatin1String("publisher");
-const QLatin1String Entry::ftSchool = QLatin1String("school");
-const QLatin1String Entry::ftSeries = QLatin1String("series");
-const QLatin1String Entry::ftStarRating = QLatin1String("x-stars");
-const QLatin1String Entry::ftTitle = QLatin1String("title");
-const QLatin1String Entry::ftUrl = QLatin1String("url");
-const QLatin1String Entry::ftUrlDate = QLatin1String("urldate");
-const QLatin1String Entry::ftVolume = QLatin1String("volume");
-const QLatin1String Entry::ftYear = QLatin1String("year");
+const QString Entry::ftAbstract = QStringLiteral("abstract");
+const QString Entry::ftAddress = QStringLiteral("address");
+const QString Entry::ftAuthor = QStringLiteral("author");
+const QString Entry::ftBookTitle = QStringLiteral("booktitle");
+const QString Entry::ftChapter = QStringLiteral("chapter");
+const QString Entry::ftColor = QStringLiteral("x-color");
+const QString Entry::ftComment = QStringLiteral("comment");
+const QString Entry::ftCrossRef = QStringLiteral("crossref");
+const QString Entry::ftDOI = QStringLiteral("doi");
+const QString Entry::ftEditor = QStringLiteral("editor");
+const QString Entry::ftFile = QStringLiteral("file");
+const QString Entry::ftISSN = QStringLiteral("issn");
+const QString Entry::ftISBN = QStringLiteral("isbn");
+const QString Entry::ftJournal = QStringLiteral("journal");
+const QString Entry::ftKeywords = QStringLiteral("keywords");
+const QString Entry::ftLocalFile = QStringLiteral("localfile");
+const QString Entry::ftLocation = QStringLiteral("location");
+const QString Entry::ftMonth = QStringLiteral("month");
+const QString Entry::ftNote = QStringLiteral("note");
+const QString Entry::ftNumber = QStringLiteral("number");
+const QString Entry::ftPages = QStringLiteral("pages");
+const QString Entry::ftPublisher = QStringLiteral("publisher");
+const QString Entry::ftSchool = QStringLiteral("school");
+const QString Entry::ftSeries = QStringLiteral("series");
+const QString Entry::ftStarRating = QStringLiteral("x-stars");
+const QString Entry::ftTitle = QStringLiteral("title");
+const QString Entry::ftUrl = QStringLiteral("url");
+const QString Entry::ftUrlDate = QStringLiteral("urldate");
+const QString Entry::ftVolume = QStringLiteral("volume");
+const QString Entry::ftYear = QStringLiteral("year");
 
-const QLatin1String Entry::etArticle = QLatin1String("article");
-const QLatin1String Entry::etBook = QLatin1String("book");
-const QLatin1String Entry::etInBook = QLatin1String("inbook");
-const QLatin1String Entry::etInProceedings = QLatin1String("inproceedings");
-const QLatin1String Entry::etMisc = QLatin1String("misc");
-const QLatin1String Entry::etPhDThesis = QLatin1String("phdthesis");
-const QLatin1String Entry::etMastersThesis = QLatin1String("mastersthesis");
-const QLatin1String Entry::etTechReport = QLatin1String("techreport");
-const QLatin1String Entry::etUnpublished = QLatin1String("unpublished");
+const QString Entry::etArticle = QStringLiteral("article");
+const QString Entry::etBook = QStringLiteral("book");
+const QString Entry::etInBook = QStringLiteral("inbook");
+const QString Entry::etInProceedings = QStringLiteral("inproceedings");
+const QString Entry::etMisc = QStringLiteral("misc");
+const QString Entry::etPhDThesis = QStringLiteral("phdthesis");
+const QString Entry::etMastersThesis = QStringLiteral("mastersthesis");
+const QString Entry::etTechReport = QStringLiteral("techreport");
+const QString Entry::etUnpublished = QStringLiteral("unpublished");
 
 quint64 Entry::internalUniqueIdCounter = 0;
 
@@ -161,17 +163,19 @@ const Value Entry::value(const QString &key) const
         if (it.key().toLower() == lcKey)
             return QMap<QString, Value>::value(it.key());
 
-    return QMap<QString, Value>::value(key);
+    return Value();
 }
 
 int Entry::remove(const QString &key)
 {
     const QString lcKey = key.toLower();
-    for (Entry::ConstIterator it = constBegin(); it != constEnd(); ++it)
-        if (it.key().toLower() == lcKey)
-            return QMap<QString, Value>::remove(it.key());
+    for (Entry::Iterator it = begin(); it != end(); ++it)
+        if (it.key().toLower() == lcKey) {
+            QMap<QString, Value>::erase(it);
+            return 1;
+        }
 
-    return QMap<QString, Value>::remove(key);
+    return 0;
 }
 
 bool Entry::contains(const QString &key) const
@@ -193,7 +197,7 @@ Entry *Entry::resolveCrossref(const Entry &original, const File *bibTeXfile)
 {
     Entry *result = new Entry(original);
 
-    if (bibTeXfile == NULL)
+    if (bibTeXfile == nullptr)
         return result;
 
     const QString crossRef = PlainTextValue::text(original.value(ftCrossRef));
@@ -231,7 +235,8 @@ QStringList Entry::authorsLastName(const Entry &entry)
 
     QStringList result;
     int maxAuthors = 16; ///< limit the number of authors considered
-    foreach (const QSharedPointer<const ValueItem> &item, value) {
+    result.reserve(maxAuthors);
+    for (const QSharedPointer<const ValueItem> &item : const_cast<const Value &>(value)) {
         QSharedPointer<const Person> person = item.dynamicCast<const Person>();
         if (!person.isNull()) {
             const QString lastName = person->lastName();
@@ -252,3 +257,8 @@ QStringList Entry::authorsLastName() const
 bool Entry::isEntry(const Element &other) {
     return typeid(other) == typeid(Entry);
 }
+
+QDebug operator<<(QDebug dbg, const Entry &entry) {
+    dbg.nospace() << "Entry " << entry.id() << " (uniqueId=" << entry.uniqueId() << "), has " << entry.count() << " key-value pairs";
+    return dbg;
+}
