@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,7 +12,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "value.h"
@@ -22,9 +22,13 @@
 #include <QSet>
 #include <QString>
 #include <QStringList>
+#include <QDebug>
 
+#ifdef HAVE_KF5
 #include <KSharedConfig>
 #include <KConfigGroup>
+#include <KLocalizedString>
+#endif // HAVE_KF5
 
 #include "file.h"
 #include "preferences.h"
@@ -93,7 +97,7 @@ bool Keyword::containsPattern(const QString &pattern, Qt::CaseSensitivity caseSe
 bool Keyword::operator==(const ValueItem &other) const
 {
     const Keyword *otherKeyword = dynamic_cast<const Keyword *>(&other);
-    if (otherKeyword != NULL) {
+    if (otherKeyword != nullptr) {
         return otherKeyword->text() == text();
     } else
         return false;
@@ -153,14 +157,14 @@ bool Person::containsPattern(const QString &pattern, Qt::CaseSensitivity caseSen
     const QString lastName = QString(m_lastName).remove(ignoredInSorting);
     const QString suffix = QString(m_suffix).remove(ignoredInSorting);
 
-    return firstName.contains(pattern, caseSensitive) || lastName.contains(pattern, caseSensitive) || suffix.contains(pattern, caseSensitive) || QString(QLatin1String("%1 %2|%2, %1")).arg(firstName, lastName).contains(pattern, caseSensitive);
+    return firstName.contains(pattern, caseSensitive) || lastName.contains(pattern, caseSensitive) || suffix.contains(pattern, caseSensitive) || QString(QStringLiteral("%1 %2|%2, %1")).arg(firstName, lastName).contains(pattern, caseSensitive);
 }
 
 bool Person::operator==(const ValueItem &other) const
 {
     const Person *otherPerson = dynamic_cast<const Person *>(&other);
-    if (otherPerson != NULL) {
-        return otherPerson->firstName() == firstName() && otherPerson->lastName() == lastName();
+    if (otherPerson != nullptr) {
+        return otherPerson->firstName() == firstName() && otherPerson->lastName() == lastName() && otherPerson->suffix() == suffix();
     } else
         return false;
 }
@@ -200,6 +204,11 @@ bool Person::isPerson(const ValueItem &other) {
     return typeid(other) == typeid(Person);
 }
 
+QDebug operator<<(QDebug dbg, const Person &person) {
+    dbg.nospace() << "Person " << Person::transcribePersonName(&person, Preferences::defaultPersonNameFormatting);
+    return dbg;
+}
+
 
 const QRegExp MacroKey::validMacroKey = QRegExp("^[a-z][-.:/+_a-z0-9]*$|^[0-9]+$", Qt::CaseInsensitive);
 
@@ -249,7 +258,7 @@ bool MacroKey::containsPattern(const QString &pattern, Qt::CaseSensitivity caseS
 bool MacroKey::operator==(const ValueItem &other) const
 {
     const MacroKey *otherMacroKey = dynamic_cast<const MacroKey *>(&other);
-    if (otherMacroKey != NULL) {
+    if (otherMacroKey != nullptr) {
         return otherMacroKey->text() == text();
     } else
         return false;
@@ -259,6 +268,11 @@ bool MacroKey::isMacroKey(const ValueItem &other) {
     return typeid(other) == typeid(MacroKey);
 }
 
+QDebug operator<<(QDebug dbg, const MacroKey &macrokey) {
+    dbg.nospace() << "MacroKey " << macrokey.text();
+    return dbg;
+}
+
 
 PlainText::PlainText(const PlainText &other)
         : m_text(other.text())
@@ -299,7 +313,7 @@ bool PlainText::containsPattern(const QString &pattern, Qt::CaseSensitivity case
 bool PlainText::operator==(const ValueItem &other) const
 {
     const PlainText *otherPlainText = dynamic_cast<const PlainText *>(&other);
-    if (otherPlainText != NULL) {
+    if (otherPlainText != nullptr) {
         return otherPlainText->text() == text();
     } else
         return false;
@@ -309,9 +323,16 @@ bool PlainText::isPlainText(const ValueItem &other) {
     return typeid(other) == typeid(PlainText);
 }
 
+QDebug operator<<(QDebug dbg, const PlainText &plainText) {
+    dbg.nospace() << "PlainText " << plainText.text();
+    return dbg;
+}
 
+
+#ifdef HAVE_KF5
 bool VerbatimText::colorLabelPairsInitialized = false;
 QList<VerbatimText::ColorLabelPair> VerbatimText::colorLabelPairs = QList<VerbatimText::ColorLabelPair>();
+#endif // HAVE_KF5
 
 VerbatimText::VerbatimText(const VerbatimText &other)
         : m_text(other.text())
@@ -347,16 +368,17 @@ bool VerbatimText::containsPattern(const QString &pattern, Qt::CaseSensitivity c
 {
     const QString text = QString(m_text).remove(ignoredInSorting);
 
+#ifdef HAVE_KF5
     /// Initialize map of labels to color (hex string) only once
     // FIXME if user changes colors/labels later, it will not be updated here
     if (!colorLabelPairsInitialized) {
         colorLabelPairsInitialized = true;
 
         /// Read data from config file
-        KSharedConfigPtr config(KSharedConfig::openConfig(QLatin1String("kbibtexrc")));
+        KSharedConfigPtr config(KSharedConfig::openConfig(QStringLiteral("kbibtexrc")));
         KConfigGroup configGroup(config, Preferences::groupColor);
-        QStringList colorCodes = configGroup.readEntry(Preferences::keyColorCodes, Preferences::defaultColorCodes);
-        QStringList colorLabels = configGroup.readEntry(Preferences::keyColorLabels, Preferences::defaultcolorLabels);
+        const QStringList colorCodes = configGroup.readEntry(Preferences::keyColorCodes, Preferences::defaultColorCodes);
+        const QStringList colorLabels = configGroup.readEntry(Preferences::keyColorLabels, Preferences::defaultColorLabels);
 
         /// Translate data from config file into internal mapping
         for (QStringList::ConstIterator itc = colorCodes.constBegin(), itl = colorLabels.constBegin(); itc != colorCodes.constEnd() && itl != colorLabels.constEnd(); ++itc, ++itl) {
@@ -366,15 +388,20 @@ bool VerbatimText::containsPattern(const QString &pattern, Qt::CaseSensitivity c
             colorLabelPairs << clp;
         }
     }
+#endif //  HAVE_KF5
 
     bool contained = text.contains(pattern, caseSensitive);
+#ifdef HAVE_KF5
     if (!contained) {
         /// Only if simple text match failed, check color labels
         /// For a match, the user's pattern has to be the start of the color label
         /// and this verbatim text has to contain the color as hex string
-        for (QList<ColorLabelPair>::ConstIterator it = colorLabelPairs.constBegin(); !contained && it != colorLabelPairs.constEnd(); ++it)
-            contained = text.compare(it->hexColor, Qt::CaseInsensitive) == 0 && it->label.contains(pattern, Qt::CaseInsensitive);
+        for (const auto &clp : const_cast<const QList<ColorLabelPair> &>(colorLabelPairs)) {
+            contained = text.compare(clp.hexColor, Qt::CaseInsensitive) == 0 && clp.label.contains(pattern, Qt::CaseInsensitive);
+            if (contained) break;
+        }
     }
+#endif // HAVE_KF5
 
     return contained;
 }
@@ -382,7 +409,7 @@ bool VerbatimText::containsPattern(const QString &pattern, Qt::CaseSensitivity c
 bool VerbatimText::operator==(const ValueItem &other) const
 {
     const VerbatimText *otherVerbatimText = dynamic_cast<const VerbatimText *>(&other);
-    if (otherVerbatimText != NULL) {
+    if (otherVerbatimText != nullptr) {
         return otherVerbatimText->text() == text();
     } else
         return false;
@@ -392,6 +419,11 @@ bool VerbatimText::isVerbatimText(const ValueItem &other) {
     return typeid(other) == typeid(VerbatimText);
 }
 
+QDebug operator<<(QDebug dbg, const VerbatimText &verbatimText) {
+    dbg.nospace() << "VerbatimText " << verbatimText.text();
+    return dbg;
+}
+
 
 Value::Value()
         : QVector<QSharedPointer<ValueItem> >()
@@ -400,10 +432,15 @@ Value::Value()
 }
 
 Value::Value(const Value &other)
-        : QVector<QSharedPointer<ValueItem> >()
+        : QVector<QSharedPointer<ValueItem> >(other)
 {
-    clear();
-    mergeFrom(other);
+    /// nothing
+}
+
+Value::Value(Value &&other)
+        : QVector<QSharedPointer<ValueItem> >(other)
+{
+    /// nothing
 }
 
 Value::~Value()
@@ -411,15 +448,8 @@ Value::~Value()
     clear();
 }
 
-void Value::merge(const Value &other)
-{
-    mergeFrom(other);
-}
-
 void Value::replace(const QString &before, const QString &after, ValueItem::ReplaceMode replaceMode)
 {
-    const QString valueText = PlainTextValue::text(*this);
-
     QSet<QSharedPointer<ValueItem> > unique;
     /// Delegate the replace operation to each individual ValueItem
     /// contained in this Value object
@@ -427,8 +457,9 @@ void Value::replace(const QString &before, const QString &after, ValueItem::Repl
         (*it)->replace(before, after, replaceMode);
 
         bool containedInUnique = false;
-        for (QSet<QSharedPointer<ValueItem> >::ConstIterator uit = unique.constBegin(); !containedInUnique && uit != unique.constEnd(); ++uit) {
-            containedInUnique = *(*uit).data() == *(*it).data();
+        for (const auto &valueItem : const_cast<const QSet<QSharedPointer<ValueItem> > &>(unique)) {
+            containedInUnique = *valueItem.data() == *(*it).data();
+            if (containedInUnique) break;
         }
 
         if (containedInUnique)
@@ -487,46 +518,58 @@ void Value::replace(const QString &before, const QSharedPointer<ValueItem> &afte
 
 bool Value::containsPattern(const QString &pattern, Qt::CaseSensitivity caseSensitive) const
 {
-    bool result = false;
-    for (Value::ConstIterator it = constBegin(); !result && it != constEnd(); ++it) {
-        result |= (*it)->containsPattern(pattern, caseSensitive);
+    for (const auto &valueItem : const_cast<const Value &>(*this)) {
+        if (valueItem->containsPattern(pattern, caseSensitive))
+            return true;
     }
-    return result;
+    return false;
 }
 
 bool Value::contains(const ValueItem &item) const
 {
-    for (Value::ConstIterator it = constBegin(); it != constEnd(); ++it)
-        if ((*it)->operator==(item))
+    for (const auto &valueItem : const_cast<const Value &>(*this))
+        if (valueItem->operator==(item))
             return true;
     return false;
 }
 
 Value &Value::operator=(const Value &rhs)
 {
-    clear();
-    mergeFrom(rhs);
-    return *this;
+    return static_cast<Value &>(QVector<QSharedPointer<ValueItem> >::operator =((rhs)));
 }
 
-void Value::mergeFrom(const Value &other)
+Value &Value::operator=(Value &&rhs)
 {
-    for (Value::ConstIterator it = other.constBegin(); it != other.constEnd(); ++it)
-        append(*it);
+    return static_cast<Value &>(QVector<QSharedPointer<ValueItem> >::operator =((rhs)));
 }
 
+Value &Value::operator<<(const QSharedPointer<ValueItem> &vi)
+{
+    return static_cast<Value &>(QVector<QSharedPointer<ValueItem> >::operator<<(vi));
+}
+
+QDebug operator<<(QDebug dbg, const Value &value) {
+    dbg.nospace() << "Value";
+    if (value.isEmpty())
+        dbg << " is empty";
+    else
+        dbg.nospace() << ": " << PlainTextValue::text(value);
+    return dbg;
+}
+
+
 QString PlainTextValue::text(const Value &value)
 {
     ValueItemType vit = VITOther;
     ValueItemType lastVit = VITOther;
 
     QString result;
-    for (Value::ConstIterator it = value.constBegin(); it != value.constEnd(); ++it) {
-        QString nextText = text(**it, vit);
+    for (const auto &valueItem : value) {
+        QString nextText = text(*valueItem, vit);
         if (!nextText.isEmpty()) {
             if (lastVit == VITPerson && vit == VITPerson)
                 result.append(i18n(" and ")); // TODO proper list of authors/editors, not just joined by "and"
-            else if (lastVit == VITPerson && vit == VITOther && nextText == QLatin1String("others")) {
+            else if (lastVit == VITPerson && vit == VITOther && nextText == QStringLiteral("others")) {
                 /// "and others" case: replace text to be appended by translated variant
                 nextText = i18n(" and others");
             } else if (lastVit == VITKeyword && vit == VITKeyword)
@@ -541,7 +584,7 @@ QString PlainTextValue::text(const Value &value)
     return result;
 }
 
-QString PlainTextValue::text(const QSharedPointer<ValueItem> &valueItem)
+QString PlainTextValue::text(const QSharedPointer<const ValueItem> &valueItem)
 {
     const ValueItem *p = valueItem.data();
     return text(*p);
@@ -558,31 +601,36 @@ QString PlainTextValue::text(const ValueItem &valueItem, ValueItemType &vit)
     QString result;
     vit = VITOther;
 
-    if (notificationListener == NULL)
+#ifdef HAVE_KF5
+    if (notificationListener == nullptr)
         notificationListener = new PlainTextValue();
+#endif // HAVE_KF5
 
+    bool isVerbatim = false;
     const PlainText *plainText = dynamic_cast<const PlainText *>(&valueItem);
-    if (plainText != NULL) {
+    if (plainText != nullptr) {
         result = plainText->text();
     } else {
         const MacroKey *macroKey = dynamic_cast<const MacroKey *>(&valueItem);
-        if (macroKey != NULL) {
+        if (macroKey != nullptr) {
             result = macroKey->text(); // TODO Use File to resolve key to full text
         } else {
             const Person *person = dynamic_cast<const Person *>(&valueItem);
-            if (person != NULL) {
+            if (person != nullptr) {
                 result = Person::transcribePersonName(person, personNameFormatting);
                 vit = VITPerson;
             } else {
                 const Keyword *keyword = dynamic_cast<const Keyword *>(&valueItem);
-                if (keyword != NULL) {
+                if (keyword != nullptr) {
                     result = keyword->text();
                     vit = VITKeyword;
                 } else {
                     const VerbatimText *verbatimText = dynamic_cast<const VerbatimText *>(&valueItem);
-                    if (verbatimText != NULL) {
+                    if (verbatimText != nullptr) {
                         result = verbatimText->text();
-                    }
+                        isVerbatim = true;
+                    } else
+                        qWarning() << "Cannot interpret ValueItem to one of its descendants";
                 }
             }
         }
@@ -602,8 +650,10 @@ QString PlainTextValue::text(const ValueItem &valueItem, ValueItemType &vit)
             /// place '\,' with a thin space
             result[j] = thinspace;
             ++i; ++j;
-        } else if (result[i] == tilde && (i < 1 || result[i - 1] != bs))  {
-            /// place '~' with a non-breaking space
+        } else if (!isVerbatim && result[i] == tilde && (i < 1 || result[i - 1] != bs))  {
+            /// replace '~' with a non-breaking space,
+            /// except if text was verbatim (e.g. a 'localfile' value
+            /// like '~/document.pdf' or 'document.pdf~')
             result[j] = nobreakspace;
             ++j;
         } else {
@@ -619,6 +669,7 @@ QString PlainTextValue::text(const ValueItem &valueItem, ValueItemType &vit)
     return result;
 }
 
+#ifdef HAVE_KF5
 PlainTextValue::PlainTextValue()
 {
     NotificationHub::registerNotificationListener(this, NotificationHub::EventConfigurationChanged);
@@ -633,10 +684,13 @@ void PlainTextValue::notificationEvent(int eventId)
 
 void PlainTextValue::readConfiguration()
 {
-    KSharedConfigPtr config(KSharedConfig::openConfig(QLatin1String("kbibtexrc")));
+    KSharedConfigPtr config(KSharedConfig::openConfig(QStringLiteral("kbibtexrc")));
     KConfigGroup configGroup(config, "General");
     personNameFormatting = configGroup.readEntry(Preferences::keyPersonNameFormatting, Preferences::defaultPersonNameFormatting);
 }
 
+PlainTextValue *PlainTextValue::notificationListener = nullptr;
 QString PlainTextValue::personNameFormatting;
-PlainTextValue *PlainTextValue::notificationListener = NULL;
+#else // HAVE_KF5
+const QString PlainTextValue::personNameFormatting = QStringLiteral("<%l><, %s><, %f>");
+#endif // HAVE_KF5
