@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,7 +12,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #ifndef KBIBTEX_GUI_ELEMENTWIDGETS_H
@@ -23,8 +23,8 @@
 #include <QLabel>
 #include <QWidget>
 
-#include <KUrl>
-#include <KIcon>
+#include <QUrl>
+#include <QIcon>
 
 #include "elementeditor.h"
 #include "entrylayout.h"
@@ -35,7 +35,7 @@ class QGridLayout;
 
 class KLineEdit;
 class KComboBox;
-class KPushButton;
+class QPushButton;
 
 class File;
 class Entry;
@@ -55,7 +55,7 @@ public:
     }
     virtual void showReqOptWidgets(bool, const QString &) = 0;
     virtual QString label() = 0;
-    virtual KIcon icon() = 0;
+    virtual QIcon icon() = 0;
     bool isModified() const;
     void setModified(bool);
 
@@ -93,26 +93,26 @@ private:
     int fieldInputCount, numCols;
     QGridLayout *gridLayout;
 
-    QSharedPointer<EntryTabLayout> etl;
+    const QSharedPointer<const EntryTabLayout> etl;
     QMap<QString, FieldInput *> bibtexKeyToWidget;
 
     void createGUI();
     void layoutGUI(bool forceVisible, const QString &entryType = QString());
 
 public:
-    EntryConfiguredWidget(QSharedPointer<EntryTabLayout> &entryTabLayout, QWidget *parent);
-    virtual ~EntryConfiguredWidget();
+    EntryConfiguredWidget(const QSharedPointer<const EntryTabLayout> &entryTabLayout, QWidget *parent);
+    ~EntryConfiguredWidget() override;
 
-    bool apply(QSharedPointer<Element> element) const;
-    bool reset(QSharedPointer<const Element> element);
-    void setReadOnly(bool isReadOnly);
-    void showReqOptWidgets(bool forceVisible, const QString &entryType);
-    QString label();
-    KIcon icon();
+    bool apply(QSharedPointer<Element> element) const override;
+    bool reset(QSharedPointer<const Element> element) override;
+    void setReadOnly(bool isReadOnly) override;
+    void showReqOptWidgets(bool forceVisible, const QString &entryType) override;
+    QString label() override;
+    QIcon icon() override;
 
-    virtual void setFile(const File *file);
+    void setFile(const File *file) override;
 
-    bool canEdit(const Element *element);
+    bool canEdit(const Element *element) override;
 };
 
 class ReferenceWidget : public ElementWidget
@@ -122,28 +122,34 @@ class ReferenceWidget : public ElementWidget
 private:
     KComboBox *entryType;
     KLineEdit *entryId;
-    KPushButton *buttonSuggestId;
+    QPushButton *buttonSuggestId;
 
     void createGUI();
 
 public:
     explicit ReferenceWidget(QWidget *parent);
 
-    bool apply(QSharedPointer<Element> element) const;
-    bool reset(QSharedPointer<const Element> element);
-    void setReadOnly(bool isReadOnly);
-    void showReqOptWidgets(bool, const QString &) {}
+    bool apply(QSharedPointer<Element> element) const override;
+    bool reset(QSharedPointer<const Element> element) override;
+    void setReadOnly(bool isReadOnly) override;
+    void showReqOptWidgets(bool, const QString &) override {}
     void setApplyElementInterface(ElementEditor::ApplyElementInterface *applyElement) {
         m_applyElement = applyElement;
     }
 
     void setOriginalElement(const QSharedPointer<Element> &orig);
-    bool isDuplicateId() const;
 
-    QString label();
-    KIcon icon();
+    /**
+     * Return the current value of the entry id/macro key editing widget.
+     *
+     * @return Current value of entry id/macro key if any, otherwise empty string
+     */
+    QString currentId() const;
 
-    bool canEdit(const Element *element);
+    QString label() override;
+    QIcon icon() override;
+
+    bool canEdit(const Element *element) override;
 
 public slots:
     void setEntryIdByDefault();
@@ -172,17 +178,17 @@ private:
 public:
     explicit FilesWidget(QWidget *parent);
 
-    bool apply(QSharedPointer<Element> element) const;
-    bool reset(QSharedPointer<const Element> element);
-    void setReadOnly(bool isReadOnly);
-    void showReqOptWidgets(bool, const QString &) {}
+    bool apply(QSharedPointer<Element> element) const override;
+    bool reset(QSharedPointer<const Element> element) override;
+    void setReadOnly(bool isReadOnly) override;
+    void showReqOptWidgets(bool, const QString &) override {}
 
-    QString label();
-    KIcon icon();
+    QString label() override;
+    QIcon icon() override;
 
-    virtual void setFile(const File *file);
+    void setFile(const File *file) override;
 
-    bool canEdit(const Element *element);
+    bool canEdit(const Element *element) override;
 
 private:
     static const QStringList keyStart;
@@ -196,10 +202,10 @@ private:
     KLineEdit *fieldName;
     FieldInput *fieldContent;
     QTreeWidget *otherFieldsList;
-    KPushButton *buttonDelete;
-    KPushButton *buttonOpen;
-    KPushButton *buttonAddApply;
-    KUrl currentUrl;
+    QPushButton *buttonDelete;
+    QPushButton *buttonOpen;
+    QPushButton *buttonAddApply;
+    QUrl currentUrl;
     const QStringList blackListed;
     QSharedPointer<Entry> internalEntry;
     QStringList deletedKeys, modifiedKeys;
@@ -210,16 +216,16 @@ private:
 
 public:
     OtherFieldsWidget(const QStringList &blacklistedFields, QWidget *parent);
-    ~OtherFieldsWidget();
+    ~OtherFieldsWidget() override;
 
-    bool apply(QSharedPointer<Element> element) const;
-    bool reset(QSharedPointer<const Element> element);
-    void setReadOnly(bool isReadOnly);
-    void showReqOptWidgets(bool, const QString &) {}
-    QString label();
-    KIcon icon();
+    bool apply(QSharedPointer<Element> element) const override;
+    bool reset(QSharedPointer<const Element> element) override;
+    void setReadOnly(bool isReadOnly) override;
+    void showReqOptWidgets(bool, const QString &) override {}
+    QString label() override;
+    QIcon icon() override;
 
-    bool canEdit(const Element *element);
+    bool canEdit(const Element *element) override;
 
 private slots:
     void listElementExecuted(QTreeWidgetItem *item, int column);
@@ -241,16 +247,16 @@ private:
 
 public:
     explicit MacroWidget(QWidget *parent);
-    ~MacroWidget();
+    ~MacroWidget() override;
 
-    bool apply(QSharedPointer<Element> element) const;
-    bool reset(QSharedPointer<const Element> element);
-    void setReadOnly(bool isReadOnly);
-    void showReqOptWidgets(bool, const QString &) {}
-    QString label();
-    KIcon icon();
+    bool apply(QSharedPointer<Element> element) const override;
+    bool reset(QSharedPointer<const Element> element) override;
+    void setReadOnly(bool isReadOnly) override;
+    void showReqOptWidgets(bool, const QString &) override {}
+    QString label() override;
+    QIcon icon() override;
 
-    bool canEdit(const Element *element);
+    bool canEdit(const Element *element) override;
 };
 
 class PreambleWidget : public ElementWidget
@@ -265,14 +271,14 @@ private:
 public:
     explicit PreambleWidget(QWidget *parent);
 
-    bool apply(QSharedPointer<Element> element) const;
-    bool reset(QSharedPointer<const Element> element);
-    void setReadOnly(bool isReadOnly);
-    void showReqOptWidgets(bool, const QString &) {}
-    QString label();
-    KIcon icon();
+    bool apply(QSharedPointer<Element> element) const override;
+    bool reset(QSharedPointer<const Element> element) override;
+    void setReadOnly(bool isReadOnly) override;
+    void showReqOptWidgets(bool, const QString &) override {}
+    QString label() override;
+    QIcon icon() override;
 
-    bool canEdit(const Element *element);
+    bool canEdit(const Element *element) override;
 };
 
 class SourceWidget : public ElementWidget
@@ -288,22 +294,23 @@ private:
 
 public:
     explicit SourceWidget(QWidget *parent);
-    ~SourceWidget();
+    ~SourceWidget() override;
 
-    bool apply(QSharedPointer<Element> element) const;
-    bool reset(QSharedPointer<const Element> element);
-    void setReadOnly(bool isReadOnly);
-    void showReqOptWidgets(bool, const QString &) {}
-    QString label();
-    KIcon icon();
+    bool apply(QSharedPointer<Element> element) const override;
+    bool reset(QSharedPointer<const Element> element) override;
+    void setReadOnly(bool isReadOnly) override;
+    void showReqOptWidgets(bool, const QString &) override {}
+    QString label() override;
+    QIcon icon() override;
 
-    bool canEdit(const Element *element);
+    bool canEdit(const Element *element) override;
 
 private slots:
     void reset();
 
 private:
-    KPushButton *m_buttonRestore;
+    class Private;
+    Private *const d;
 };
 
 #endif // KBIBTEX_GUI_ELEMENTWIDGETS_H
