@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,18 +12,18 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "settingsfileexporterpdfpswidget.h"
 
 #include <QFormLayout>
-#include <QLineEdit>
+#include <KLineEdit>
 
 #include <KSharedConfig>
 #include <KConfigGroup>
 #include <KComboBox>
-#include <KLocale>
+#include <KLocalizedString>
 
 #include "guihelper.h"
 #include "fileexportertoolchain.h"
@@ -45,10 +45,10 @@ private:
 public:
 
     SettingsFileExporterPDFPSWidgetPrivate(SettingsFileExporterPDFPSWidget *parent)
-            : p(parent), config(KSharedConfig::openConfig(QLatin1String("kbibtexrc"))), configGroupName(QLatin1String("FileExporterPDFPS")), configGroupNameGeneral(QLatin1String("General")) {
-        paperSizeLabelToName.insert(i18n("A4"), QLatin1String("a4"));
-        paperSizeLabelToName.insert(i18n("Letter"), QLatin1String("letter"));
-        paperSizeLabelToName.insert(i18n("Legal"), QLatin1String("legal"));
+            : p(parent), config(KSharedConfig::openConfig(QStringLiteral("kbibtexrc"))), configGroupName(QStringLiteral("FileExporterPDFPS")), configGroupNameGeneral(QStringLiteral("General")) {
+        paperSizeLabelToName.insert(i18n("A4"), QStringLiteral("a4"));
+        paperSizeLabelToName.insert(i18n("Letter"), QStringLiteral("letter"));
+        paperSizeLabelToName.insert(i18n("Legal"), QStringLiteral("legal"));
 
         setupGUI();
     }
@@ -80,7 +80,7 @@ public:
     }
 
     void resetToDefaults() {
-        int row = GUIHelper::selectValue(comboBoxPaperSize->model(), paperSizeLabelToName[FileExporter::defaultPaperSize]);
+        int row = GUIHelper::selectValue(comboBoxPaperSize->model(), FileExporter::defaultPaperSize);
         comboBoxPaperSize->setCurrentIndex(row);
         row = GUIHelper::selectValue(comboBoxBabelLanguage->model(), FileExporterToolchain::defaultBabelLanguage);
         comboBoxBabelLanguage->setCurrentIndex(row);
@@ -92,30 +92,31 @@ public:
         QFormLayout *layout = new QFormLayout(p);
 
         comboBoxPaperSize = new KComboBox(false, p);
-        comboBoxPaperSize->setObjectName("comboBoxPaperSize");
+        comboBoxPaperSize->setObjectName(QStringLiteral("comboBoxPaperSize"));
         layout->addRow(i18n("Paper Size:"), comboBoxPaperSize);
         QStringList paperSizeLabelToNameKeys = paperSizeLabelToName.keys();
         paperSizeLabelToNameKeys.sort();
-        foreach (const QString &labelText, paperSizeLabelToNameKeys) {
+        for (const QString &labelText : const_cast<const QStringList &>(paperSizeLabelToNameKeys)) {
             comboBoxPaperSize->addItem(labelText, paperSizeLabelToName[labelText]);
         }
-        connect(comboBoxPaperSize, SIGNAL(currentIndexChanged(int)), p, SIGNAL(changed()));
+        connect(comboBoxPaperSize, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged), p, &SettingsAbstractWidget::changed);
 
         comboBoxBabelLanguage = new KComboBox(true, p);
-        comboBoxBabelLanguage->setObjectName("comboBoxBabelLanguage");
+        comboBoxBabelLanguage->setObjectName(QStringLiteral("comboBoxBabelLanguage"));
         layout->addRow(i18n("Language for 'babel':"), comboBoxBabelLanguage);
-        comboBoxBabelLanguage->addItem(QLatin1String("english"));
-        comboBoxBabelLanguage->addItem(QLatin1String("ngerman"));
-        comboBoxBabelLanguage->addItem(QLatin1String("swedish"));
-        connect(comboBoxBabelLanguage->lineEdit(), SIGNAL(textChanged(QString)), p, SIGNAL(changed()));
+        comboBoxBabelLanguage->addItem(QStringLiteral("english"));
+        comboBoxBabelLanguage->addItem(QStringLiteral("ngerman"));
+        comboBoxBabelLanguage->addItem(QStringLiteral("swedish"));
+        connect(comboBoxBabelLanguage->lineEdit(), &QLineEdit::textChanged, p, &SettingsFileExporterPDFPSWidget::changed);
 
         comboBoxBibliographyStyle = new KComboBox(true, p);
-        comboBoxBibliographyStyle->setObjectName("comboBoxBibliographyStyle");
+        comboBoxBibliographyStyle->setObjectName(QStringLiteral("comboBoxBibliographyStyle"));
         layout->addRow(i18n("Bibliography style:"), comboBoxBibliographyStyle);
-        foreach (const QString &style, QStringList() << QLatin1String("abbrv") << QLatin1String("alpha") << QLatin1String("plain") << QLatin1String("agsm") << QLatin1String("dcu") << QLatin1String("jmr") << QLatin1String("jphysicsB") << QLatin1String("kluwer") << QLatin1String("nederlands")) {
+        static const QStringList styles = QStringList() << QString(QStringLiteral("abbrv")) << QString(QStringLiteral("alpha")) << QString(QStringLiteral("plain")) << QString(QStringLiteral("agsm")) << QString(QStringLiteral("dcu")) << QString(QStringLiteral("jmr")) << QString(QStringLiteral("jphysicsB")) << QString(QStringLiteral("kluwer")) << QString(QStringLiteral("nederlands"));
+        for (const QString &style : styles) {
             comboBoxBibliographyStyle->addItem(style);
         }
-        connect(comboBoxBibliographyStyle->lineEdit(), SIGNAL(textChanged(QString)), p, SIGNAL(changed()));
+        connect(comboBoxBibliographyStyle->lineEdit(), &QLineEdit::textChanged, p, &SettingsFileExporterPDFPSWidget::changed);
     }
 };
 
@@ -135,9 +136,9 @@ QString SettingsFileExporterPDFPSWidget::label() const
     return i18n("PDF & Postscript");
 }
 
-KIcon SettingsFileExporterPDFPSWidget::icon() const
+QIcon SettingsFileExporterPDFPSWidget::icon() const
 {
-    return KIcon("application-pdf");
+    return QIcon::fromTheme(QStringLiteral("application-pdf"));
 }
 
 void SettingsFileExporterPDFPSWidget::loadState()
