@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,7 +12,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "settingsuserinterfacewidget.h"
@@ -22,13 +22,13 @@
 #include <QBoxLayout>
 
 #include <KComboBox>
-#include <KLocale>
+#include <KLocalizedString>
 #include <KSharedConfig>
 #include <KConfigGroup>
 
 #include "preferences.h"
 #include "elementwidgets.h"
-#include "filemodel.h"
+#include "models/filemodel.h"
 
 class SettingsUserInterfaceWidget::SettingsUserInterfaceWidgetPrivate
 {
@@ -45,7 +45,7 @@ private:
 
 public:
     SettingsUserInterfaceWidgetPrivate(SettingsUserInterfaceWidget *parent)
-            : p(parent), config(KSharedConfig::openConfig(QLatin1String("kbibtexrc"))) {
+            : p(parent), config(KSharedConfig::openConfig(QStringLiteral("kbibtexrc"))) {
         setupGUI();
     }
 
@@ -54,16 +54,9 @@ public:
         checkBoxShowComments->setChecked(configGroup.readEntry(FileModel::keyShowComments, FileModel::defaultShowComments));
         checkBoxShowMacros->setChecked(configGroup.readEntry(FileModel::keyShowMacros, FileModel::defaultShowMacros));
 
-        const QStringList styles = configGroup.readEntry("BibTeXStyles", QStringList());
-        foreach (const QString &style, styles) {
-            QStringList item = style.split(QLatin1String("|"));
-            QString itemLabel = item.at(0);
-            item.removeFirst();
-            comboBoxBibliographySystem->addItem(itemLabel, item);
-        }
-        int styleIndex = comboBoxBibliographySystem->findData(configGroup.readEntry("CurrentStyle", QString()));
+        int styleIndex = comboBoxBibliographySystem->findData(configGroup.readEntry("CurrentStyle", QString(QStringLiteral("bibtex"))));
         if (styleIndex < 0) styleIndex = 0;
-        comboBoxBibliographySystem->setCurrentIndex(styleIndex);
+        if (styleIndex < comboBoxBibliographySystem->count()) comboBoxBibliographySystem->setCurrentIndex(styleIndex);
 
         comboBoxElementDoubleClickAction->setCurrentIndex(configGroup.readEntry(Preferences::keyElementDoubleClickAction, (int)Preferences::defaultElementDoubleClickAction));
     }
@@ -89,27 +82,29 @@ public:
 
         checkBoxShowComments = new QCheckBox(p);
         layout->addRow(i18n("Show Comments:"), checkBoxShowComments);
-        connect(checkBoxShowComments, SIGNAL(toggled(bool)), p, SIGNAL(changed()));
+        connect(checkBoxShowComments, &QCheckBox::toggled, p, &SettingsUserInterfaceWidget::changed);
 
         checkBoxShowMacros = new QCheckBox(p);
         layout->addRow(i18n("Show Macros:"), checkBoxShowMacros);
-        connect(checkBoxShowMacros, SIGNAL(toggled(bool)), p, SIGNAL(changed()));
+        connect(checkBoxShowMacros, &QCheckBox::toggled, p, &SettingsUserInterfaceWidget::changed);
 
         comboBoxBibliographySystem = new KComboBox(p);
-        comboBoxBibliographySystem->setObjectName("comboBoxBibtexStyle");
+        comboBoxBibliographySystem->setObjectName(QStringLiteral("comboBoxBibtexStyle"));
+        comboBoxBibliographySystem->addItem(i18n("BibTeX"), QStringLiteral("bibtex"));
+        comboBoxBibliographySystem->addItem(i18n("BibLaTeX"), QStringLiteral("biblatex"));
         layout->addRow(i18n("Bibliography System:"), comboBoxBibliographySystem);
-        connect(comboBoxBibliographySystem, SIGNAL(currentIndexChanged(int)), p, SIGNAL(changed()));
+        connect(comboBoxBibliographySystem, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged), p, &SettingsUserInterfaceWidget::changed);
 
         comboBoxElementDoubleClickAction = new KComboBox(p);
-        comboBoxElementDoubleClickAction->setObjectName("comboBoxElementDoubleClickAction");
+        comboBoxElementDoubleClickAction->setObjectName(QStringLiteral("comboBoxElementDoubleClickAction"));
         comboBoxElementDoubleClickAction->addItem(i18n("Open Editor")); ///< ActionOpenEditor = 0
         comboBoxElementDoubleClickAction->addItem(i18n("View Document")); ///< ActionViewDocument = 1
         layout->addRow(i18n("When double-clicking an element:"), comboBoxElementDoubleClickAction);
-        connect(comboBoxElementDoubleClickAction, SIGNAL(currentIndexChanged(int)), p, SIGNAL(changed()));
+        connect(comboBoxElementDoubleClickAction, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged), p, &SettingsUserInterfaceWidget::changed);
     }
 };
 
-const QString SettingsUserInterfaceWidget::SettingsUserInterfaceWidgetPrivate::configGroupName = QLatin1String("User Interface");
+const QString SettingsUserInterfaceWidget::SettingsUserInterfaceWidgetPrivate::configGroupName = QStringLiteral("User Interface");
 
 
 SettingsUserInterfaceWidget::SettingsUserInterfaceWidget(QWidget *parent)
@@ -123,9 +118,9 @@ QString SettingsUserInterfaceWidget::label() const
     return i18n("User Interface");
 }
 
-KIcon SettingsUserInterfaceWidget::icon() const
+QIcon SettingsUserInterfaceWidget::icon() const
 {
-    return KIcon("user-identity");
+    return QIcon::fromTheme(QStringLiteral("user-identity"));
 }
 
 SettingsUserInterfaceWidget::~SettingsUserInterfaceWidget()
