@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,7 +12,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "fileexporterbibtex.h"
@@ -22,15 +22,11 @@
 #include <QTextCodec>
 #include <QTextStream>
 #include <QStringList>
-#include <QLabel>
-#include <QLineEdit>
-#include <QFormLayout>
-#include <QAbstractItemModel>
 
-#include <KDebug>
-#include <KDialog>
-#include <KLocale>
-#include <KComboBox>
+#ifdef HAVE_KF5
+#include <KSharedConfig>
+#include <KConfigGroup>
+#endif // HAVE_KF5
 
 #include "preferences.h"
 #include "file.h"
@@ -41,14 +37,12 @@
 #include "value.h"
 #include "comment.h"
 #include "encoderlatex.h"
-#include "encoderutf8.h"
 #include "bibtexentries.h"
 #include "bibtexfields.h"
-#include "iconvlatex.h"
+#include "textencoder.h"
+#include "logging_io.h"
 
-#define encodercheck(encoder, text) ((encoder)?(encoder)->encode((text)):(text))
-
-FileExporterBibTeX *FileExporterBibTeX::staticFileExporterBibTeX = NULL;
+FileExporterBibTeX *FileExporterBibTeX::staticFileExporterBibTeX = nullptr;
 
 class FileExporterBibTeX::FileExporterBibTeXPrivate
 {
@@ -65,27 +59,31 @@ public:
     QString personNameFormatting;
     QString listSeparator;
     bool cancelFlag;
-    IConvLaTeX *iconvLaTeX;
+    QTextCodec *destinationCodec;
+#ifdef HAVE_KF5
     KSharedConfigPtr config;
     const QString configGroupName, configGroupNameGeneral;
+#endif // HAVE_KF5
 
     FileExporterBibTeXPrivate(FileExporterBibTeX *parent)
-            : p(parent), keywordCasing(KBibTeX::cLowerCase), quoteComment(Preferences::qcNone), protectCasing(Qt::PartiallyChecked), cancelFlag(false), iconvLaTeX(NULL), config(KSharedConfig::openConfig(QLatin1String("kbibtexrc"))), configGroupName("FileExporterBibTeX"), configGroupNameGeneral("General") {
+            : p(parent), keywordCasing(KBibTeX::cLowerCase), quoteComment(Preferences::qcNone), protectCasing(Qt::PartiallyChecked), cancelFlag(false), destinationCodec(nullptr), config(KSharedConfig::openConfig(QStringLiteral("kbibtexrc"))), configGroupName(QStringLiteral("FileExporterBibTeX")), configGroupNameGeneral(QStringLiteral("General")) {
         // nothing
     }
 
-    ~FileExporterBibTeXPrivate() {
-        delete iconvLaTeX;
-    }
-
     void loadState() {
+#ifdef HAVE_KF5
         KConfigGroup configGroup(config, configGroupName);
         encoding = configGroup.readEntry(Preferences::keyEncoding, Preferences::defaultEncoding);
         QString stringDelimiter = configGroup.readEntry(Preferences::keyStringDelimiter, Preferences::defaultStringDelimiter);
         if (stringDelimiter.length() != 2)
             stringDelimiter = Preferences::defaultStringDelimiter;
+#else // HAVE_KF5
+        encoding = QStringLiteral("LaTeX");
+        const QString stringDelimiter = QStringLiteral("{}");
+#endif // HAVE_KF5
         stringOpenDelimiter = stringDelimiter[0];
         stringCloseDelimiter = stringDelimiter[1];
+#ifdef HAVE_KF5
         keywordCasing = (KBibTeX::Casing)configGroup.readEntry(Preferences::keyKeywordCasing, (int)Preferences::defaultKeywordCasing);
         quoteComment = (Preferences::QuoteComment)configGroup.readEntry(Preferences::keyQuoteComment, (int)Preferences::defaultQuoteComment);
         protectCasing = (Qt::CheckState)configGroup.readEntry(Preferences::keyProtectCasing, (int)Preferences::defaultProtectCasing);
@@ -97,10 +95,17 @@ public:
             KConfigGroup configGroupGeneral(config, configGroupNameGeneral);
             personNameFormatting = configGroupGeneral.readEntry(Preferences::keyPersonNameFormatting, Preferences::defaultPersonNameFormatting);
         }
+#else // HAVE_KF5
+        keywordCasing = KBibTeX::cLowerCase;
+        quoteComment = qcNone;
+        protectCasing = Qt::PartiallyChecked;
+        personNameFormatting = QStringLiteral("<%l><, %s><, %f>");
+        listSeparator = QStringLiteral("; ");
+#endif // HAVE_KF5
     }
 
     void loadStateFromFile(const File *bibtexfile) {
-        if (bibtexfile == NULL) return;
+        if (bibtexfile == nullptr) return;
 
         if (bibtexfile->hasProperty(File::Encoding))
             encoding = bibtexfile->property(File::Encoding).toString();
@@ -110,7 +115,11 @@ public:
         if (bibtexfile->hasProperty(File::StringDelimiter)) {
             QString stringDelimiter = bibtexfile->property(File::StringDelimiter).toString();
             if (stringDelimiter.length() != 2)
+#ifdef HAVE_KF5
                 stringDelimiter = Preferences::defaultStringDelimiter;
+#else // HAVE_KF5
+                stringDelimiter = QStringLiteral("{}");
+#endif // HAVE_KF5
             stringOpenDelimiter = stringDelimiter[0];
             stringCloseDelimiter = stringDelimiter[1];
         }
@@ -133,13 +142,13 @@ public:
     bool writeEntry(QIODevice *iodevice, const Entry &entry) {
         const BibTeXEntries *be = BibTeXEntries::self();
         const BibTeXFields *bf = BibTeXFields::self();
-        EncoderLaTeX *laTeXEncoder = EncoderLaTeX::instance();
+        const EncoderLaTeX &laTeXEncoder = EncoderLaTeX::instance();
 
         /// write start of a entry (entry type and id) in plain ASCII
         iodevice->putChar('@');
         iodevice->write(be->format(entry.type(), keywordCasing).toLatin1().data());
         iodevice->putChar('{');
-        iodevice->write(laTeXEncoder->convertToPlainAscii(entry.id()).toLatin1());
+        iodevice->write(laTeXEncoder.convertToPlainAscii(entry.id()).toLatin1());
 
         for (Entry::ConstIterator it = entry.constBegin(); it != entry.constEnd(); ++it) {
             const QString key = it.key();
@@ -149,7 +158,7 @@ public:
             QString text = p->internalValueToBibTeX(value, key, leUTF8);
             if (text.isEmpty()) {
                 /// ignore empty key-value pairs
-                kWarning() << "Value for field " << key << " is empty" << endl;
+                qCWarning(LOG_KBIBTEX_IO) << "Value for field " << key << " is empty" << endl;
                 continue;
             }
 
@@ -165,11 +174,11 @@ public:
             iodevice->putChar(',');
             iodevice->putChar('\n');
             iodevice->putChar('\t');
-            iodevice->write(laTeXEncoder->convertToPlainAscii(bf->format(key, keywordCasing)).toLatin1());
+            iodevice->write(laTeXEncoder.convertToPlainAscii(bf->format(key, keywordCasing)).toLatin1());
             iodevice->putChar(' ');
             iodevice->putChar('=');
             iodevice->putChar(' ');
-            iodevice->write(iconvLaTeX->encode(text));
+            iodevice->write(TextEncoder::encode(text, destinationCodec));
         }
         iodevice->putChar('\n');
         iodevice->putChar('}');
@@ -189,13 +198,13 @@ public:
             removeProtectiveCasing(text);
 
         iodevice->putChar('@');
-        iodevice->write(be->format(QLatin1String("String"), keywordCasing).toLatin1().data());
+        iodevice->write(be->format(QStringLiteral("String"), keywordCasing).toLatin1().data());
         iodevice->putChar('{');
-        iodevice->write(iconvLaTeX->encode(macro.key()));
+        iodevice->write(TextEncoder::encode(macro.key(), destinationCodec));
         iodevice->putChar(' ');
         iodevice->putChar('=');
         iodevice->putChar(' ');
-        iodevice->write(iconvLaTeX->encode(text));
+        iodevice->write(TextEncoder::encode(text, destinationCodec));
         iodevice->putChar('}');
         iodevice->putChar('\n');
         iodevice->putChar('\n');
@@ -210,16 +219,16 @@ public:
 
         if (comment.useCommand() || quoteComment == Preferences::qcCommand) {
             iodevice->putChar('@');
-            iodevice->write(be->format(QLatin1String("Comment"), keywordCasing).toLatin1().data());
+            iodevice->write(be->format(QStringLiteral("Comment"), keywordCasing).toLatin1().data());
             iodevice->putChar('{');
-            iodevice->write(iconvLaTeX->encode(text));
+            iodevice->write(TextEncoder::encode(text, destinationCodec));
             iodevice->putChar('}');
             iodevice->putChar('\n');
             iodevice->putChar('\n');
         } else if (quoteComment == Preferences::qcPercentSign) {
             QStringList commentLines = text.split('\n', QString::SkipEmptyParts);
             for (QStringList::Iterator it = commentLines.begin(); it != commentLines.end(); ++it) {
-                const QByteArray line = iconvLaTeX->encode(*it);
+                const QByteArray line = TextEncoder::encode(*it, destinationCodec);
                 if (line.length() == 0 || line[0] != QLatin1Char('%')) {
                     /// Guarantee that every line starts with
                     /// a percent sign
@@ -230,7 +239,7 @@ public:
             }
             iodevice->putChar('\n');
         } else {
-            iodevice->write(iconvLaTeX->encode(text));
+            iodevice->write(TextEncoder::encode(text, destinationCodec));
             iodevice->putChar('\n');
             iodevice->putChar('\n');
         }
@@ -242,11 +251,11 @@ public:
         const BibTeXEntries *be = BibTeXEntries::self();
 
         iodevice->putChar('@');
-        iodevice->write(be->format(QLatin1String("Preamble"), keywordCasing).toLatin1().data());
+        iodevice->write(be->format(QStringLiteral("Preamble"), keywordCasing).toLatin1().data());
         iodevice->putChar('{');
         /// Remember: strings from preamble do not get encoded,
         /// may contain raw LaTeX commands and code
-        iodevice->write(iconvLaTeX->encode(p->internalValueToBibTeX(preamble.value(), QString(), leRaw)));
+        iodevice->write(TextEncoder::encode(p->internalValueToBibTeX(preamble.value(), QString(), leRaw), destinationCodec));
         iodevice->putChar('}');
         iodevice->putChar('\n');
         iodevice->putChar('\n');
@@ -255,7 +264,11 @@ public:
     }
 
     QString addProtectiveCasing(QString &text) {
-        if (text.length() < 2 && (text[0] != QLatin1Char('"') || text[text.length() - 1] != QLatin1Char('"')) && (text[0] != QLatin1Char('{') || text[text.length() - 1] != QLatin1Char('}'))) {
+        /// Check if either
+        ///  - text is too short (less than two characters)  or
+        ///  - text neither starts/stops with double quotation marks
+        ///    nor starts with { and stops with }
+        if (text.length() < 2 || ((text[0] != QLatin1Char('"') || text[text.length() - 1] != QLatin1Char('"')) && (text[0] != QLatin1Char('{') || text[text.length() - 1] != QLatin1Char('}')))) {
             /// Nothing to protect, as this is no text string
             return text;
         }
@@ -263,11 +276,10 @@ public:
         bool addBrackets = true;
 
         if (text[1] == QLatin1Char('{') && text[text.length() - 2] == QLatin1Char('}')) {
-            /// If the given text looks like this:  {{...}}
+            /// If the given text looks like this:  {{...}}  or  "{...}"
             /// still check that it is not like this: {{..}..{..}}
             addBrackets = false;
-            int count = 0;
-            for (int i = text.length() - 2; !addBrackets && i > 1; --i) {
+            for (int i = text.length() - 2, count = 0; !addBrackets && i > 1; --i) {
                 if (text[i] == QLatin1Char('{')) ++count;
                 else if (text[i] == QLatin1Char('}')) --count;
                 if (count == 0) addBrackets = true;
@@ -275,13 +287,17 @@ public:
         }
 
         if (addBrackets)
-            text.insert(1, QLatin1String("{")).insert(text.length(),  QLatin1String("}"));
+            text.insert(1, QStringLiteral("{")).insert(text.length() - 1,  QStringLiteral("}"));
 
         return text;
     }
 
     QString removeProtectiveCasing(QString &text) {
-        if (text.length() < 2 && (text[0] != QLatin1Char('"') || text[text.length() - 1] != QLatin1Char('"')) && (text[0] != QLatin1Char('{') || text[text.length() - 1] != QLatin1Char('}'))) {
+        /// Check if either
+        ///  - text is too short (less than two characters)  or
+        ///  - text neither starts/stops with double quotation marks
+        ///    nor starts with { and stops with }
+        if (text.length() < 2 || ((text[0] != QLatin1Char('"') || text[text.length() - 1] != QLatin1Char('"')) && (text[0] != QLatin1Char('{') || text[text.length() - 1] != QLatin1Char('}')))) {
             /// Nothing to protect, as this is no text string
             return text;
         }
@@ -290,9 +306,10 @@ public:
             /// Nothing to remove
             return text;
 
+        /// If the given text looks like this:  {{...}}  or  "{...}"
+        /// still check that it is not like this: {{..}..{..}}
         bool removeBrackets = true;
-        int count = 0;
-        for (int i = text.length() - 2; removeBrackets && i > 1; --i) {
+        for (int i = text.length() - 2, count = 0; removeBrackets && i > 1; --i) {
             if (text[i] == QLatin1Char('{')) ++count;
             else if (text[i] == QLatin1Char('}')) --count;
             if (count == 0) removeBrackets = false;
@@ -308,23 +325,22 @@ public:
         int p = -1;
         while ((p = text.indexOf(QLatin1Char('"'), p + 1)) > 0)
             if (p == 0 || text[p - 1] != QLatin1Char('\\')) {
-                text.insert(p + 1, QLatin1String("}")).insert(p, QLatin1String("{"));
+                text.insert(p + 1, QStringLiteral("}")).insert(p, QStringLiteral("{"));
                 ++p;
             }
         return text;
     }
 
     void applyEncoding(QString &encoding) {
-        encoding = encoding.isEmpty() ? QLatin1String("latex") : encoding.toLower();
-        delete iconvLaTeX;
-        iconvLaTeX = new IConvLaTeX(encoding == QLatin1String("latex") ? QLatin1String("us-ascii") : encoding);
+        encoding = encoding.isEmpty() ? QStringLiteral("latex") : encoding.toLower();
+        destinationCodec = QTextCodec::codecForName(encoding == QStringLiteral("latex") ? "us-ascii" : encoding.toLatin1());
     }
 
     bool requiresPersonQuoting(const QString &text, bool isLastName) {
         if (isLastName && !text.contains(QChar(' ')))
             /** Last name contains NO spaces, no quoting necessary */
             return false;
-        else if (!isLastName && !text.contains(" and "))
+        else if (!isLastName && !text.contains(QStringLiteral(" and ")))
             /** First name contains no " and " no quoting necessary */
             return false;
         else if (isLastName && !text.isEmpty() && text[0].isLower())
@@ -349,8 +365,8 @@ public:
 };
 
 
-FileExporterBibTeX::FileExporterBibTeX()
-        : FileExporter(), d(new FileExporterBibTeXPrivate(this))
+FileExporterBibTeX::FileExporterBibTeX(QObject *parent)
+        : FileExporter(parent), d(new FileExporterBibTeXPrivate(this))
 {
     // nothing
 }
@@ -368,7 +384,7 @@ void FileExporterBibTeX::setEncoding(const QString &encoding)
 bool FileExporterBibTeX::save(QIODevice *iodevice, const File *bibtexfile, QStringList * /*errorLog*/)
 {
     if (!iodevice->isWritable() && !iodevice->open(QIODevice::WriteOnly)) {
-        kDebug() << "Output device not writable";
+        qCWarning(LOG_KBIBTEX_IO) << "Output device not writable";
         return false;
     }
 
@@ -379,8 +395,8 @@ bool FileExporterBibTeX::save(QIODevice *iodevice, const File *bibtexfile, QStri
     d->loadState();
     d->loadStateFromFile(bibtexfile);
 
-    if (d->encoding != QLatin1String("latex")) {
-        Comment encodingComment(QLatin1String("x-kbibtex-encoding=") + d->encoding, true);
+    if (d->encoding != QStringLiteral("latex")) {
+        Comment encodingComment(QStringLiteral("x-kbibtex-encoding=") + d->encoding, true);
         result &= d->writeComment(iodevice, encodingComment);
     }
 
@@ -427,7 +443,7 @@ bool FileExporterBibTeX::save(QIODevice *iodevice, const File *bibtexfile, QStri
             emit progress(++currentPos, totalElements);
         } else {
             QSharedPointer<const Comment> comment = element.dynamicCast<const Comment>();
-            if (!comment.isNull() && !comment->text().startsWith(QLatin1String("x-kbibtex-"))) {
+            if (!comment.isNull() && !comment->text().startsWith(QStringLiteral("x-kbibtex-"))) {
                 result &= d->writeComment(iodevice, *comment);
                 emit progress(++currentPos, totalElements);
             } else if (!allPreamblesAndMacrosProcessed) {
@@ -463,7 +479,7 @@ bool FileExporterBibTeX::save(QIODevice *iodevice, const File *bibtexfile, QStri
 bool FileExporterBibTeX::save(QIODevice *iodevice, const QSharedPointer<const Element> element, const File *bibtexfile, QStringList * /*errorLog*/)
 {
     if (!iodevice->isWritable() && !iodevice->open(QIODevice::WriteOnly)) {
-        kDebug() << "Output device not writable";
+        qCWarning(LOG_KBIBTEX_IO) << "Output device not writable";
         return false;
     }
 
@@ -506,25 +522,31 @@ void FileExporterBibTeX::cancel()
 
 QString FileExporterBibTeX::valueToBibTeX(const Value &value, const QString &key, UseLaTeXEncoding useLaTeXEncoding)
 {
-    if (staticFileExporterBibTeX == NULL) {
-        staticFileExporterBibTeX = new FileExporterBibTeX();
+    if (staticFileExporterBibTeX == nullptr) {
+        staticFileExporterBibTeX = new FileExporterBibTeX(nullptr);
         staticFileExporterBibTeX->d->loadState();
     }
     return staticFileExporterBibTeX->internalValueToBibTeX(value, key, useLaTeXEncoding);
 }
 
+QString FileExporterBibTeX::applyEncoder(const QString &input, UseLaTeXEncoding useLaTeXEncoding) const {
+    switch (useLaTeXEncoding) {
+    case leLaTeX: return EncoderLaTeX::instance().encode(input, Encoder::TargetEncodingASCII);
+    case leUTF8: return EncoderLaTeX::instance().encode(input, Encoder::TargetEncodingUTF8);
+    default: return input;
+    }
+}
+
 QString FileExporterBibTeX::internalValueToBibTeX(const Value &value, const QString &key, UseLaTeXEncoding useLaTeXEncoding)
 {
     if (value.isEmpty())
         return QString();
 
-    EncoderLaTeX *encoder = useLaTeXEncoding == leLaTeX ? EncoderLaTeX::instance() : (useLaTeXEncoding == leUTF8 ? EncoderUTF8::instance() : NULL);
-
     QString result;
     bool isOpen = false;
     QSharedPointer<const ValueItem> prev;
-    for (Value::ConstIterator it = value.constBegin(); it != value.constEnd(); ++it) {
-        QSharedPointer<const MacroKey> macroKey = (*it).dynamicCast<const MacroKey>();
+    for (const auto &valueItem : value) {
+        QSharedPointer<const MacroKey> macroKey = valueItem.dynamicCast<const MacroKey>();
         if (!macroKey.isNull()) {
             if (isOpen) result.append(d->stringCloseDelimiter);
             isOpen = false;
@@ -532,9 +554,9 @@ QString FileExporterBibTeX::internalValueToBibTeX(const Value &value, const QStr
             result.append(macroKey->text());
             prev = macroKey;
         } else {
-            QSharedPointer<const PlainText> plainText = (*it).dynamicCast<const PlainText>();
+            QSharedPointer<const PlainText> plainText = valueItem.dynamicCast<const PlainText>();
             if (!plainText.isNull()) {
-                QString textBody = encodercheck(encoder, plainText->text());
+                QString textBody = applyEncoder(plainText->text(), useLaTeXEncoding);
                 if (!isOpen) {
                     if (!result.isEmpty()) result.append(" # ");
                     result.append(d->stringOpenDelimiter);
@@ -553,17 +575,18 @@ QString FileExporterBibTeX::internalValueToBibTeX(const Value &value, const QStr
                 result.append(textBody);
                 prev = plainText;
             } else {
-                QSharedPointer<const VerbatimText> verbatimText = (*it).dynamicCast<const VerbatimText>();
+                QSharedPointer<const VerbatimText> verbatimText = valueItem.dynamicCast<const VerbatimText>();
                 if (!verbatimText.isNull()) {
                     QString textBody = verbatimText->text();
                     if (!isOpen) {
                         if (!result.isEmpty()) result.append(" # ");
                         result.append(d->stringOpenDelimiter);
                     } else if (!prev.dynamicCast<const VerbatimText>().isNull()) {
-                        if (key.toLower().startsWith(Entry::ftUrl) || key.toLower().startsWith(Entry::ftLocalFile) || key.toLower().startsWith(Entry::ftDOI))
+                        const QString keyToLower(key.toLower());
+                        if (keyToLower.startsWith(Entry::ftUrl) || keyToLower.startsWith(Entry::ftLocalFile) || keyToLower.startsWith(Entry::ftDOI))
                             /// Filenames and alike have be separated by a semicolon,
                             /// as a plain comma may be part of the filename or URL
-                            result.append(QLatin1String("; "));
+                            result.append(QStringLiteral("; "));
                         else
                             result.append(' ');
                     } else {
@@ -576,7 +599,7 @@ QString FileExporterBibTeX::internalValueToBibTeX(const Value &value, const QStr
                     result.append(textBody);
                     prev = verbatimText;
                 } else {
-                    QSharedPointer<const Person> person = (*it).dynamicCast<const Person>();
+                    QSharedPointer<const Person> person = valueItem.dynamicCast<const Person>();
                     if (!person.isNull()) {
                         QString firstName = person->firstName();
                         if (!firstName.isEmpty() && d->requiresPersonQuoting(firstName, false))
@@ -592,7 +615,7 @@ QString FileExporterBibTeX::internalValueToBibTeX(const Value &value, const QStr
                         /// if name contains a suffix like "Jr."
                         /// Otherwise name could not be parsed again reliable
                         const QString pnf = suffix.isEmpty() ? d->personNameFormatting : Preferences::personNameFormatLastFirst;
-                        QString thisName = encodercheck(encoder, Person::transcribePersonName(pnf, firstName, lastName, suffix));
+                        QString thisName = applyEncoder(Person::transcribePersonName(pnf, firstName, lastName, suffix), useLaTeXEncoding);
 
                         if (!isOpen) {
                             if (!result.isEmpty()) result.append(" # ");
@@ -609,9 +632,9 @@ QString FileExporterBibTeX::internalValueToBibTeX(const Value &value, const QStr
                         result.append(thisName);
                         prev = person;
                     } else {
-                        QSharedPointer<const Keyword> keyword = (*it).dynamicCast<const Keyword>();
+                        QSharedPointer<const Keyword> keyword = valueItem.dynamicCast<const Keyword>();
                         if (!keyword.isNull()) {
-                            QString textBody = encodercheck(encoder, keyword->text());
+                            QString textBody = applyEncoder(keyword->text(), useLaTeXEncoding);
                             if (!isOpen) {
                                 if (!result.isEmpty()) result.append(" # ");
                                 result.append(d->stringOpenDelimiter);
@@ -631,7 +654,7 @@ QString FileExporterBibTeX::internalValueToBibTeX(const Value &value, const QStr
                 }
             }
         }
-        prev = *it;
+        prev = valueItem;
     }
 
     if (isOpen) result.append(d->stringCloseDelimiter);
