@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,7 +12,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "fileimporterris.h"
@@ -23,11 +23,10 @@
 #include <QCoreApplication>
 #include <QStringList>
 
-#include <KDebug>
-
-#include "kbibtexnamespace.h"
+#include "kbibtex.h"
 #include "entry.h"
 #include "value.h"
+#include "logging_io.h"
 
 #define appendValue(entry, fieldname, newvalue) { Value value = (entry)->value((fieldname)); value.append((newvalue)); (entry)->insert((fieldname), value); }
 
@@ -55,14 +54,14 @@ public:
     RISitemList readElement(QTextStream &textStream) {
         RISitemList result;
         QString line = textStream.readLine();
-        while (!line.startsWith(QLatin1String("TY  - ")) && !textStream.atEnd())
+        while (!line.startsWith(QStringLiteral("TY  - ")) && !textStream.atEnd())
             line = textStream.readLine();
         if (textStream.atEnd())
             return result;
 
         QString key, value;
-        while (!line.startsWith(QLatin1String("ER  -")) && !textStream.atEnd()) {
-            if (line.mid(2, 3) == QLatin1String("  -")) {
+        while (!line.startsWith(QStringLiteral("ER  -")) && !textStream.atEnd()) {
+            if (line.mid(2, 3) == QStringLiteral("  -")) {
                 if (!value.isEmpty()) {
                     RISitem item;
                     item.key = key;
@@ -95,55 +94,55 @@ public:
     Element *nextElement(QTextStream &textStream) {
         RISitemList list = readElement(textStream);
         if (list.empty())
-            return NULL;
+            return nullptr;
 
         QString entryType = Entry::etMisc;
-        Entry *entry = new Entry(entryType, QString(QLatin1String("RIS_%1")).arg(referenceCounter++));
+        Entry *entry = new Entry(entryType, QString(QStringLiteral("RIS_%1")).arg(referenceCounter++));
         QString journalName, startPage, endPage, date;
         int fieldCounter = 0;
 
         for (RISitemList::iterator it = list.begin(); it != list.end(); ++it) {
-            if ((*it).key == QLatin1String("TY")) {
-                if ((*it).value.startsWith(QLatin1String("BOOK")) || (*it).value.startsWith(QLatin1String("SER")))
+            if ((*it).key == QStringLiteral("TY")) {
+                if ((*it).value.startsWith(QStringLiteral("BOOK")) || (*it).value.startsWith(QStringLiteral("SER")))
                     entryType = Entry::etBook;
-                else if ((*it).value.startsWith(QLatin1String("CHAP")))
+                else if ((*it).value.startsWith(QStringLiteral("CHAP")))
                     entryType = Entry::etInBook;
-                else if ((*it).value.startsWith(QLatin1String("CONF")))
+                else if ((*it).value.startsWith(QStringLiteral("CONF")))
                     entryType = Entry::etInProceedings;
-                else if ((*it).value.startsWith(QLatin1String("JFULL")) || (*it).value.startsWith(QLatin1String("JOUR")) || (*it).value.startsWith(QLatin1String("MGZN")))
+                else if ((*it).value.startsWith(QStringLiteral("JFULL")) || (*it).value.startsWith(QStringLiteral("JOUR")) || (*it).value.startsWith(QStringLiteral("MGZN")))
                     entryType = Entry::etArticle;
-                else if ((*it).value.startsWith(QLatin1String("RPRT")))
+                else if ((*it).value.startsWith(QStringLiteral("RPRT")))
                     entryType = Entry::etTechReport;
-                else if ((*it).value.startsWith(QLatin1String("THES")))
+                else if ((*it).value.startsWith(QStringLiteral("THES")))
                     entryType = Entry::etPhDThesis; // FIXME what about etMastersThesis?
-                else if ((*it).value.startsWith(QLatin1String("UNPB")))
+                else if ((*it).value.startsWith(QStringLiteral("UNPB")))
                     entryType = Entry::etUnpublished;
                 entry->setType(entryType);
-            } else if ((*it).key == QLatin1String("AU") || (*it).key == QLatin1String("A1")) {
+            } else if ((*it).key == QStringLiteral("AU") || (*it).key == QStringLiteral("A1")) {
                 Person *person = splitName((*it).value);
                 if (person != NULL)
                     appendValue(entry, Entry::ftAuthor, QSharedPointer<Person>(person));
-            } else if ((*it).key == QLatin1String("ED") || (*it).key == QLatin1String("A2")) {
+            } else if ((*it).key == QStringLiteral("ED") || (*it).key == QStringLiteral("A2")) {
                 Person *person = splitName((*it).value);
                 if (person != NULL)
                     appendValue(entry, Entry::ftEditor, QSharedPointer<Person>(person));
-            } else if ((*it).key == QLatin1String("ID")) {
+            } else if ((*it).key == QStringLiteral("ID")) {
                 entry->setId((*it).value);
-            } else if ((*it).key == QLatin1String("Y1") || (*it).key == QLatin1String("PY")) {
+            } else if ((*it).key == QStringLiteral("Y1") || (*it).key == QStringLiteral("PY")) {
                 date = (*it).value;
-            } else if ((*it).key == QLatin1String("Y2")) {
+            } else if ((*it).key == QStringLiteral("Y2")) {
                 if (date.isEmpty())
                     date = (*it).value;
-            } else if ((*it).key == QLatin1String("AB") || (*it).key == QLatin1String("N2")) {
+            } else if ((*it).key == QStringLiteral("AB") || (*it).key == QStringLiteral("N2")) {
                 appendValue(entry, Entry::ftAbstract, QSharedPointer<PlainText>(new PlainText((*it).value)));
-            } else if ((*it).key == QLatin1String("N1")) {
+            } else if ((*it).key == QStringLiteral("N1")) {
                 appendValue(entry, Entry::ftNote, QSharedPointer<PlainText>(new PlainText((*it).value)));
-            } else if ((*it).key == QLatin1String("KW")) {
+            } else if ((*it).key == QStringLiteral("KW")) {
                 QString text = (*it).value;
                 QRegExp splitRegExp;
-                if (text.contains(";"))
+                if (text.contains(QStringLiteral(";")))
                     splitRegExp = QRegExp("\\s*[;\\n]\\s*");
-                else if (text.contains(","))
+                else if (text.contains(QStringLiteral(",")))
                     splitRegExp = QRegExp("\\s*[,\\n]\\s*");
                 else
                     splitRegExp = QRegExp("\\n");
@@ -151,43 +150,43 @@ public:
                 for (QStringList::Iterator it = newKeywords.begin(); it != newKeywords.end();
                         ++it)
                     appendValue(entry, Entry::ftKeywords, QSharedPointer<Keyword>(new Keyword(*it)));
-            } else if ((*it).key == QLatin1String("TI") || (*it).key == QLatin1String("T1")) {
+            } else if ((*it).key == QStringLiteral("TI") || (*it).key == QStringLiteral("T1")) {
                 appendValue(entry, Entry::ftTitle, QSharedPointer<PlainText>(new PlainText((*it).value)));
-            } else if ((*it).key == QLatin1String("T3")) {
+            } else if ((*it).key == QStringLiteral("T3")) {
                 appendValue(entry, Entry::ftSeries, QSharedPointer<PlainText>(new PlainText((*it).value)));
-            } else if ((*it).key == QLatin1String("JO") || (*it).key == QLatin1String("J1") || (*it).key == QLatin1String("J2")) {
+            } else if ((*it).key == QStringLiteral("JO") || (*it).key == QStringLiteral("J1") || (*it).key == QStringLiteral("J2")) {
                 if (journalName.isEmpty())
                     journalName = (*it).value;
-            } else if ((*it).key == QLatin1String("JF") || (*it).key == QLatin1String("JA")) {
+            } else if ((*it).key == QStringLiteral("JF") || (*it).key == QStringLiteral("JA")) {
                 journalName = (*it).value;
-            } else if ((*it).key == QLatin1String("VL")) {
+            } else if ((*it).key == QStringLiteral("VL")) {
                 appendValue(entry, Entry::ftVolume, QSharedPointer<PlainText>(new PlainText((*it).value)));
-            } else if ((*it).key == QLatin1String("CP")) {
+            } else if ((*it).key == QStringLiteral("CP")) {
                 appendValue(entry, Entry::ftChapter, QSharedPointer<PlainText>(new PlainText((*it).value)));
-            } else if ((*it).key == QLatin1String("IS")) {
+            } else if ((*it).key == QStringLiteral("IS")) {
                 appendValue(entry, Entry::ftNumber, QSharedPointer<PlainText>(new PlainText((*it).value)));
-            } else if ((*it).key == QLatin1String("DO")) {
+            } else if ((*it).key == QStringLiteral("DO")) {
                 appendValue(entry, Entry::ftDOI, QSharedPointer<PlainText>(new PlainText((*it).value)));
-            } else if ((*it).key == QLatin1String("PB")) {
+            } else if ((*it).key == QStringLiteral("PB")) {
                 appendValue(entry, Entry::ftPublisher, QSharedPointer<PlainText>(new PlainText((*it).value)));
-            } else if ((*it).key == QLatin1String("IN")) {
+            } else if ((*it).key == QStringLiteral("IN")) {
                 appendValue(entry, Entry::ftSchool, QSharedPointer<PlainText>(new PlainText((*it).value)));
-            } else if ((*it).key == QLatin1String("SN")) {
+            } else if ((*it).key == QStringLiteral("SN")) {
                 const QString fieldName = entryType == Entry::etBook || entryType == Entry::etInBook ? Entry::ftISBN : Entry::ftISSN;
                 appendValue(entry, fieldName, QSharedPointer<PlainText>(new PlainText((*it).value)));
-            } else if ((*it).key == QLatin1String("CY")) {
+            } else if ((*it).key == QStringLiteral("CY")) {
                 appendValue(entry, Entry::ftLocation, QSharedPointer<PlainText>(new PlainText((*it).value)));
-            }  else if ((*it).key == QLatin1String("AD")) {
+            }  else if ((*it).key == QStringLiteral("AD")) {
                 appendValue(entry, Entry::ftAddress, QSharedPointer<PlainText>(new PlainText((*it).value)));
-            } else if ((*it).key == QLatin1String("L1") || (*it).key == QLatin1String("L2") || (*it).key == QLatin1String("L3") || (*it).key == QLatin1String("UR")) {
+            } else if ((*it).key == QStringLiteral("L1") || (*it).key == QStringLiteral("L2") || (*it).key == QStringLiteral("L3") || (*it).key == QStringLiteral("UR")) {
                 const QString fieldName = KBibTeX::doiRegExp.indexIn((*it).value) >= 0 ? Entry::ftDOI : (KBibTeX::urlRegExp.indexIn((*it).value) >= 0 ? Entry::ftUrl : Entry::ftLocalFile);
                 appendValue(entry, fieldName, QSharedPointer<PlainText>(new PlainText((*it).value)));
-            } else if ((*it).key == QLatin1String("SP")) {
+            } else if ((*it).key == QStringLiteral("SP")) {
                 startPage = (*it).value;
-            } else if ((*it).key == QLatin1String("EP")) {
+            } else if ((*it).key == QStringLiteral("EP")) {
                 endPage = (*it).value;
             } else {
-                const QString fieldName = QString(QLatin1String("RISfield_%1_%2")).arg(fieldCounter++).arg((*it).key.left(2));
+                const QString fieldName = QString(QStringLiteral("RISfield_%1_%2")).arg(fieldCounter++).arg((*it).key.left(2));
                 appendValue(entry, fieldName, QSharedPointer<PlainText>(new PlainText((*it).value)));
             }
         }
@@ -213,7 +212,7 @@ public:
             entry->insert(Entry::ftPages, value);
         }
 
-        QStringList dateFragments = date.split(QLatin1String("/"), QString::SkipEmptyParts);
+        QStringList dateFragments = date.split(QStringLiteral("/"), QString::SkipEmptyParts);
         if (dateFragments.count() > 0) {
             bool ok;
             int year = dateFragments[0].toInt(&ok);
@@ -222,7 +221,7 @@ public:
                 value.append(QSharedPointer<PlainText>(new PlainText(QString::number(year))));
                 entry->insert(Entry::ftYear, value);
             } else
-                kDebug() << "invalid year: " << year;
+                qCDebug(LOG_KBIBTEX_IO) << "invalid year: " << year;
         }
         if (dateFragments.count() > 1) {
             bool ok;
@@ -232,7 +231,7 @@ public:
                 value.append(QSharedPointer<MacroKey>(new MacroKey(KBibTeX::MonthsTriple[month - 1])));
                 entry->insert(Entry::ftMonth, value);
             } else
-                kDebug() << "invalid month: " << month;
+                qCDebug(LOG_KBIBTEX_IO) << "invalid month: " << month;
         }
 
         return entry;
@@ -240,8 +239,8 @@ public:
 
 };
 
-FileImporterRIS::FileImporterRIS()
-        : FileImporter(), d(new FileImporterRISPrivate(this))
+FileImporterRIS::FileImporterRIS(QObject *parent)
+        : FileImporter(parent), d(new FileImporterRISPrivate(this))
 {
 // nothing
 }
@@ -255,8 +254,8 @@ FileImporterRIS::~FileImporterRIS()
 File *FileImporterRIS::load(QIODevice *iodevice)
 {
     if (!iodevice->isReadable() && !iodevice->open(QIODevice::ReadOnly)) {
-        kDebug() << "Input device not readable";
-        return NULL;
+        qCDebug(LOG_KBIBTEX_IO) << "Input device not readable";
+        return nullptr;
     }
 
     d->cancelFlag = false;
@@ -268,7 +267,7 @@ File *FileImporterRIS::load(QIODevice *iodevice)
         emit progress(textStream.pos(), iodevice->size());
         QCoreApplication::instance()->processEvents();
         Element *element = d->nextElement(textStream);
-        if (element != NULL)
+        if (element != nullptr)
             result->append(QSharedPointer<Element>(element));
         QCoreApplication::instance()->processEvents();
     }
@@ -276,7 +275,7 @@ File *FileImporterRIS::load(QIODevice *iodevice)
 
     if (d->cancelFlag) {
         delete result;
-        result = NULL;
+        result = nullptr;
     }
 
     iodevice->close();
@@ -285,7 +284,7 @@ File *FileImporterRIS::load(QIODevice *iodevice)
 
 bool FileImporterRIS::guessCanDecode(const QString &text)
 {
-    return text.indexOf("TY  - ") >= 0;
+    return text.indexOf(QStringLiteral("TY  - ")) >= 0;
 }
 
 void FileImporterRIS::cancel()
