@@ -12,78 +12,85 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "fileinfo.h"
 
-#include <poppler-qt4.h>
+#include <poppler-qt5.h>
 
 #include <QFileInfo>
 #include <QDir>
 #include <QTextStream>
-#include <QtConcurrentRun>
+#include <QStandardPaths>
+#include <QtConcurrent/QtConcurrent>
 
 #include <KSharedConfig>
 #include <KConfigGroup>
-#include <KStandardDirs>
 
-#include "kbibtexnamespace.h"
+#include "kbibtex.h"
 #include "entry.h"
 
 FileInfo::FileInfo()
 {
-    // TODO
+    /// nothing
 }
 
-const QString FileInfo::mimetypeOctetStream = QLatin1String("application/octet-stream");
-const QString FileInfo::mimetypeHTML = QLatin1String("text/html");
-const QString FileInfo::mimetypeBibTeX = QLatin1String("text/x-bibtex");
-const QString FileInfo::mimetypeRIS = QLatin1String("application/x-research-info-systems");
-const QString FileInfo::mimetypePDF = QLatin1String("application/pdf");
+const QString FileInfo::mimetypeOctetStream = QStringLiteral("application/octet-stream");
+const QString FileInfo::mimetypeHTML = QStringLiteral("text/html");
+const QString FileInfo::mimetypeBibTeX = QStringLiteral("text/x-bibtex");
+const QString FileInfo::mimetypeRIS = QStringLiteral("application/x-research-info-systems");
+const QString FileInfo::mimetypePDF = QStringLiteral("application/pdf");
 
-KMimeType::Ptr FileInfo::mimeTypeForUrl(const KUrl &url)
+QMimeType FileInfo::mimeTypeForUrl(const QUrl &url)
 {
-    static const QString invalidExtension = QLatin1String("XXXXXXXXXXXXXXXXXXXXXXX");
-    static const KMimeType::Ptr mimetypeHTMLPtr(KMimeType::mimeType(mimetypeHTML));
-    static const KMimeType::Ptr mimetypeOctetStreamPtr(KMimeType::mimeType(mimetypeOctetStream));
-    static const KMimeType::Ptr mimetypeBibTeXPtr(KMimeType::mimeType(mimetypeBibTeX));
-    static const KMimeType::Ptr mimetypePDFPtr(KMimeType::mimeType(mimetypePDF));
+    static QMimeDatabase db;
+    static const QMimeType mtHTML(db.mimeTypeForName(mimetypeHTML));
+    static const QMimeType mtOctetStream(db.mimeTypeForName(mimetypeOctetStream));
+    static const QMimeType mtBibTeX(db.mimeTypeForName(mimetypeBibTeX));
+    static const QMimeType mtPDF(db.mimeTypeForName(mimetypePDF));
+    static const QMimeType mtRIS(db.mimeTypeForName(mimetypeRIS));
     /// Test if mime type for BibTeX is registered before determining file extension
-    static const QString mimetypeBibTeXExt = mimetypeBibTeXPtr.isNull() ? invalidExtension : mimetypeBibTeXPtr->mainExtension().mid(1);
-    static const KMimeType::Ptr mimetypeRISPtr(KMimeType::mimeType(mimetypeRIS));
+    static const QString mimetypeBibTeXExt = mtBibTeX.preferredSuffix();
     /// Test if mime type for RIS is registered before determining file extension
-    static const QString mimetypeRISExt = mimetypeRISPtr.isNull() ? invalidExtension : mimetypeRISPtr->mainExtension().mid(1);
+    static const QString mimetypeRISExt = mtRIS.preferredSuffix();
     /// Test if mime type for PDF is registered before determining file extension
-    static const QString mimetypePDFExt = mimetypePDFPtr.isNull() ? invalidExtension : mimetypePDFPtr->mainExtension().mid(1);
+    static const QString mimetypePDFExt = mtPDF.preferredSuffix();
 
-    const QString extension = KMimeType::extractKnownExtension(url.fileName()).toLower();
+    const QString extension = db.suffixForFileName(url.fileName()).toLower();
+    /// First, check preferred suffixes
     if (extension == mimetypeBibTeXExt)
-        return mimetypeBibTeXPtr;
+        return mtBibTeX;
     else if (extension == mimetypeRISExt)
-        return mimetypeRISPtr;
+        return mtRIS;
     else if (extension == mimetypePDFExt)
-        return mimetypePDFPtr;
-    // TODO other extensions
+        return mtPDF;
+    /// Second, check any other suffixes
+    else if (mtBibTeX.suffixes().contains(extension))
+        return mtBibTeX;
+    else if (mtRIS.suffixes().contains(extension))
+        return mtRIS;
+    else if (mtPDF.suffixes().contains(extension))
+        return mtPDF;
 
     /// Let the KDE subsystem guess the mime type
-    KMimeType::Ptr result = KMimeType::findByUrl(url);
+    QMimeType result = db.mimeTypeForUrl(url);
     /// Fall back to application/octet-stream if something goes wrong
-    if (result.isNull())
-        result = mimetypeOctetStreamPtr;
+    if (!result.isValid())
+        result = mtOctetStream;
 
     /// In case that KDE could not determine mime type,
     /// do some educated guesses on our own
-    if (result.isNull() || result->name() == mimetypeOctetStream) {
-        if (url.protocol().startsWith(QLatin1String("http")))
-            result = mimetypeHTMLPtr;
+    if (result.name() == mimetypeOctetStream) {
+        if (url.scheme().startsWith(QStringLiteral("http")))
+            result = mtHTML;
         // TODO more tests?
     }
 
     return result;
 }
 
-void FileInfo::urlsInText(const QString &text, TestExistence testExistence, const QString &baseDirectory, QList<KUrl> &result)
+void FileInfo::urlsInText(const QString &text, TestExistence testExistence, const QString &baseDirectory, QList<QUrl> &result)
 {
     if (text.isEmpty())
         return;
@@ -95,7 +102,7 @@ void FileInfo::urlsInText(const QString &text, TestExistence testExistence, cons
     int pos = 0;
     while ((pos = KBibTeX::doiRegExp.indexIn(internalText, pos)) != -1) {
         QString match = KBibTeX::doiRegExp.cap(0);
-        KUrl url(doiUrlPrefix() + match.remove("\\"));
+        QUrl url(doiUrlPrefix() + match.remove(QStringLiteral("\\")));
         if (url.isValid() && !result.contains(url))
             result << url;
         /// remove match from internal text to avoid duplicates
@@ -105,7 +112,7 @@ void FileInfo::urlsInText(const QString &text, TestExistence testExistence, cons
         ///   'Lore ipsum http://doi.example.org/10.1000/38-abc Lore ipsum'
         /// also remove 'http://doi.example.org/' from the text, keeping only
         ///   'Lore ipsum  Lore ipsum'
-        static const QRegExp genericDoiUrlPrefix(QLatin1String("http[s]?://[a-z0-9./]+/")); ///< looks like an URL
+        static const QRegExp genericDoiUrlPrefix(QStringLiteral("http[s]?://[a-z0-9./]+/")); ///< looks like an URL
         const int urlStartPos = genericDoiUrlPrefix.lastIndexIn(internalText, pos);
         if (urlStartPos >= 0 && genericDoiUrlPrefix.cap(0).length() > pos - urlStartPos)
             /// genericDoiUrlPrefix.cap(0) may contain (parts of) DOI
@@ -115,14 +122,23 @@ void FileInfo::urlsInText(const QString &text, TestExistence testExistence, cons
     }
 
     const QStringList fileList = internalText.split(KBibTeX::fileListSeparatorRegExp, QString::SkipEmptyParts);
-    for (QStringList::ConstIterator filesIt = fileList.constBegin(); filesIt != fileList.constEnd(); ++filesIt) {
-        internalText = *filesIt;
+    for (const QString &text : fileList) {
+        internalText = text;
 
         /// If testing for the actual existence of a filename found in the text ...
         if (testExistence == TestExistenceYes) {
             /// If a base directory (e.g. the location of the parent .bib file) is given
             /// and the potential filename fragment is NOT an absolute path, ...
-            if (!baseDirectory.isEmpty() &&
+            if (internalText.startsWith(QStringLiteral("~") + QDir::separator())) {
+                const QString fullFilename = QDir::homePath() + internalText.mid(1);
+                const QFileInfo fileInfo(fullFilename);
+                const QUrl url = QUrl::fromLocalFile(fileInfo.canonicalFilePath());
+                if (fileInfo.exists() && fileInfo.isFile() && url.isValid() && !result.contains(url)) {
+                    result << url;
+                    /// Stop searching for URLs or filenames in current internal text
+                    continue;
+                }
+            } else if (!baseDirectory.isEmpty() &&
                     // TODO the following test assumes that absolute paths start
                     // with a dir separator, which may only be true on Unix/Linux,
                     // but not Windows. May be a test for 'first character is a letter,
@@ -131,7 +147,7 @@ void FileInfo::urlsInText(const QString &text, TestExistence testExistence, cons
                 /// To get the absolute path, prepend filename fragment with base directory
                 const QString fullFilename = baseDirectory + QDir::separator() + internalText;
                 const QFileInfo fileInfo(fullFilename);
-                const KUrl url = KUrl::fromPath(fileInfo.canonicalFilePath());
+                const QUrl url = QUrl::fromLocalFile(fileInfo.canonicalFilePath());
                 if (fileInfo.exists() && fileInfo.isFile() && url.isValid() && !result.contains(url)) {
                     result << url;
                     /// Stop searching for URLs or filenames in current internal text
@@ -141,7 +157,7 @@ void FileInfo::urlsInText(const QString &text, TestExistence testExistence, cons
                 /// Either the filename fragment is an absolute path OR no base directory
                 /// was given (current working directory is assumed), ...
                 const QFileInfo fileInfo(internalText);
-                const KUrl url = KUrl::fromPath(fileInfo.canonicalFilePath());
+                const QUrl url = QUrl::fromLocalFile(fileInfo.canonicalFilePath());
                 if (fileInfo.exists() && fileInfo.isFile() && url.isValid() && !result.contains(url)) {
                     result << url;
                     /// stop searching for URLs or filenames in current internal text
@@ -154,8 +170,8 @@ void FileInfo::urlsInText(const QString &text, TestExistence testExistence, cons
         pos = 0;
         while ((pos = KBibTeX::urlRegExp.indexIn(internalText, pos)) != -1) {
             const QString match = KBibTeX::urlRegExp.cap(0);
-            KUrl url(match);
-            if (url.isValid() && (testExistence == TestExistenceNo || !url.isLocalFile() || QFileInfo(url.toLocalFile()).exists()) && !result.contains(url))
+            QUrl url(match);
+            if (url.isValid() && (testExistence == TestExistenceNo || !url.isLocalFile() || QFileInfo::exists(url.toLocalFile())) && !result.contains(url))
                 result << url;
             /// remove match from internal text to avoid duplicates
             internalText = internalText.left(pos) + internalText.mid(pos + match.length());
@@ -164,10 +180,10 @@ void FileInfo::urlsInText(const QString &text, TestExistence testExistence, cons
         /// explicitly check URL entry, may be an URL even if http:// or alike is missing
         pos = 0;
         while ((pos = KBibTeX::domainNameRegExp.indexIn(internalText, pos)) > -1) {
-            int pos2 = internalText.indexOf(" ", pos + 1);
+            int pos2 = internalText.indexOf(QStringLiteral(" "), pos + 1);
             if (pos2 < 0) pos2 = internalText.length();
             QString match = internalText.mid(pos, pos2 - pos);
-            KUrl url("http://" + match);
+            QUrl url("http://" + match);
             if (url.isValid() && !result.contains(url))
                 result << url;
             /// remove match from internal text to avoid duplicates
@@ -178,8 +194,8 @@ void FileInfo::urlsInText(const QString &text, TestExistence testExistence, cons
         pos = 0;
         while ((pos = KBibTeX::fileRegExp.indexIn(internalText, pos)) != -1) {
             QString match = KBibTeX::fileRegExp.cap(0);
-            KUrl url(match);
-            if (url.isValid() && (testExistence == TestExistenceNo || !url.isLocalFile() || QFileInfo(url.toLocalFile()).exists()) && !result.contains(url))
+            QUrl url(match);
+            if (url.isValid() && (testExistence == TestExistenceNo || !url.isLocalFile() || QFileInfo::exists(url.toLocalFile())) && !result.contains(url))
                 result << url;
             /// remove match from internal text to avoid duplicates
             internalText = internalText.left(pos) + internalText.mid(pos + match.length());
@@ -187,51 +203,50 @@ void FileInfo::urlsInText(const QString &text, TestExistence testExistence, cons
     }
 }
 
-QList<KUrl> FileInfo::entryUrls(const Entry *entry, const KUrl &bibTeXUrl, TestExistence testExistence)
+QList<QUrl> FileInfo::entryUrls(const QSharedPointer<const Entry> &entry, const QUrl &bibTeXUrl, TestExistence testExistence)
 {
-    QList<KUrl> result;
-    if (entry == NULL || entry->isEmpty())
+    QList<QUrl> result;
+    if (entry.isNull() || entry->isEmpty())
         return result;
 
     if (entry->contains(Entry::ftDOI)) {
         const QString doi = PlainTextValue::text(entry->value(Entry::ftDOI));
         if (!doi.isEmpty() && KBibTeX::doiRegExp.indexIn(doi) == 0) {
             QString match = KBibTeX::doiRegExp.cap(0);
-            KUrl url(doiUrlPrefix() + match.remove("\\"));
+            QUrl url(doiUrlPrefix() + match.remove(QStringLiteral("\\")));
             result.append(url);
         }
     }
-    static const QString etPMID = QLatin1String("pmid");
+    static const QString etPMID = QStringLiteral("pmid");
     if (entry->contains(etPMID)) {
         const QString pmid = PlainTextValue::text(entry->value(etPMID));
         bool ok = false;
         ok &= pmid.toInt(&ok) > 0;
         if (ok) {
-            KUrl url(QLatin1String("https://www.ncbi.nlm.nih.gov/pubmed/") + pmid);
+            QUrl url(QStringLiteral("https://www.ncbi.nlm.nih.gov/pubmed/") + pmid);
             result.append(url);
         }
     }
-    static const QString etEPrint = QLatin1String("eprint");
+    static const QString etEPrint = QStringLiteral("eprint");
     if (entry->contains(etEPrint)) {
         const QString eprint = PlainTextValue::text(entry->value(etEPrint));
         if (!eprint.isEmpty()) {
-            KUrl url(QLatin1String("http://arxiv.org/search?query=") + eprint);
+            QUrl url(QStringLiteral("http://arxiv.org/search?query=") + eprint);
             result.append(url);
         }
     }
 
-    const QString baseDirectory = bibTeXUrl.isValid() ? bibTeXUrl.directory() : QString();
+    const QString baseDirectory = bibTeXUrl.isValid() ? bibTeXUrl.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash).path() : QString();
 
-    static const QRegExp regExpEscapedChars = QRegExp("\\\\+([&_~])");
+    static const QRegExp regExpEscapedChars = QRegExp(QStringLiteral("\\\\+([&_~])"));
     for (Entry::ConstIterator it = entry->constBegin(); it != entry->constEnd(); ++it) {
         /// skip abstracts, they contain sometimes strange text fragments
         /// that are mistaken for URLs
         if (it.key().toLower() == Entry::ftAbstract) continue;
 
-        Value v = it.value();
-
-        for (Value::ConstIterator vit = v.constBegin(); vit != v.constEnd(); ++vit) {
-            QString plainText = PlainTextValue::text(*(*vit));
+        const Value v = it.value();
+        for (const auto &valueItem : v) {
+            QString plainText = PlainTextValue::text(*valueItem);
 
             int pos = -1;
             while ((pos = regExpEscapedChars.indexIn(plainText, pos + 1)) != -1)
@@ -243,14 +258,15 @@ QList<KUrl> FileInfo::entryUrls(const Entry *entry, const KUrl &bibTeXUrl, TestE
 
     if (!baseDirectory.isEmpty()) {
         /// File types supported by "document preview"
-        static const QStringList documentFileExtensions = QStringList() << ".pdf" << ".pdf.gz" << ".pdf.bz2" << ".ps" << ".ps.gz" << ".ps.bz2" << ".eps" << ".eps.gz" << ".eps.bz2" << ".html" << ".xhtml" << ".htm" << ".dvi" << ".djvu" << ".wwf" << ".jpeg" << ".jpg" << ".png" << ".gif" << ".tif" << ".tiff";
+        static const QStringList documentFileExtensions = QStringList() << QStringLiteral(".pdf") << QStringLiteral(".pdf.gz") << QStringLiteral(".pdf.bz2") << QStringLiteral(".ps") << QStringLiteral(".ps.gz") << QStringLiteral(".ps.bz2") << QStringLiteral(".eps") << QStringLiteral(".eps.gz") << QStringLiteral(".eps.bz2") << QStringLiteral(".html") << QStringLiteral(".xhtml") << QStringLiteral(".htm") << QStringLiteral(".dvi") << QStringLiteral(".djvu") << QStringLiteral(".wwf") << QStringLiteral(".jpeg") << QStringLiteral(".jpg") << QStringLiteral(".png") << QStringLiteral(".gif") << QStringLiteral(".tif") << QStringLiteral(".tiff");
+        result.reserve(result.size() + documentFileExtensions.size() * 2);
 
         /// check if in the same directory as the BibTeX file
         /// a PDF file exists which filename is based on the entry's id
-        for (QStringList::ConstIterator extensionIt = documentFileExtensions.constBegin(); extensionIt != documentFileExtensions.constEnd(); ++extensionIt) {
-            const QFileInfo fi(baseDirectory + QDir::separator() + entry->id() + *extensionIt);
+        for (const QString &extension : documentFileExtensions) {
+            const QFileInfo fi(baseDirectory + QDir::separator() + entry->id() + extension);
             if (fi.exists()) {
-                const KUrl url = KUrl::fromPath(fi.canonicalFilePath());
+                const QUrl url = QUrl::fromLocalFile(fi.canonicalFilePath());
                 if (!result.contains(url))
                     result << url;
             }
@@ -261,10 +277,10 @@ QList<KUrl> FileInfo::entryUrls(const Entry *entry, const KUrl &bibTeXUrl, TestE
         /// which filename is based on the entry's id
         QString basename = bibTeXUrl.fileName().remove(QRegExp("\\.[^.]{2,5}$"));
         QString directory = baseDirectory + QDir::separator() + basename;
-        for (QStringList::ConstIterator extensionIt = documentFileExtensions.constBegin(); extensionIt != documentFileExtensions.constEnd(); ++extensionIt) {
-            const QFileInfo fi(directory + QDir::separator() + entry->id() + *extensionIt);
+        for (const QString &extension : documentFileExtensions) {
+            const QFileInfo fi(directory + QDir::separator() + entry->id() + extension);
             if (fi.exists()) {
-                const KUrl url = KUrl::fromPath(fi.canonicalFilePath());
+                const QUrl url = QUrl::fromLocalFile(fi.canonicalFilePath());
                 if (!result.contains(url))
                     result << url;
             }
@@ -277,11 +293,15 @@ QList<KUrl> FileInfo::entryUrls(const Entry *entry, const KUrl &bibTeXUrl, TestE
 QString FileInfo::pdfToText(const QString &pdfFilename)
 {
     /// Build filename for text file where PDF file's plain text is cached
-    static const QRegExp invalidChars("[^-a-z0-9_]", Qt::CaseInsensitive);
-    QString textFilename = QString(pdfFilename).remove(invalidChars).append(QLatin1String(".txt")).prepend(KStandardDirs::locateLocal("cache", "pdftotext/"));
+    static const QRegExp invalidChars(QStringLiteral("[^-a-z0-9_]"), Qt::CaseInsensitive);
+    const QString cacheDirectory = QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + QStringLiteral("/pdftotext");
+    if (!QDir(cacheDirectory).exists() && !QDir::home().mkdir(cacheDirectory))
+        /// Could not create cache directory
+        return QString();
+    const QString textFilename = QString(pdfFilename).remove(invalidChars).append(QStringLiteral(".txt")).prepend(QStringLiteral("/")).prepend(cacheDirectory);
 
     /// First, check if there is a cache text file
-    if (QFileInfo(textFilename).exists()) {
+    if (QFileInfo::exists(textFilename)) {
         /// Load text from cache file
         QFile f(textFilename);
         if (f.open(QFile::ReadOnly)) {
@@ -291,7 +311,7 @@ QString FileInfo::pdfToText(const QString &pdfFilename)
             return text;
         }
     } else
-        /// No cache text exists, so run text extraction in another thread
+        /// No cache file exists, so run text extraction in another thread
         QtConcurrent::run(extractPDFTextToCache, pdfFilename, textFilename);
 
     return QString();
@@ -306,16 +326,16 @@ void FileInfo::extractPDFTextToCache(const QString &pdfFilename, const QString &
 
     /// Load PDF file through Poppler
     Poppler::Document *doc = Poppler::Document::load(pdfFilename);
-    if (doc != NULL) {
+    if (doc != nullptr) {
         static const int maxPages = 64;
         /// Build text by appending each page's text
         for (int i = 0; i < qMin(maxPages, doc->numPages()); ++i)
-            text.append(doc->page(i)->text(QRect())).append(QLatin1String("\n\n"));
+            text.append(doc->page(i)->text(QRect())).append(QStringLiteral("\n\n"));
         if (doc->numPages() > maxPages)
-            msgList << QString(QLatin1String("### Skipped %1 pages as PDF file contained too many pages (limit is %2 pages) ###")).arg(doc->numPages() - maxPages).arg(maxPages);
+            msgList << QString(QStringLiteral("### Skipped %1 pages as PDF file contained too many pages (limit is %2 pages) ###")).arg(doc->numPages() - maxPages).arg(maxPages);
         delete doc;
     } else
-        msgList << QLatin1String("### Skipped as file could not be opened as PDF file ###");
+        msgList << QStringLiteral("### Skipped as file could not be opened as PDF file ###");
 
     /// Save text in cache file
     QFile f(cacheFilename);
@@ -325,9 +345,9 @@ void FileInfo::extractPDFTextToCache(const QString &pdfFilename, const QString &
         ts << text.left(maxCharacters); ///< keep only the first 2^18 many characters
 
         if (text.length() > maxCharacters)
-            msgList << QString(QLatin1String("### Text too long, skipping %1 characters ###")).arg(text.length() - maxCharacters);
+            msgList << QString(QStringLiteral("### Text too long, skipping %1 characters ###")).arg(text.length() - maxCharacters);
         /// Write all messages (warnings) to end of text file
-        foreach (const QString &msg, msgList)
+        for (const QString &msg : const_cast<const QStringList &>(msgList))
             ts << endl << msg;
 
         f.close();
@@ -336,9 +356,9 @@ void FileInfo::extractPDFTextToCache(const QString &pdfFilename, const QString &
 
 QString FileInfo::doiUrlPrefix()
 {
-    KSharedConfigPtr config(KSharedConfig::openConfig(QLatin1String("kbibtexrc")));
-    static const QString configGroupNameNetworking(QLatin1String("Networking"));
-    static const QString keyDOIUrlPrefix(QLatin1String("DOIUrlPrefix"));
+    KSharedConfigPtr config(KSharedConfig::openConfig(QStringLiteral("kbibtexrc")));
+    static const QString configGroupNameNetworking(QStringLiteral("Networking"));
+    static const QString keyDOIUrlPrefix(QStringLiteral("DOIUrlPrefix"));
     KConfigGroup configGroup(config, configGroupNameNetworking);
     return configGroup.readEntry(keyDOIUrlPrefix, KBibTeX::doiUrlPrefix);
 }
