@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,33 +12,36 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "onlinesearchabstract.h"
 
 #include <QFileInfo>
 #include <QNetworkReply>
+#include <QDir>
 #include <QTimer>
+#include <QStandardPaths>
+#ifdef HAVE_QTWIDGETS
 #include <QListWidgetItem>
 #include <QDBusConnection>
 #include <QDBusConnectionInterface>
+#endif // HAVE_QTWIDGETS
 
-#include <KStandardDirs>
-#include <kio/netaccess.h>
-#include <KDebug>
-#include <KLocale>
+#ifdef HAVE_KF5
+#include <KLocalizedString>
 #include <KMessageBox>
-#include <KPassivePopup>
+#endif // HAVE_KF5
 
 #include "encoderlatex.h"
 #include "internalnetworkaccessmanager.h"
-#include "kbibtexnamespace.h"
+#include "kbibtex.h"
+#include "logging_networking.h"
 
-const QString OnlineSearchAbstract::queryKeyFreeText = QLatin1String("free");
-const QString OnlineSearchAbstract::queryKeyTitle = QLatin1String("title");
-const QString OnlineSearchAbstract::queryKeyAuthor = QLatin1String("author");
-const QString OnlineSearchAbstract::queryKeyYear = QLatin1String("year");
+const QString OnlineSearchAbstract::queryKeyFreeText = QStringLiteral("free");
+const QString OnlineSearchAbstract::queryKeyTitle = QStringLiteral("title");
+const QString OnlineSearchAbstract::queryKeyAuthor = QStringLiteral("author");
+const QString OnlineSearchAbstract::queryKeyYear = QStringLiteral("year");
 
 const int OnlineSearchAbstract::resultNoError = 0;
 const int OnlineSearchAbstract::resultCancelled = 0; /// may get redefined in the future!
@@ -50,47 +53,65 @@ const int OnlineSearchAbstract::resultInvalidArguments = 4;
 const char *OnlineSearchAbstract::httpUnsafeChars = "%:/=+$?&\0";
 
 
+#ifdef HAVE_QTWIDGETS
 QStringList OnlineSearchQueryFormAbstract::authorLastNames(const Entry &entry)
 {
-    QStringList result;
-    EncoderLaTeX *encoder = EncoderLaTeX::instance();
-
+    const EncoderLaTeX &encoder = EncoderLaTeX::instance();
     const Value v = entry[Entry::ftAuthor];
-    QSharedPointer<Person> p;
-    foreach (const QSharedPointer<ValueItem> &vi, v)
-        if (!(p = vi.dynamicCast<Person>()).isNull())
-            result.append(encoder->convertToPlainAscii(p->lastName()));
+    QStringList result;
+    result.reserve(v.size());
+    for (const QSharedPointer<ValueItem> &vi : v) {
+        QSharedPointer<const Person> p = vi.dynamicCast<const Person>();
+        if (!p.isNull())
+            result.append(encoder.convertToPlainAscii(p->lastName()));
+    }
 
     return result;
 }
+#endif // HAVE_QTWIDGETS
 
-OnlineSearchAbstract::OnlineSearchAbstract(QWidget *parent)
-        : QObject(parent), m_hasBeenCanceled(false), m_delayedStoppedSearchReturnCode(0)
+OnlineSearchAbstract::OnlineSearchAbstract(QObject *parent)
+    : QObject(parent), m_hasBeenCanceled(false), numSteps(0), curStep(0), m_previousBusyState(false), m_delayedStoppedSearchReturnCode(0)
 {
     m_parent = parent;
 }
 
-KIcon OnlineSearchAbstract::icon(QListWidgetItem *listWidgetItem)
+QIcon OnlineSearchAbstract::icon(QListWidgetItem *listWidgetItem)
 {
-    static const QRegExp invalidChars(QLatin1String("[^-a-z0-9_]"), Qt::CaseInsensitive);
-    const QString fileNameStem = KStandardDirs::locateLocal("cache", QLatin1String("favicons/")) + QString(favIconUrl()).remove(invalidChars);
-    const QStringList fileNameExtensions = QStringList() << QLatin1String(".ico") << QLatin1String(".png") << QString();
+    static const QRegExp invalidChars(QStringLiteral("[^-a-z0-9_]"), Qt::CaseInsensitive);
+    const QString cacheDirectory = QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + QStringLiteral("/favicons/");
+    QDir().mkpath(cacheDirectory);
+    const QString fileNameStem = cacheDirectory + QString(favIconUrl()).remove(invalidChars);
+    const QStringList fileNameExtensions = QStringList() << QStringLiteral(".ico") << QStringLiteral(".png") << QString();
 
-    foreach (const QString &extension, fileNameExtensions) {
+    for (const QString &extension : fileNameExtensions) {
         const QString fileName = fileNameStem + extension;
-        if (QFileInfo(fileName).exists())
-            return KIcon(fileName);
+        if (QFileInfo::exists(fileName))
+            return QIcon(fileName);
     }
 
     QNetworkRequest request(favIconUrl());
-    QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
+    QNetworkReply *reply = InternalNetworkAccessManager::instance().get(request);
     reply->setObjectName(fileNameStem);
-    if (listWidgetItem != NULL)
+    if (listWidgetItem != nullptr)
         m_iconReplyToListWidgetItem.insert(reply, listWidgetItem);
-    connect(reply, SIGNAL(finished()), this, SLOT(iconDownloadFinished()));
-    return KIcon(QLatin1String("applications-internet"));
+    connect(reply, &QNetworkReply::finished, this, &OnlineSearchAbstract::iconDownloadFinished);
+    return QIcon::fromTheme(QStringLiteral("applications-internet"));
 }
 
+#ifdef HAVE_QTWIDGETS
+OnlineSearchQueryFormAbstract *OnlineSearchAbstract::customWidget(QWidget *) {
+    return nullptr;
+}
+
+void OnlineSearchAbstract::startSearchFromForm()
+{
+    m_hasBeenCanceled = false;
+    curStep = numSteps = 0;
+    delayedStoppedSearch(resultNoError);
+}
+#endif // HAVE_QTWIDGETS
+
 QString OnlineSearchAbstract::name()
 {
     static const QRegExp invalidChars("[^-a-z0-9]", Qt::CaseInsensitive);
@@ -99,9 +120,17 @@ QString OnlineSearchAbstract::name()
     return m_name;
 }
 
+bool OnlineSearchAbstract::busy() const
+{
+    return numSteps > 0 && curStep < numSteps;
+}
+
 void OnlineSearchAbstract::cancel()
 {
     m_hasBeenCanceled = true;
+
+    curStep = numSteps = 0;
+    refreshBusyProperty();
 }
 
 QStringList OnlineSearchAbstract::splitRespectingQuotationMarks(const QString &text)
@@ -125,29 +154,30 @@ QStringList OnlineSearchAbstract::splitRespectingQuotationMarks(const QString &t
 
 bool OnlineSearchAbstract::handleErrors(QNetworkReply *reply)
 {
-    KUrl url;
+    QUrl url;
     return handleErrors(reply, url);
 }
 
-bool OnlineSearchAbstract::handleErrors(QNetworkReply *reply, KUrl &newUrl)
+bool OnlineSearchAbstract::handleErrors(QNetworkReply *reply, QUrl &newUrl)
 {
-    newUrl = KUrl();
+    newUrl = QUrl();
     if (m_hasBeenCanceled) {
-        kDebug() << "Searching" << label() << "got cancelled";
-        emit stoppedSearch(resultCancelled);
+        stopSearch(resultCancelled);
         return false;
     } else if (reply->error() != QNetworkReply::NoError) {
         m_hasBeenCanceled = true;
         const QString errorString = reply->errorString();
-        kWarning() << "Search using" << label() << "failed (error code" << reply->error() << "(" << errorString << "), HTTP code" << reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt() << ":" << reply->attribute(QNetworkRequest::HttpReasonPhraseAttribute).toByteArray() << ")";
-        sendVisualNotification(errorString.isEmpty() ? i18n("Searching '%1' failed for unknown reason.", label()) : i18n("Searching '%1' failed with error message:\n\n%2", label(), errorString), label(), "kbibtex", 7 * 1000);
+        qCWarning(LOG_KBIBTEX_NETWORKING) << "Search using" << label() << "failed (error code" << reply->error() << "(" << errorString << "), HTTP code" << reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt() << ":" << reply->attribute(QNetworkRequest::HttpReasonPhraseAttribute).toByteArray() << ")";
+#ifdef HAVE_KF5
+        sendVisualNotification(errorString.isEmpty() ? i18n("Searching '%1' failed for unknown reason.", label()) : i18n("Searching '%1' failed with error message:\n\n%2", label(), errorString), label(), QStringLiteral("kbibtex"), 7 * 1000);
+#endif // HAVE_KF5
 
         int resultCode = resultUnspecifiedError;
         if (reply->error() == QNetworkReply::AuthenticationRequiredError || reply->error() == QNetworkReply::ProxyAuthenticationRequiredError)
             resultCode = resultAuthorizationRequired;
         else if (reply->error() == QNetworkReply::HostNotFoundError || reply->error() == QNetworkReply::TimeoutError)
             resultCode = resultNetworkError;
-        emit stoppedSearch(resultCode);
+        stopSearch(resultCode);
 
         return false;
     }
@@ -160,35 +190,34 @@ bool OnlineSearchAbstract::handleErrors(QNetworkReply *reply, KUrl &newUrl)
      */
     if (reply->attribute(QNetworkRequest::RedirectionTargetAttribute).isValid()) {
         newUrl = reply->url().resolved(reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl());
-        kDebug() << "Redirection from" << reply->url().toString() << "to" << newUrl.pathOrUrl();
     } else if (reply->size() == 0)
-        kWarning() << "Search using" << label() << "on url" << reply->url().toString() << "returned no data";
+        qCWarning(LOG_KBIBTEX_NETWORKING) << "Search using" << label() << "on url" << reply->url().toDisplayString() << "returned no data";
 
     return true;
 }
 
+#ifdef HAVE_KF5
 /**
  * Display a passive notification popup using the D-Bus interface.
  * Copied from KDialog with modifications.
  */
 void OnlineSearchAbstract::sendVisualNotification(const QString &text, const QString &title, const QString &icon, int timeout)
 {
-    static const QString dbusServiceName = QLatin1String("org.freedesktop.Notifications");
-    static const QString dbusInterfaceName = QLatin1String("org.freedesktop.Notifications");
-    static const QString dbusPath = QLatin1String("/org/freedesktop/Notifications");
+    static const QString dbusServiceName = QStringLiteral("org.freedesktop.Notifications");
+    static const QString dbusInterfaceName = QStringLiteral("org.freedesktop.Notifications");
+    static const QString dbusPath = QStringLiteral("/org/freedesktop/Notifications");
 
     // check if service already exists on plugin instantiation
     QDBusConnectionInterface *interface = QDBusConnection::sessionBus().interface();
-    if (interface == NULL || !interface->isServiceRegistered(dbusServiceName)) {
-        //kDebug() << dbusServiceName << "D-Bus service not registered";
+    if (interface == nullptr || !interface->isServiceRegistered(dbusServiceName)) {
         return;
     }
 
     if (timeout <= 0)
         timeout = 10 * 1000;
 
-    QDBusMessage m = QDBusMessage::createMethodCall(dbusServiceName, dbusPath, dbusInterfaceName, "Notify");
-    QList<QVariant> args = QList<QVariant>() << QLatin1String("kdialog") << 0U << icon << title << text << QStringList() << QVariantMap() << timeout;
+    QDBusMessage m = QDBusMessage::createMethodCall(dbusServiceName, dbusPath, dbusInterfaceName, QStringLiteral("Notify"));
+    QList<QVariant> args = QList<QVariant>() << QStringLiteral("kdialog") << 0U << icon << title << text << QStringList() << QVariantMap() << timeout;
     m.setArguments(args);
 
     QDBusMessage replyMsg = QDBusConnection::sessionBus().call(m);
@@ -199,15 +228,16 @@ void OnlineSearchAbstract::sendVisualNotification(const QString &text, const QSt
         // Not displaying any error messages as this is optional for kdialog
         // and KPassivePopup is a perfectly valid fallback.
         //else {
-        //  kDebug() << "Error: received reply with no arguments.";
+        //  qCDebug(LOG_KBIBTEX_NETWORKING) << "Error: received reply with no arguments.";
         //}
     } else if (replyMsg.type() == QDBusMessage::ErrorMessage) {
-        //kDebug() << "Error: failed to send D-Bus message";
-        //kDebug() << replyMsg;
+        //qCDebug(LOG_KBIBTEX_NETWORKING) << "Error: failed to send D-Bus message";
+        //qCDebug(LOG_KBIBTEX_NETWORKING) << replyMsg;
     } else {
-        //kDebug() << "Unexpected reply type";
+        //qCDebug(LOG_KBIBTEX_NETWORKING) << "Unexpected reply type";
     }
 }
+#endif // HAVE_KF5
 
 QString OnlineSearchAbstract::encodeURL(QString rawText)
 {
@@ -229,18 +259,18 @@ QString OnlineSearchAbstract::decodeURL(QString rawText)
         if (ok)
             rawText = rawText.replace(mimeRegExp.cap(0), c);
     }
-    rawText = rawText.replace(QLatin1String("&amp;"), QLatin1String("&")).replace(QLatin1Char('+'), QLatin1String(" "));
+    rawText = rawText.replace(QStringLiteral("&amp;"), QStringLiteral("&")).replace(QLatin1Char('+'), QStringLiteral(" "));
     return rawText;
 }
 
 QMap<QString, QString> OnlineSearchAbstract::formParameters(const QString &htmlText, int startPos)
 {
     /// how to recognize HTML tags
-    static const QString formTagEnd = QLatin1String("</form>");
-    static const QString inputTagBegin = QLatin1String("<input ");
-    static const QString selectTagBegin = QLatin1String("<select ");
-    static const QString selectTagEnd = QLatin1String("</select>");
-    static const QString optionTagBegin = QLatin1String("<option ");
+    static const QString formTagEnd = QStringLiteral("</form>");
+    static const QString inputTagBegin = QStringLiteral("<input ");
+    static const QString selectTagBegin = QStringLiteral("<select ");
+    static const QString selectTagEnd = QStringLiteral("</select>");
+    static const QString optionTagBegin = QStringLiteral("<option ");
     /// regular expressions to test or retrieve attributes in HTML tags
     static const QRegExp inputTypeRegExp("<input[^>]+\\btype=[\"]?([^\" >\n\t]*)", Qt::CaseInsensitive);
     static const QRegExp inputNameRegExp("<input[^>]+\\bname=[\"]?([^\" >\n\t]*)", Qt::CaseInsensitive);
@@ -256,7 +286,7 @@ QMap<QString, QString> OnlineSearchAbstract::formParameters(const QString &htmlT
     /// determined boundaries of (only) "form" tag
     int endPos = htmlText.indexOf(formTagEnd, startPos, Qt::CaseInsensitive);
     if (startPos < 0 || endPos < 0) {
-        kWarning() << "Could not locate form in text";
+        qCWarning(LOG_KBIBTEX_NETWORKING) << "Could not locate form in text";
         return result;
     }
 
@@ -270,14 +300,14 @@ QMap<QString, QString> OnlineSearchAbstract::formParameters(const QString &htmlT
 
         if (!inputName.isEmpty()) {
             /// get value of input types
-            if (inputType == QLatin1String("hidden") || inputType == QLatin1String("text") || inputType == QLatin1String("submit"))
+            if (inputType == QStringLiteral("hidden") || inputType == QStringLiteral("text") || inputType == QStringLiteral("submit"))
                 result[inputName] = inputValue;
-            else if (inputType == QLatin1String("radio")) {
+            else if (inputType == QStringLiteral("radio")) {
                 /// must be selected
                 if (htmlText.indexOf(inputIsCheckedRegExp, p) == p) {
                     result[inputName] = inputValue;
                 }
-            } else if (inputType == QLatin1String("checkbox")) {
+            } else if (inputType == QStringLiteral("checkbox")) {
                 /// must be checked
                 if (htmlText.indexOf(inputIsCheckedRegExp, p) == p) {
                     /// multiple checkbox values with the same name are possible
@@ -323,29 +353,42 @@ void OnlineSearchAbstract::iconDownloadFinished()
     QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
 
     if (reply->error() == QNetworkReply::NoError) {
+        const QUrl redirUrl = reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl();
+        if (redirUrl.isValid()) {
+            QNetworkRequest request(redirUrl);
+            QNetworkReply *newReply = InternalNetworkAccessManager::instance().get(request);
+            newReply->setObjectName(reply->objectName());
+            QListWidgetItem *listWidgetItem = m_iconReplyToListWidgetItem.value(reply, nullptr);
+            m_iconReplyToListWidgetItem.remove(reply);
+            if (listWidgetItem != nullptr)
+                m_iconReplyToListWidgetItem.insert(newReply, listWidgetItem);
+            connect(newReply, &QNetworkReply::finished, this, &OnlineSearchAbstract::iconDownloadFinished);
+            return;
+        }
+
         const QByteArray iconData = reply->readAll();
         if (iconData.size() < 10) {
             /// Unlikely that an icon's data is less than 10 bytes,
             /// must be an error.
-            kWarning() << "Received invalid icon data from " << reply->url().toString();
+            qCWarning(LOG_KBIBTEX_NETWORKING) << "Received invalid icon data from " << reply->url().toDisplayString();
             return;
         }
 
         QString extension;
         if (iconData[1] == 'P' && iconData[2] == 'N' && iconData[3] == 'G') {
             /// PNG files have string "PNG" at second to fourth byte
-            extension = QLatin1String(".png");
+            extension = QStringLiteral(".png");
         } else if (iconData[0] == (char)0x00 && iconData[1] == (char)0x00 && iconData[2] == (char)0x01 && iconData[3] == (char)0x00) {
             /// Microsoft Icon have first two bytes always 0x0000,
             /// third and fourth byte is 0x0001 (for .ico)
-            extension = QLatin1String(".ico");
+            extension = QStringLiteral(".ico");
         } else if (iconData[0] == '<') {
             /// HTML or XML code
             const QString htmlCode = QString::fromUtf8(iconData);
-            kDebug() << "Received XML or HTML data from " << reply->url().toString() << ": " << htmlCode.left(128);
+            qCDebug(LOG_KBIBTEX_NETWORKING) << "Received XML or HTML data from " << reply->url().toDisplayString() << ": " << htmlCode.left(128);
             return;
         } else {
-            kWarning() << "Favicon is of unknown format: " << reply->url().toString();
+            qCWarning(LOG_KBIBTEX_NETWORKING) << "Favicon is of unknown format: " << reply->url().toDisplayString();
             return;
         }
         const QString filename = reply->objectName() + extension;
@@ -355,22 +398,26 @@ void OnlineSearchAbstract::iconDownloadFinished()
             iconFile.write(iconData);
             iconFile.close();
 
-            QListWidgetItem *listWidgetItem = m_iconReplyToListWidgetItem.value(reply, NULL);
-            if (listWidgetItem != NULL)
-                listWidgetItem->setIcon(KIcon(filename));
+            QListWidgetItem *listWidgetItem = m_iconReplyToListWidgetItem.value(reply, nullptr);
+            if (listWidgetItem != nullptr)
+                listWidgetItem->setIcon(QIcon(filename));
+        } else {
+            qCWarning(LOG_KBIBTEX_NETWORKING) << "Could not save icon data from URL" << reply->url().toDisplayString() << "to file" << filename;
+            return;
         }
     } else
-        kWarning() << "Could not download icon " << reply->url().toString();
+        qCWarning(LOG_KBIBTEX_NETWORKING) << "Could not download icon from URL " << reply->url().toDisplayString() << ": " << reply->errorString();
 }
 
 void OnlineSearchAbstract::dumpToFile(const QString &filename, const QString &text)
 {
-    const QString usedFilename = KStandardDirs::locateLocal("tmp", filename, true);
+    const QString usedFilename = QDir::tempPath() + QLatin1Char('/') + filename;
 
     QFile f(usedFilename);
     if (f.open(QFile::WriteOnly)) {
-        kDebug() << "Dumping text" << squeezeText(text, 96) << "to" << usedFilename;
+        qCDebug(LOG_KBIBTEX_NETWORKING) << "Dumping text" << KBibTeX::squeezeText(text, 96) << "to" << usedFilename;
         QTextStream ts(&f);
+        ts.setCodec("UTF-8");
         ts << text;
         f.close();
     }
@@ -379,13 +426,13 @@ void OnlineSearchAbstract::dumpToFile(const QString &filename, const QString &te
 void OnlineSearchAbstract::delayedStoppedSearch(int returnCode)
 {
     m_delayedStoppedSearchReturnCode = returnCode;
-    QTimer::singleShot(500, this, SLOT(delayedStoppedSearchTimer()));
+    QTimer::singleShot(500, this, &OnlineSearchAbstract::delayedStoppedSearchTimer);
 }
 
 void OnlineSearchAbstract::delayedStoppedSearchTimer()
 {
     emit progress(1, 1);
-    emit stoppedSearch(m_delayedStoppedSearchReturnCode);
+    stopSearch(m_delayedStoppedSearchReturnCode);
 }
 
 void OnlineSearchAbstract::sanitizeEntry(QSharedPointer<Entry> entry)
@@ -394,9 +441,12 @@ void OnlineSearchAbstract::sanitizeEntry(QSharedPointer<Entry> entry)
 
     /// Sometimes, there is no identifier, so set a random one
     if (entry->id().isEmpty())
-        entry->setId(QString(QLatin1String("entry-%1")).arg(QString::number(qrand(), 36)));
+        entry->setId(QString(QStringLiteral("entry-%1")).arg(QString::number(qrand(), 36)));
+    /// Missing entry type? Set it to 'misc'
+    if (entry->type().isEmpty())
+        entry->setType(Entry::etMisc);
 
-    const QString ftIssue = QLatin1String("issue");
+    const QString ftIssue = QStringLiteral("issue");
     if (entry->contains(ftIssue)) {
         /// ACM's Digital Library uses "issue" instead of "number" -> fix that
         Value v = entry->value(ftIssue);
@@ -406,7 +456,7 @@ void OnlineSearchAbstract::sanitizeEntry(QSharedPointer<Entry> entry)
 
     /// If entry contains a description field but no abstract,
     /// rename description field to abstract
-    const QString ftDescription = QLatin1String("description");
+    const QString ftDescription = QStringLiteral("description");
     if (!entry->contains(Entry::ftAbstract) && entry->contains(ftDescription)) {
         Value v = entry->value(ftDescription);
         entry->remove(ftDescription);
@@ -416,12 +466,12 @@ void OnlineSearchAbstract::sanitizeEntry(QSharedPointer<Entry> entry)
     /// Remove "dblp" artifacts in abstracts and keywords
     if (entry->contains(Entry::ftAbstract)) {
         const QString abstract = PlainTextValue::text(entry->value(Entry::ftAbstract));
-        if (abstract == QLatin1String("dblp"))
+        if (abstract == QStringLiteral("dblp"))
             entry->remove(Entry::ftAbstract);
     }
     if (entry->contains(Entry::ftKeywords)) {
         const QString keywords = PlainTextValue::text(entry->value(Entry::ftKeywords));
-        if (keywords == QLatin1String("dblp"))
+        if (keywords == QStringLiteral("dblp"))
             entry->remove(Entry::ftKeywords);
     }
 
@@ -429,7 +479,7 @@ void OnlineSearchAbstract::sanitizeEntry(QSharedPointer<Entry> entry)
         /// Fix strigns for months: "September" -> "sep"
         const QString monthStr = PlainTextValue::text(entry->value(Entry::ftMonth));
 
-        static const QRegExp longMonth = QRegExp(QLatin1String("(jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)[a-z]+"), Qt::CaseInsensitive);
+        static const QRegExp longMonth = QRegExp(QStringLiteral("(jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)[a-z]*"), Qt::CaseInsensitive);
         if (monthStr.indexOf(longMonth) == 0 && monthStr == longMonth.cap(0)) {
             /// String used for month is actually a full name, therefore replace it
             entry->remove(Entry::ftMonth);
@@ -453,8 +503,8 @@ bool OnlineSearchAbstract::publishEntry(QSharedPointer<Entry> entry)
     if (entry.isNull()) return false;
 
     Value v;
-    v.append(QSharedPointer<VerbatimText>(new VerbatimText(label())));
-    entry->insert("x-fetchedfrom", v);
+    v.append(QSharedPointer<PlainText>(new PlainText(label())));
+    entry->insert(QStringLiteral("x-fetchedfrom"), v);
 
     sanitizeEntry(entry);
 
@@ -462,3 +512,19 @@ bool OnlineSearchAbstract::publishEntry(QSharedPointer<Entry> entry)
 
     return true;
 }
+
+void OnlineSearchAbstract::stopSearch(int errorCode) {
+    if (errorCode == resultNoError)
+        curStep = numSteps;
+    else
+        curStep = numSteps = 0;
+    emit stoppedSearch(errorCode);
+}
+
+void OnlineSearchAbstract::refreshBusyProperty() {
+    const bool newBusyState = busy();
+    if (newBusyState != m_previousBusyState) {
+        m_previousBusyState = newBusyState;
+        emit busyChanged();
+    }
+}
