@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,40 +12,49 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "onlinesearchbibsonomy.h"
 
 #include <QBuffer>
+#ifdef HAVE_QTWIDGETS
 #include <QLayout>
 #include <QSpinBox>
 #include <QLabel>
+#include <QIcon>
+#endif // HAVE_QTWIDGETS
 #include <QNetworkReply>
 
-#include <KLocale>
-#include <KDebug>
-#include <KIcon>
-#include <KLineEdit>
+#ifdef HAVE_KF5
+#include <KLocalizedString>
+#include <KConfigGroup>
+#endif // HAVE_KF5
+#ifdef HAVE_QTWIDGETS
 #include <KComboBox>
 #include <KMessageBox>
-#include <KConfigGroup>
+#include <KLineEdit>
+#endif // HAVE_QTWIDGETS
 
 #include "fileimporterbibtex.h"
 #include "file.h"
 #include "entry.h"
 #include "internalnetworkaccessmanager.h"
+#include "logging_networking.h"
 
+#ifdef HAVE_QTWIDGETS
 class OnlineSearchBibsonomy::OnlineSearchQueryFormBibsonomy : public OnlineSearchQueryFormAbstract
 {
+    Q_OBJECT
+
 private:
     QString configGroupName;
 
     void loadState() {
         KConfigGroup configGroup(config, configGroupName);
-        comboBoxSearchWhere->setCurrentIndex(configGroup.readEntry(QLatin1String("searchWhere"), 0));
-        lineEditSearchTerm->setText(configGroup.readEntry(QLatin1String("searchTerm"), QString()));
-        numResultsField->setValue(configGroup.readEntry(QLatin1String("numResults"), 10));
+        comboBoxSearchWhere->setCurrentIndex(configGroup.readEntry(QStringLiteral("searchWhere"), 0));
+        lineEditSearchTerm->setText(configGroup.readEntry(QStringLiteral("searchTerm"), QString()));
+        numResultsField->setValue(configGroup.readEntry(QStringLiteral("numResults"), 10));
     }
 
 public:
@@ -54,7 +63,7 @@ public:
     QSpinBox *numResultsField;
 
     OnlineSearchQueryFormBibsonomy(QWidget *widget)
-            : OnlineSearchQueryFormAbstract(widget), configGroupName(QLatin1String("Search Engine Bibsonomy")) {
+            : OnlineSearchQueryFormAbstract(widget), configGroupName(QStringLiteral("Search Engine Bibsonomy")) {
         QGridLayout *layout = new QGridLayout(this);
         layout->setMargin(0);
 
@@ -71,8 +80,8 @@ public:
 
         lineEditSearchTerm = new KLineEdit(this);
         layout->addWidget(lineEditSearchTerm, 0, 1, 1, 1);
-        lineEditSearchTerm->setClearButtonShown(true);
-        connect(lineEditSearchTerm, SIGNAL(returnPressed()), this, SIGNAL(returnPressed()));
+        lineEditSearchTerm->setClearButtonEnabled(true);
+        connect(lineEditSearchTerm, &KLineEdit::returnPressed, this, &OnlineSearchQueryFormBibsonomy::returnPressed);
 
         QLabel *label = new QLabel(i18n("Number of Results:"), this);
         layout->addWidget(label, 1, 0, 1, 1);
@@ -89,23 +98,24 @@ public:
         loadState();
     }
 
-    bool readyToStart() const {
+    bool readyToStart() const override {
         return !lineEditSearchTerm->text().isEmpty();
     }
 
-    void copyFromEntry(const Entry &entry) {
+    void copyFromEntry(const Entry &entry) override {
         comboBoxSearchWhere->setCurrentIndex(comboBoxSearchWhere->count() - 1);
-        lineEditSearchTerm->setText(authorLastNames(entry).join(QLatin1String(" ")) + QLatin1Char(' ') + PlainTextValue::text(entry[Entry::ftTitle]));
+        lineEditSearchTerm->setText(authorLastNames(entry).join(QStringLiteral(" ")) + QLatin1Char(' ') + PlainTextValue::text(entry[Entry::ftTitle]));
     }
 
     void saveState() {
         KConfigGroup configGroup(config, configGroupName);
-        configGroup.writeEntry(QLatin1String("searchWhere"), comboBoxSearchWhere->currentIndex());
-        configGroup.writeEntry(QLatin1String("searchTerm"), lineEditSearchTerm->text());
-        configGroup.writeEntry(QLatin1String("numResults"), numResultsField->value());
+        configGroup.writeEntry(QStringLiteral("searchWhere"), comboBoxSearchWhere->currentIndex());
+        configGroup.writeEntry(QStringLiteral("searchTerm"), lineEditSearchTerm->text());
+        configGroup.writeEntry(QStringLiteral("numResults"), numResultsField->value());
         config->sync();
     }
 };
+#endif // HAVE_QTWIDGETS
 
 class OnlineSearchBibsonomy::OnlineSearchBibsonomyPrivate
 {
@@ -113,37 +123,44 @@ private:
     OnlineSearchBibsonomy *p;
 
 public:
+#ifdef HAVE_QTWIDGETS
     OnlineSearchQueryFormBibsonomy *form;
-    int numSteps, curStep;
+#endif // HAVE_QTWIDGETS
 
     OnlineSearchBibsonomyPrivate(OnlineSearchBibsonomy *parent)
-            : p(parent), form(NULL), numSteps(0), curStep(0) {
+            : p(parent)
+#ifdef HAVE_QTWIDGETS
+        , form(nullptr)
+#endif // HAVE_QTWIDGETS
+    {
         // nothing
     }
 
-    KUrl buildQueryUrl() {
-        if (form == NULL) {
-            kWarning() << "Cannot build query url if no form is specified";
-            return KUrl();
+#ifdef HAVE_QTWIDGETS
+    QUrl buildQueryUrl() {
+        if (form == nullptr) {
+            qCWarning(LOG_KBIBTEX_NETWORKING) << "Cannot build query url if no form is specified";
+            return QUrl();
         }
 
         QString queryString = p->encodeURL(form->lineEditSearchTerm->text());
-        return KUrl("http://www.bibsonomy.org/bib/" + form->comboBoxSearchWhere->itemData(form->comboBoxSearchWhere->currentIndex()).toString() + "/" + queryString + QString(QLatin1String("?items=%1")).arg(form->numResultsField->value()));
+        return QUrl(QStringLiteral("https://www.bibsonomy.org/bib/") + form->comboBoxSearchWhere->itemData(form->comboBoxSearchWhere->currentIndex()).toString() + "/" + queryString + QString(QStringLiteral("?items=%1")).arg(form->numResultsField->value()));
     }
+#endif // HAVE_QTWIDGETS
 
-    KUrl buildQueryUrl(const QMap<QString, QString> &query, int numResults) {
-        QString url = QLatin1String("http://www.bibsonomy.org/bib/");
+    QUrl buildQueryUrl(const QMap<QString, QString> &query, int numResults) {
+        QString url = QStringLiteral("https://www.bibsonomy.org/bib/");
 
         bool hasFreeText = !query[queryKeyFreeText].isEmpty();
         bool hasTitle = !query[queryKeyTitle].isEmpty();
         bool hasAuthor = !query[queryKeyAuthor].isEmpty();
         bool hasYear = !query[queryKeyYear].isEmpty();
 
-        QString searchType = QLatin1String("search");
+        QString searchType = QStringLiteral("search");
         if (hasAuthor && !hasFreeText && !hasTitle && !hasYear) {
             /// if only the author field is used, a special author search
             /// on BibSonomy can be used
-            searchType = QLatin1String("author");
+            searchType = QStringLiteral("author");
         }
 
         QStringList queryFragments;
@@ -151,25 +168,25 @@ public:
             queryFragments << p->encodeURL(it.value());
         }
 
-        QString queryString = queryFragments.join(QLatin1String("%20"));
-        url.append(searchType + QLatin1Char('/') + queryString + QString(QLatin1String("?items=%1")).arg(numResults));
+        QString queryString = queryFragments.join(QStringLiteral("%20"));
+        url.append(searchType + QLatin1Char('/') + queryString + QString(QStringLiteral("?items=%1")).arg(numResults));
 
-        return KUrl(url);
+        return QUrl(url);
     }
 
     void sanitizeEntry(QSharedPointer<Entry> entry) {
         /// if entry contains a description field but no abstract,
         /// rename description field to abstract
-        const QString ftDescription = QLatin1String("description");
+        const QString ftDescription = QStringLiteral("description");
         if (!entry->contains(Entry::ftAbstract) && entry->contains(ftDescription)) {
-            Value v = entry->value(QLatin1String("description"));
+            Value v = entry->value(QStringLiteral("description"));
             entry->insert(Entry::ftAbstract, v);
             entry->remove(ftDescription);
         }
     }
 };
 
-OnlineSearchBibsonomy::OnlineSearchBibsonomy(QWidget *parent)
+OnlineSearchBibsonomy::OnlineSearchBibsonomy(QObject *parent)
         : OnlineSearchAbstract(parent), d(new OnlineSearchBibsonomyPrivate(this))
 {
     // nothing
@@ -183,32 +200,28 @@ OnlineSearchBibsonomy::~OnlineSearchBibsonomy()
 void OnlineSearchBibsonomy::startSearch(const QMap<QString, QString> &query, int numResults)
 {
     m_hasBeenCanceled = false;
-    d->curStep = 0;
-    d->numSteps = 1;
+    emit progress(curStep = 0, numSteps = 1);
 
     QNetworkRequest request(d->buildQueryUrl(query, numResults));
-    QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
-    InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
-    connect(reply, SIGNAL(finished()), this, SLOT(downloadDone()));
-
-    emit progress(0, d->numSteps);
+    QNetworkReply *reply = InternalNetworkAccessManager::instance().get(request);
+    InternalNetworkAccessManager::instance().setNetworkReplyTimeout(reply);
+    connect(reply, &QNetworkReply::finished, this, &OnlineSearchBibsonomy::downloadDone);
 }
 
-void OnlineSearchBibsonomy::startSearch()
+#ifdef HAVE_QTWIDGETS
+void OnlineSearchBibsonomy::startSearchFromForm()
 {
     m_hasBeenCanceled = false;
-    d->curStep = 0;
-    d->numSteps = 1;
+    emit progress(curStep = 0, numSteps = 1);
 
     QNetworkRequest request(d->buildQueryUrl());
-    QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
-    InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
-    connect(reply, SIGNAL(finished()), this, SLOT(downloadDone()));
+    QNetworkReply *reply = InternalNetworkAccessManager::instance().get(request);
+    InternalNetworkAccessManager::instance().setNetworkReplyTimeout(reply);
+    connect(reply, &QNetworkReply::finished, this, &OnlineSearchBibsonomy::downloadDone);
 
-    emit progress(0, d->numSteps);
-
-    d->form->saveState();
+    emit progress(0, numSteps);
 }
+#endif // HAVE_QTWIDGETS
 
 QString OnlineSearchBibsonomy::label() const
 {
@@ -217,29 +230,26 @@ QString OnlineSearchBibsonomy::label() const
 
 QString OnlineSearchBibsonomy::favIconUrl() const
 {
-    return QLatin1String("http://www.bibsonomy.org/resources/image/favicon.png");
+    return QStringLiteral("https://www.bibsonomy.org/resources/image/favicon.png");
 }
 
+#ifdef HAVE_QTWIDGETS
 OnlineSearchQueryFormAbstract *OnlineSearchBibsonomy::customWidget(QWidget *parent)
 {
-    if (d->form == NULL)
+    if (d->form == nullptr)
         d->form = new OnlineSearchBibsonomy::OnlineSearchQueryFormBibsonomy(parent);
     return d->form;
 }
+#endif // HAVE_QTWIDGETS
 
-KUrl OnlineSearchBibsonomy::homepage() const
+QUrl OnlineSearchBibsonomy::homepage() const
 {
-    return KUrl("http://www.bibsonomy.org/");
-}
-
-void OnlineSearchBibsonomy::cancel()
-{
-    OnlineSearchAbstract::cancel();
+    return QUrl(QStringLiteral("https://www.bibsonomy.org/"));
 }
 
 void OnlineSearchBibsonomy::downloadDone()
 {
-    emit progress(++d->curStep, d->numSteps);
+    emit progress(++curStep, numSteps);
 
     QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
 
@@ -248,32 +258,31 @@ void OnlineSearchBibsonomy::downloadDone()
         QString bibTeXcode = QString::fromUtf8(reply->readAll().constData());
 
         if (!bibTeXcode.isEmpty()) {
-            FileImporterBibTeX importer;
-            File *bibtexFile = importer.fromString(bibTeXcode);
+            FileImporterBibTeX importer(this);
+            const File *bibtexFile = importer.fromString(bibTeXcode);
 
             bool hasEntries = false;
-            if (bibtexFile != NULL) {
-                for (File::ConstIterator it = bibtexFile->constBegin(); it != bibtexFile->constEnd(); ++it) {
-                    QSharedPointer<Entry> entry = (*it).dynamicCast<Entry>();
+            if (bibtexFile != nullptr) {
+                for (const auto &element : const_cast<const File &>(*bibtexFile)) {
+                    QSharedPointer<Entry> entry = element.dynamicCast<Entry>();
                     hasEntries |= publishEntry(entry);
                 }
 
-                if (!hasEntries)
-                    kDebug() << "No hits found in" << reply->url().toString();
-                emit stoppedSearch(resultNoError);
-                emit progress(d->numSteps, d->numSteps);
+                stopSearch(resultNoError);
+                emit progress(curStep = numSteps, numSteps);
 
                 delete bibtexFile;
             } else {
-                kWarning() << "No valid BibTeX file results returned on request on" << reply->url().toString();
-                emit stoppedSearch(resultUnspecifiedError);
+                qCWarning(LOG_KBIBTEX_NETWORKING) << "No valid BibTeX file results returned on request on" << reply->url().toDisplayString();
+                stopSearch(resultUnspecifiedError);
             }
         } else {
             /// returned file is empty
-            kDebug() << "No hits found in" << reply->url().toString();
-            emit stoppedSearch(resultNoError);
-            emit progress(d->numSteps, d->numSteps);
+            stopSearch(resultNoError);
+            emit progress(curStep = numSteps, numSteps);
         }
     } else
-        kDebug() << "url was" << reply->url().toString();
+        qCWarning(LOG_KBIBTEX_NETWORKING) << "url was" << reply->url().toDisplayString();
 }
+
+#include "onlinesearchbibsonomy.moc"
