@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,7 +12,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "onlinesearchideasrepec.h"
@@ -20,51 +20,53 @@
 #include <QSet>
 #include <QNetworkRequest>
 #include <QNetworkReply>
+#include <QUrlQuery>
 
-#include <KLocale>
-#include <KDebug>
+#include <KLocalizedString>
 
 #include "fileimporterbibtex.h"
 #include "internalnetworkaccessmanager.h"
+#include "logging_networking.h"
 
 class OnlineSearchIDEASRePEc::OnlineSearchIDEASRePEcPrivate
 {
 public:
-    int numSteps, curStep;
-    QSet<QString> publicationLinks;
+    QSet<const QUrl> publicationLinks;
 
-    KUrl buildQueryUrl(const QMap<QString, QString> &query, int numResults) {
-        QString urlBase = QLatin1String("https://ideas.repec.org/cgi-bin/htsearch?cmd=Search%21&form=extended&m=all&fmt=url&wm=wrd&sp=1&sy=1&dt=range");
+    QUrl buildQueryUrl(const QMap<QString, QString> &query, int numResults) {
+        QString urlBase = QStringLiteral("https://ideas.repec.org/cgi-bin/htsearch?cmd=Search%21&form=extended&m=all&fmt=url&wm=wrd&sp=1&sy=1&dt=range");
 
         bool hasFreeText = !query[queryKeyFreeText].isEmpty();
         bool hasTitle = !query[queryKeyTitle].isEmpty();
         bool hasAuthor = !query[queryKeyAuthor].isEmpty();
-        bool hasYear = QRegExp(QLatin1String("^(19|20)[0-9]{2}$")).indexIn(query[queryKeyYear]) == 0;
+        bool hasYear = QRegExp(QStringLiteral("^(19|20)[0-9]{2}$")).indexIn(query[queryKeyYear]) == 0;
 
-        QString fieldWF = QLatin1String("4BFF"); ///< search whole record by default
+        QString fieldWF = QStringLiteral("4BFF"); ///< search whole record by default
         QString fieldQ, fieldDB, fieldDE;
         if (hasAuthor && !hasFreeText && !hasTitle) {
             /// If only the author field is used, search explictly for author
-            fieldWF = QLatin1String("000F");
+            fieldWF = QStringLiteral("000F");
             fieldQ = query[queryKeyAuthor];
         } else if (!hasAuthor && !hasFreeText && hasTitle) {
             /// If only the title field is used, search explictly for title
-            fieldWF = QLatin1String("00F0");
+            fieldWF = QStringLiteral("00F0");
             fieldQ = query[queryKeyTitle];
         } else {
             fieldQ = query[queryKeyFreeText] + QLatin1Char(' ') + query[queryKeyTitle] + QLatin1Char(' ') + query[queryKeyAuthor] + QLatin1Char(' ');
         }
         if (hasYear) {
-            fieldDB = QLatin1String("01/01/") + query[queryKeyYear];
-            fieldDE = QLatin1String("31/12/") + query[queryKeyYear];
+            fieldDB = QStringLiteral("01/01/") + query[queryKeyYear];
+            fieldDE = QStringLiteral("31/12/") + query[queryKeyYear];
         }
 
-        KUrl url(urlBase);
-        url.addQueryItem(QLatin1String("ps"), QString::number(numResults));
-        url.addQueryItem(QLatin1String("db"), fieldDB);
-        url.addQueryItem(QLatin1String("de"), fieldDE);
-        url.addQueryItem(QLatin1String("q"), fieldQ);
-        url.addQueryItem(QLatin1String("wf"), fieldWF);
+        QUrl url(urlBase);
+        QUrlQuery q(url);
+        q.addQueryItem(QStringLiteral("ps"), QString::number(numResults));
+        q.addQueryItem(QStringLiteral("db"), fieldDB);
+        q.addQueryItem(QStringLiteral("de"), fieldDE);
+        q.addQueryItem(QStringLiteral("q"), fieldQ);
+        q.addQueryItem(QStringLiteral("wf"), fieldWF);
+        url.setQuery(q);
 
         return url;
     }
@@ -82,25 +84,16 @@ OnlineSearchIDEASRePEc::~OnlineSearchIDEASRePEc()
     delete d;
 }
 
-void OnlineSearchIDEASRePEc::startSearch()
-{
-    m_hasBeenCanceled = false;
-    delayedStoppedSearch(resultNoError);
-}
-
 void OnlineSearchIDEASRePEc::startSearch(const QMap<QString, QString> &query, int numResults)
 {
-    const KUrl url = d->buildQueryUrl(query, numResults);
-    d->curStep = 0;
-    d->numSteps = 2 * numResults + 1;
+    const QUrl url = d->buildQueryUrl(query, numResults);
+    emit progress(curStep = 0, numSteps = 2 * numResults + 1);
     m_hasBeenCanceled = false;
 
     QNetworkRequest request(url);
-    QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
-    InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
-    connect(reply, SIGNAL(finished()), this, SLOT(downloadListDone()));
-
-    emit progress(0, d->numSteps);
+    QNetworkReply *reply = InternalNetworkAccessManager::instance().get(request);
+    InternalNetworkAccessManager::instance().setNetworkReplyTimeout(reply);
+    connect(reply, &QNetworkReply::finished, this, &OnlineSearchIDEASRePEc::downloadListDone);
 }
 
 
@@ -111,77 +104,69 @@ QString OnlineSearchIDEASRePEc::label() const
 
 QString OnlineSearchIDEASRePEc::favIconUrl() const
 {
-    return QLatin1String("https://ideas.repec.org/favicon.ico");
+    return QStringLiteral("https://ideas.repec.org/favicon.ico");
 }
 
-OnlineSearchQueryFormAbstract *OnlineSearchIDEASRePEc::customWidget(QWidget *)
+QUrl OnlineSearchIDEASRePEc::homepage() const
 {
-    return NULL;
-}
-
-KUrl OnlineSearchIDEASRePEc::homepage() const
-{
-    return KUrl("https://ideas.repec.org/");
-}
-
-void OnlineSearchIDEASRePEc::cancel()
-{
-    OnlineSearchAbstract::cancel();
+    return QUrl(QStringLiteral("https://ideas.repec.org/"));
 }
 
 void OnlineSearchIDEASRePEc::downloadListDone()
 {
-    emit progress(++d->curStep, d->numSteps);
+    emit progress(++curStep, numSteps);
 
     QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
 
-    KUrl redirUrl;
+    QUrl redirUrl;
     if (handleErrors(reply, redirUrl)) {
         if (redirUrl.isValid()) {
             /// redirection to another url
-            ++d->numSteps;
+            ++numSteps;
 
             QNetworkRequest request(redirUrl);
-            QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request);
-            InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
-            connect(newReply, SIGNAL(finished()), this, SLOT(downloadListDone()));
+            QNetworkReply *newReply = InternalNetworkAccessManager::instance().get(request);
+            InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
+            connect(newReply, &QNetworkReply::finished, this, &OnlineSearchIDEASRePEc::downloadListDone);
         } else {
             /// ensure proper treatment of UTF-8 characters
             const QString htmlCode = QString::fromUtf8(reply->readAll().constData());
 
-            static const QRegExp publicationLinkRegExp(QLatin1String("http[s]?://ideas.repec.org/[a-z]/\\S{,8}/\\S{2,24}/\\S{,64}.html"));
+            const int ol1 = htmlCode.indexOf(QStringLiteral(" results for "));
+            const int ol2 = htmlCode.indexOf(QStringLiteral("</ol>"), ol1 + 2);
             d->publicationLinks.clear();
-            int p = -1;
-            while ((p = publicationLinkRegExp.indexIn(htmlCode, p + 1)) >= 0) {
-                QString c = publicationLinkRegExp.cap(0);
-                /// Rewrite URL to be https instead of http, avoids HTTP redirection
-                c = c.replace(QLatin1String("http://"), QLatin1String("https://"));
-                d->publicationLinks.insert(c);
+            if (ol1 > 0 && ol2 > ol1) {
+                const QString olHtmlCode = htmlCode.mid(ol1, ol2 - ol1 + 5);
+                static const QRegExp publicationLinkRegExp(QStringLiteral("\"/[a-z](/[^\"]+){1,6}[.]html"));
+                int p = -1;
+                while ((p = publicationLinkRegExp.indexIn(olHtmlCode, p + 1)) > 0) {
+                    const QUrl c = reply->url().resolved(QUrl(publicationLinkRegExp.cap().mid(1)));
+                    d->publicationLinks.insert(c);
+                }
+                numSteps += 2 * d->publicationLinks.count(); ///< update number of steps
             }
-            d->numSteps += 2 * d->publicationLinks.count(); ///< update number of steps
 
             if (d->publicationLinks.isEmpty()) {
-                kDebug() << "No publication links found";
-                emit stoppedSearch(resultNoError);
-                emit progress(1, 1);
+                stopSearch(resultNoError);
+                emit progress(curStep = numSteps, numSteps);
             } else {
-                QSet<QString>::Iterator it = d->publicationLinks.begin();
-                const QString publicationLink = *it;
+                QSet<const QUrl>::Iterator it = d->publicationLinks.begin();
+                const QUrl publicationLink = *it;
                 d->publicationLinks.erase(it);
-                QNetworkRequest request = QNetworkRequest(QUrl(publicationLink));
-                reply = InternalNetworkAccessManager::self()->get(request, reply);
-                InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
-                connect(reply, SIGNAL(finished()), this, SLOT(downloadPublicationDone()));
+                QNetworkRequest request = QNetworkRequest(publicationLink);
+                reply = InternalNetworkAccessManager::instance().get(request, reply);
+                InternalNetworkAccessManager::instance().setNetworkReplyTimeout(reply);
+                connect(reply, &QNetworkReply::finished, this, &OnlineSearchIDEASRePEc::downloadPublicationDone);
             }
         }
     } else
-        kDebug() << "url was" << reply->url().toString();
+        qCWarning(LOG_KBIBTEX_NETWORKING) << "url was" << reply->url().toDisplayString();
 
 }
 
 void OnlineSearchIDEASRePEc::downloadPublicationDone()
 {
-    emit progress(++d->curStep, d->numSteps);
+    emit progress(++curStep, numSteps);
 
     QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
 
@@ -190,14 +175,14 @@ void OnlineSearchIDEASRePEc::downloadPublicationDone()
         const QString htmlCode = QString::fromUtf8(reply->readAll().constData());
 
         QString downloadUrl;
-        static const QString downloadFormStart = QLatin1String("<FORM METHOD=GET ACTION=\"/cgi-bin/get_doc.pl\"");
+        static const QString downloadFormStart = QStringLiteral("<FORM METHOD=GET ACTION=\"/cgi-bin/get_doc.pl\"");
         if (htmlCode.contains(downloadFormStart)) {
             QMap<QString, QString> form = formParameters(htmlCode, htmlCode.indexOf(downloadFormStart, Qt::CaseInsensitive));
-            downloadUrl = form[QLatin1String("url")];
+            downloadUrl = form[QStringLiteral("url")];
         }
 
-        QMap<QString, QString> form = formParameters(htmlCode, htmlCode.indexOf(QLatin1String("<form method=\"post\" action=\"/cgi-bin/refs.cgi\""), Qt::CaseInsensitive));
-        form[QLatin1String("output")] = QLatin1String("2"); ///< enforce BibTeX output
+        QMap<QString, QString> form = formParameters(htmlCode, htmlCode.indexOf(QStringLiteral("<form method=\"post\" action=\"/cgi-bin/refs.cgi\""), Qt::CaseInsensitive));
+        form[QStringLiteral("output")] = QStringLiteral("2"); ///< enforce BibTeX output
 
         QString body;
         QMap<QString, QString>::ConstIterator it = form.constBegin();
@@ -207,21 +192,21 @@ void OnlineSearchIDEASRePEc::downloadPublicationDone()
             ++it;
         }
 
-        const KUrl url = KUrl(QLatin1String("https://ideas.repec.org/cgi-bin/refs.cgi"));
+        const QUrl url = QUrl(QStringLiteral("https://ideas.repec.org/cgi-bin/refs.cgi"));
         QNetworkRequest request(url);
         request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
-        reply = InternalNetworkAccessManager::self()->post(request, body.toUtf8());
+        reply = InternalNetworkAccessManager::instance().post(request, body.toUtf8());
         reply->setProperty("downloadurl", QVariant::fromValue<QString>(downloadUrl));
-        InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
-        connect(reply, SIGNAL(finished()), this, SLOT(downloadBibTeXDone()));
+        InternalNetworkAccessManager::instance().setNetworkReplyTimeout(reply);
+        connect(reply, &QNetworkReply::finished, this, &OnlineSearchIDEASRePEc::downloadBibTeXDone);
 
     } else
-        kDebug() << "url was" << reply->url().toString();
+        qCWarning(LOG_KBIBTEX_NETWORKING) << "url was" << reply->url().toDisplayString();
 }
 
 void OnlineSearchIDEASRePEc::downloadBibTeXDone()
 {
-    emit progress(++d->curStep, d->numSteps);
+    emit progress(++curStep, numSteps);
 
     QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
     const QString downloadUrl = reply->property("downloadurl").toString();
@@ -231,13 +216,12 @@ void OnlineSearchIDEASRePEc::downloadBibTeXDone()
         const QString bibTeXcode = QString::fromUtf8(reply->readAll().constData());
 
         if (!bibTeXcode.isEmpty()) {
-            FileImporterBibTeX importer;
+            FileImporterBibTeX importer(this);
             File *bibtexFile = importer.fromString(bibTeXcode);
 
-            bool hasEntries = false;
-            if (bibtexFile != NULL) {
-                for (File::ConstIterator it = bibtexFile->constBegin(); it != bibtexFile->constEnd(); ++it) {
-                    QSharedPointer<Entry> entry = (*it).dynamicCast<Entry>();
+            if (bibtexFile != nullptr) {
+                for (const auto &element : const_cast<const File &>(*bibtexFile)) {
+                    QSharedPointer<Entry> entry = element.dynamicCast<Entry>();
                     if (!entry.isNull()) {
                         if (!downloadUrl.isEmpty()) {
                             /// There is an external document associated with this BibTeX entry
@@ -248,32 +232,28 @@ void OnlineSearchIDEASRePEc::downloadBibTeXDone()
 
                         Value v;
                         v.append(QSharedPointer<VerbatimText>(new VerbatimText(label())));
-                        entry->insert("x-fetchedfrom", v);
+                        entry->insert(QStringLiteral("x-fetchedfrom"), v);
                         emit foundEntry(entry);
-                        hasEntries = true;
                     }
 
                 }
 
-                if (!hasEntries)
-                    kDebug() << "No hits found in" << reply->url().toString() << "(was" << downloadUrl << ")";
-
                 delete bibtexFile;
             }
         }
 
         if (d->publicationLinks.isEmpty()) {
-            emit stoppedSearch(resultNoError);
+            stopSearch(resultNoError);
             emit progress(1, 1);
         } else {
-            QSet<QString>::Iterator it = d->publicationLinks.begin();
-            const QString publicationLink = *it;
+            QSet<const QUrl>::Iterator it = d->publicationLinks.begin();
+            const QUrl publicationLink = *it;
             d->publicationLinks.erase(it);
-            QNetworkRequest request = QNetworkRequest(QUrl(publicationLink));
-            reply = InternalNetworkAccessManager::self()->get(request, reply);
-            InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
-            connect(reply, SIGNAL(finished()), this, SLOT(downloadPublicationDone()));
+            QNetworkRequest request = QNetworkRequest(publicationLink);
+            reply = InternalNetworkAccessManager::instance().get(request, reply);
+            InternalNetworkAccessManager::instance().setNetworkReplyTimeout(reply);
+            connect(reply, &QNetworkReply::finished, this, &OnlineSearchIDEASRePEc::downloadPublicationDone);
         }
     } else
-        kDebug() << "url was" << reply->url().toString() << "(was" << downloadUrl << ")";
+        qCWarning(LOG_KBIBTEX_NETWORKING) << "url was" << reply->url().toDisplayString() << "(was" << downloadUrl << ")";
 }
