@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,20 +12,22 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "onlinesearchjstor.h"
 
 #include <QNetworkRequest>
 #include <QNetworkReply>
+#include <QUrlQuery>
 
-#include <KLocale>
-#include <KDebug>
+#ifdef HAVE_KF5
+#include <KLocalizedString>
+#endif // HAVE_KF5
 
 #include "internalnetworkaccessmanager.h"
-#include "iocommon.h"
 #include "fileimporterbibtex.h"
+#include "logging_networking.h"
 
 class OnlineSearchJStor::OnlineSearchJStorPrivate
 {
@@ -33,20 +35,20 @@ private:
     // UNUSED OnlineSearchJStor *p;
 
 public:
-    int numExpectedResults, curStep, numSteps;
+    int numExpectedResults;
     static const QString jstorBaseUrl;
-    KUrl queryUrl;
+    QUrl queryUrl;
 
     OnlineSearchJStorPrivate(OnlineSearchJStor */* UNUSED parent*/)
-        : /* UNUSED p(parent), */ numExpectedResults(0), curStep(0), numSteps(0)
+        : /* UNUSED p(parent), */ numExpectedResults(0)
     {
         /// nothing
     }
 };
 
-const QString OnlineSearchJStor::OnlineSearchJStorPrivate::jstorBaseUrl = QLatin1String("http://www.jstor.org/");
+const QString OnlineSearchJStor::OnlineSearchJStorPrivate::jstorBaseUrl = QStringLiteral("https://www.jstor.org/");
 
-OnlineSearchJStor::OnlineSearchJStor(QWidget *parent)
+OnlineSearchJStor::OnlineSearchJStor(QObject *parent)
         : OnlineSearchAbstract(parent), d(new OnlineSearchJStorPrivate(this))
 {
     /// nothing
@@ -60,57 +62,51 @@ OnlineSearchJStor::~OnlineSearchJStor()
 void OnlineSearchJStor::startSearch(const QMap<QString, QString> &query, int numResults)
 {
     m_hasBeenCanceled = false;
-    d->curStep = 0;
-    d->numSteps = 3;
+    emit progress(curStep = 0, numSteps = 3);
     d->numExpectedResults = numResults;
 
     /// Build search URL, to be used in the second step
     /// after fetching the start page
-    d->queryUrl = KUrl(OnlineSearchJStorPrivate::jstorBaseUrl);
-    d->queryUrl.setPath("/action/doAdvancedSearch");
-    d->queryUrl.addQueryItem("Search", "Search");
-    d->queryUrl.addQueryItem("wc", "on"); /// include external references, too
-    d->queryUrl.addQueryItem("la", ""); /// no specific language
-    d->queryUrl.addQueryItem("jo", ""); /// no specific journal
-    d->queryUrl.addQueryItem("hp", QString::number(numResults)); /// hits per page
+    d->queryUrl = QUrl(OnlineSearchJStorPrivate::jstorBaseUrl);
+    QUrlQuery q(d->queryUrl);
+    d->queryUrl.setPath(QStringLiteral("/action/doAdvancedSearch"));
+    q.addQueryItem(QStringLiteral("Search"), QStringLiteral("Search"));
+    q.addQueryItem(QStringLiteral("wc"), QStringLiteral("on")); /// include external references, too
+    q.addQueryItem(QStringLiteral("la"), QStringLiteral("")); /// no specific language
+    q.addQueryItem(QStringLiteral("jo"), QStringLiteral("")); /// no specific journal
+    q.addQueryItem(QStringLiteral("hp"), QString::number(numResults)); /// hits per page
     int queryNumber = 0;
-    QStringList elements = splitRespectingQuotationMarks(query[queryKeyTitle]);
-    foreach (const QString &element, elements) {
-        if (queryNumber > 0) d->queryUrl.addQueryItem(QString(QLatin1String("c%1")).arg(queryNumber), "AND"); ///< join search terms with an AND operation
-        d->queryUrl.addQueryItem(QString(QLatin1String("f%1")).arg(queryNumber), "ti");
-        d->queryUrl.addQueryItem(QString(QLatin1String("q%1")).arg(queryNumber), element);
+    const QStringList elementsTitle = splitRespectingQuotationMarks(query[queryKeyTitle]);
+    for (const QString &element : elementsTitle) {
+        if (queryNumber > 0) q.addQueryItem(QString(QStringLiteral("c%1")).arg(queryNumber), QStringLiteral("AND")); ///< join search terms with an AND operation
+        q.addQueryItem(QString(QStringLiteral("f%1")).arg(queryNumber), QStringLiteral("ti"));
+        q.addQueryItem(QString(QStringLiteral("q%1")).arg(queryNumber), element);
         ++queryNumber;
     }
-    elements = splitRespectingQuotationMarks(query[queryKeyAuthor]);
-    foreach (const QString &element, elements) {
-        if (queryNumber > 0) d->queryUrl.addQueryItem(QString(QLatin1String("c%1")).arg(queryNumber), "AND"); ///< join search terms with an AND operation
-        d->queryUrl.addQueryItem(QString(QLatin1String("f%1")).arg(queryNumber), "au");
-        d->queryUrl.addQueryItem(QString(QLatin1String("q%1")).arg(queryNumber), element);
+    const QStringList elementsAuthor = splitRespectingQuotationMarks(query[queryKeyAuthor]);
+    for (const QString &element : elementsAuthor) {
+        if (queryNumber > 0) q.addQueryItem(QString(QStringLiteral("c%1")).arg(queryNumber), QStringLiteral("AND")); ///< join search terms with an AND operation
+        q.addQueryItem(QString(QStringLiteral("f%1")).arg(queryNumber), QStringLiteral("au"));
+        q.addQueryItem(QString(QStringLiteral("q%1")).arg(queryNumber), element);
         ++queryNumber;
     }
-    elements = splitRespectingQuotationMarks(query[queryKeyFreeText]);
-    foreach (const QString &element, elements) {
-        if (queryNumber > 0) d->queryUrl.addQueryItem(QString(QLatin1String("c%1")).arg(queryNumber), "AND"); ///< join search terms with an AND operation
-        d->queryUrl.addQueryItem(QString(QLatin1String("f%1")).arg(queryNumber), "all");
-        d->queryUrl.addQueryItem(QString(QLatin1String("q%1")).arg(queryNumber), element);
+    const QStringList elementsFreeText = splitRespectingQuotationMarks(query[queryKeyFreeText]);
+    for (const QString &element : elementsFreeText) {
+        if (queryNumber > 0) q.addQueryItem(QString(QStringLiteral("c%1")).arg(queryNumber), QStringLiteral("AND")); ///< join search terms with an AND operation
+        q.addQueryItem(QString(QStringLiteral("f%1")).arg(queryNumber), QStringLiteral("all"));
+        q.addQueryItem(QString(QStringLiteral("q%1")).arg(queryNumber), element);
         ++queryNumber;
     }
     if (!query[queryKeyYear].isEmpty()) {
-        d->queryUrl.addQueryItem("sd", query[queryKeyYear]);
-        d->queryUrl.addQueryItem("ed", query[queryKeyYear]);
+        q.addQueryItem(QStringLiteral("sd"), query[queryKeyYear]);
+        q.addQueryItem(QStringLiteral("ed"), query[queryKeyYear]);
     }
+    d->queryUrl.setQuery(q);
 
     QNetworkRequest request(OnlineSearchJStorPrivate::jstorBaseUrl);
-    QNetworkReply *reply = InternalNetworkAccessManager::self()->get(request);
-    InternalNetworkAccessManager::self()->setNetworkReplyTimeout(reply);
-    connect(reply, SIGNAL(finished()), this, SLOT(doneFetchingStartPage()));
-    emit progress(d->curStep, d->numSteps);
-}
-
-void OnlineSearchJStor::startSearch()
-{
-    m_hasBeenCanceled = false;
-    delayedStoppedSearch(resultNoError);
+    QNetworkReply *reply = InternalNetworkAccessManager::instance().get(request);
+    InternalNetworkAccessManager::instance().setNetworkReplyTimeout(reply);
+    connect(reply, &QNetworkReply::finished, this, &OnlineSearchJStor::doneFetchingStartPage);
 }
 
 QString OnlineSearchJStor::label() const
@@ -120,53 +116,43 @@ QString OnlineSearchJStor::label() const
 
 QString OnlineSearchJStor::favIconUrl() const
 {
-    return QLatin1String("http://www.jstor.org/assets/search_20151218T0921/files/search/images/favicon.ico");
+    return QStringLiteral("https://www.jstor.org/assets/search_20151218T0921/files/search/images/favicon.ico");
 }
 
-OnlineSearchQueryFormAbstract *OnlineSearchJStor::customWidget(QWidget *)
+QUrl OnlineSearchJStor::homepage() const
 {
-    return NULL;
-}
-
-KUrl OnlineSearchJStor::homepage() const
-{
-    return KUrl("http://www.jstor.org/");
-}
-
-void OnlineSearchJStor::cancel()
-{
-    OnlineSearchAbstract::cancel();
+    return QUrl(QStringLiteral("https://www.jstor.org/"));
 }
 
 void OnlineSearchJStor::doneFetchingStartPage()
 {
-    emit progress(++d->curStep, d->numSteps);
+    emit progress(++curStep, numSteps);
 
     QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
 
-    KUrl redirUrl;
+    QUrl redirUrl;
     if (handleErrors(reply, redirUrl)) {
         if (redirUrl.isValid()) {
-            ++d->numSteps;
+            ++numSteps;
 
             /// redirection to another url
             QNetworkRequest request(redirUrl);
-            QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request, reply->url());
-            InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
-            connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingStartPage()));
+            QNetworkReply *newReply = InternalNetworkAccessManager::instance().get(request, reply->url());
+            InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
+            connect(newReply, &QNetworkReply::finished, this, &OnlineSearchJStor::doneFetchingStartPage);
         } else {
             QNetworkRequest request(d->queryUrl);
-            QNetworkReply *newReply = InternalNetworkAccessManager::self()->get(request);
-            InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
-            connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingResultPage()));
+            QNetworkReply *newReply = InternalNetworkAccessManager::instance().get(request);
+            InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
+            connect(newReply, &QNetworkReply::finished, this, &OnlineSearchJStor::doneFetchingResultPage);
         }
     } else
-        kDebug() << "url was" << reply->url().toString();
+        qCWarning(LOG_KBIBTEX_NETWORKING) << "url was" << reply->url().toDisplayString();
 }
 
 void OnlineSearchJStor::doneFetchingResultPage()
 {
-    emit progress(++d->curStep, d->numSteps);
+    emit progress(++curStep, numSteps);
 
     QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
 
@@ -188,31 +174,31 @@ void OnlineSearchJStor::doneFetchingResultPage()
 
         if (uniqueDOIs.isEmpty()) {
             /// No results found
-            emit progress(d->numSteps, d->numSteps);
-            emit stoppedSearch(resultNoError);
+            emit progress(curStep = numSteps, numSteps);
+            stopSearch(resultNoError);
         } else {
             /// Build POST request that should return a BibTeX document
             QString body;
             int numResults = 0;
             for (QStringList::ConstIterator it = uniqueDOIs.constBegin(); numResults < d->numExpectedResults && it != uniqueDOIs.constEnd(); ++it, ++numResults) {
-                if (!body.isEmpty()) body.append(QLatin1String("&"));
-                body.append(QLatin1String("citations=") + encodeURL(*it));
+                if (!body.isEmpty()) body.append(QStringLiteral("&"));
+                body.append(QStringLiteral("citations=") + encodeURL(*it));
             }
-            KUrl bibTeXUrl = KUrl(OnlineSearchJStorPrivate::jstorBaseUrl);
-            bibTeXUrl.setPath("/citation/bulk/text");
+            QUrl bibTeXUrl = QUrl(OnlineSearchJStorPrivate::jstorBaseUrl);
+            bibTeXUrl.setPath(QStringLiteral("/citation/bulk/text"));
             QNetworkRequest request(bibTeXUrl);
             request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
-            QNetworkReply *newReply = InternalNetworkAccessManager::self()->post(request, body.toUtf8());
-            InternalNetworkAccessManager::self()->setNetworkReplyTimeout(newReply);
-            connect(newReply, SIGNAL(finished()), this, SLOT(doneFetchingBibTeXCode()));
+            QNetworkReply *newReply = InternalNetworkAccessManager::instance().post(request, body.toUtf8());
+            InternalNetworkAccessManager::instance().setNetworkReplyTimeout(newReply);
+            connect(newReply, &QNetworkReply::finished, this, &OnlineSearchJStor::doneFetchingBibTeXCode);
         }
     } else
-        kDebug() << "url was" << reply->url().toString();
+        qCWarning(LOG_KBIBTEX_NETWORKING) << "url was" << reply->url().toDisplayString();
 }
 
 void OnlineSearchJStor::doneFetchingBibTeXCode()
 {
-    emit progress(++d->curStep, d->numSteps);
+    emit progress(++curStep, numSteps);
 
     QNetworkReply *reply = static_cast<QNetworkReply *>(sender());
 
@@ -220,23 +206,22 @@ void OnlineSearchJStor::doneFetchingBibTeXCode()
         /// ensure proper treatment of UTF-8 characters
         const QString bibTeXcode = QString::fromUtf8(reply->readAll().constData());
 
-        FileImporterBibTeX importer;
+        FileImporterBibTeX importer(this);
         File *bibtexFile = importer.fromString(bibTeXcode);
         int numFoundResults = 0;
-        if (bibtexFile != NULL) {
-            for (File::ConstIterator it = bibtexFile->constBegin(); it != bibtexFile->constEnd(); ++it) {
-                QSharedPointer<Entry> entry = (*it).dynamicCast<Entry>();
+        if (bibtexFile != nullptr) {
+            for (const auto &element : const_cast<const File &>(*bibtexFile)) {
+                QSharedPointer<Entry> entry = element.dynamicCast<Entry>();
                 if (publishEntry(entry))
                     ++numFoundResults;
-
             }
             delete bibtexFile;
         }
 
-        emit progress(d->numSteps, d->numSteps);
-        emit stoppedSearch(numFoundResults > 0 ? resultNoError : resultUnspecifiedError);
+        emit progress(curStep = numSteps, numSteps);
+        stopSearch(numFoundResults > 0 ? resultNoError : resultUnspecifiedError);
     } else
-        kDebug() << "url was" << reply->url().toString();
+        qCWarning(LOG_KBIBTEX_NETWORKING) << "url was" << reply->url().toDisplayString();
 }
 
 void OnlineSearchJStor::sanitizeEntry(QSharedPointer<Entry> entry)
@@ -251,16 +236,16 @@ void OnlineSearchJStor::sanitizeEntry(QSharedPointer<Entry> entry)
     }
 
     QString url = PlainTextValue::text(entry->value(Entry::ftUrl));
-    if (url.startsWith(QLatin1String("http://www.jstor.org/stable/"))) {
+    if (url.startsWith(QStringLiteral("https://www.jstor.org/stable/"))) {
         /// use JSTOR's own stable ID for entry ID
         entry->setId("jstor" + url.mid(28).replace(QLatin1Char(','), QString()));
         /// store JSTOR's own stable ID
         Value v;
         v.append(QSharedPointer<VerbatimText>(new VerbatimText(url.mid(28))));
-        entry->insert(QLatin1String("jstor_id"), v);
+        entry->insert(QStringLiteral("jstor_id"), v);
     }
 
-    const QString formattedDateKey = QLatin1String("jstor_formatteddate");
+    const QString formattedDateKey = QStringLiteral("jstor_formatteddate");
     const QString formattedDate = PlainTextValue::text(entry->value(formattedDateKey));
 
     /// first, try to guess month by inspecting the beginning
@@ -268,27 +253,27 @@ void OnlineSearchJStor::sanitizeEntry(QSharedPointer<Entry> entry)
     const QString formattedDateLower = formattedDate.toLower();
     int i;
     for (i = 0; i < 12; ++i)
-        if (formattedDateLower.startsWith(MonthsTriple[i])) break;
+        if (formattedDateLower.startsWith(KBibTeX::MonthsTriple[i])) break;
     entry->remove(formattedDateKey);
     if (i < 12) {
         Value v;
-        v.append(QSharedPointer<MacroKey>(new MacroKey(MonthsTriple[i])));
+        v.append(QSharedPointer<MacroKey>(new MacroKey(KBibTeX::MonthsTriple[i])));
         entry->insert(Entry::ftMonth, v);
     }
     /// guessing failed, therefore extract first part if it exists
-    else if ((i = formattedDate.indexOf(",")) >= 0) {
+    else if ((i = formattedDate.indexOf(QStringLiteral(","))) >= 0) {
         /// text may be a season ("Winter")
         Value v;
         v.append(QSharedPointer<PlainText>(new PlainText(formattedDate.left(i))));
         entry->insert(Entry::ftMonth, v);
     } else {
         /// this case happens if the field only contains a year
-        kDebug() << "Cannot extract month/season from date" << formattedDate;
+        //qCDebug(LOG_KBIBTEX_NETWORKING) << "Cannot extract month/season from date" << formattedDate;
     }
 
     /// page field may start with "pp. ", remove that
     QString pages = PlainTextValue::text(entry->value(Entry::ftPages)).toLower();
-    if (pages.startsWith(QLatin1String("pp. "))) {
+    if (pages.startsWith(QStringLiteral("pp. "))) {
         pages = pages.mid(4);
         entry->remove(Entry::ftPages);
         Value v;
@@ -296,6 +281,13 @@ void OnlineSearchJStor::sanitizeEntry(QSharedPointer<Entry> entry)
         entry->insert(Entry::ftPages, v);
     }
 
+    /// Some entries may have no 'author' field, but a 'bookauthor' field instead
+    if (!entry->contains(Entry::ftAuthor) && entry->contains(QStringLiteral("bookauthor"))) {
+        Value v = entry->value(QStringLiteral("bookauthor"));
+        entry->remove(QStringLiteral("bookauthor"));
+        entry->insert(Entry::ftAuthor, v);
+    }
+
     for (QMap<QString, Value>::Iterator it = entry->begin(); it != entry->end();) {
         if (PlainTextValue::text(it.value()).isEmpty())
             it = entry->erase(it);
