@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,17 +12,16 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "api.h"
 
+#include <QUrl>
+#include <QUrlQuery>
 #include <QDateTime>
 #include <QTimer>
 
-#include <KUrl>
-#include <KDebug>
-
 using namespace Zotero;
 
 class Zotero::API::Private
@@ -31,7 +30,7 @@ private:
     // UNUSED Zotero::API *p;
 
 public:
-    const KUrl apiBaseUrl;
+    const QUrl apiBaseUrl;
     const int userOrGroupPrefix;
     const QString apiKey;
 
@@ -39,7 +38,7 @@ public:
 
     Private(RequestScope requestScope, int prefix, const QString &_apiKey, Zotero::API */* UNUSED parent*/)
         : // UNUSED p(parent),
-          apiBaseUrl(KUrl(QString(QLatin1String("https://api.zotero.org/%1/%2%3")).arg(requestScope == GroupRequest ? QLatin1String("groups") : QLatin1String("users")).arg(prefix).arg(_apiKey.isEmpty() ? QString() : QString(QLatin1String("?key=%1")).arg(_apiKey)))),
+          apiBaseUrl(QUrl(QString(QStringLiteral("https://api.zotero.org/%1/%2")).arg(requestScope == GroupRequest ? QStringLiteral("groups") : QStringLiteral("users")).arg(prefix))),
           userOrGroupPrefix(prefix),
           apiKey(_apiKey), backoffElapseTime(QDateTime::currentDateTime().addSecs(-5)) {
         /// nothing
@@ -59,23 +58,16 @@ API::~API()
     delete d;
 }
 
-void API::addLimitToUrl(KUrl &url) const
+void API::addLimitToUrl(QUrl &url) const
 {
-    static const QString limitKey = QLatin1String("limit");
-    url.removeQueryItem(limitKey);
-    url.addQueryItem(limitKey, QString::number(Zotero::API::limit));
+    static const QString limitKey = QStringLiteral("limit");
+    QUrlQuery query(url);
+    query.removeQueryItem(limitKey);
+    query.addQueryItem(limitKey, QString::number(Zotero::API::limit));
+    url.setQuery(query);
 }
 
-void API::addKeyToUrl(KUrl &url) const
-{
-    if (!d->apiKey.isEmpty()) {
-        static const QString keyKey = QLatin1String("key");
-        url.removeQueryItem(keyKey);
-        url.addQueryItem(keyKey, d->apiKey);
-    }
-}
-
-KUrl API::baseUrl() const
+QUrl API::baseUrl() const
 {
     return d->apiBaseUrl;
 }
@@ -85,12 +77,12 @@ int API::userOrGroupPrefix() const
     return d->userOrGroupPrefix;
 }
 
-QNetworkRequest API::request(const KUrl &url) const
+QNetworkRequest API::request(const QUrl &url) const
 {
     QNetworkRequest request(url);
     request.setRawHeader("Zotero-API-Version", "3");
     request.setRawHeader("Accept", "application/atom+xml");
-    request.setRawHeader("Authorization", QString(QLatin1String("Bearer ")).append(d->apiKey).toAscii().constData());
+    request.setRawHeader("Authorization", QString(QStringLiteral("Bearer ")).append(d->apiKey).toLatin1().constData());
     return request;
 }
 
@@ -100,14 +92,12 @@ void API::startBackoff(int duration) {
         emit backoffModeStart();
         /// Use single-shot timer and functor to emit signal
         /// that backoff mode has finished
-        QTimer::singleShot((duration + 1) * 1000, this, SLOT(startBackoffSingleShot()));
+        QTimer::singleShot((duration + 1) * 1000, this, [ = ]() {
+            emit backoffModeEnd();
+        });
     }
 }
 
-void API::startBackoffSingleShot() {
-    emit backoffModeEnd();
-}
-
 bool API::inBackoffMode() const {
     return d->backoffElapseTime >= QDateTime::currentDateTime();
 }
