@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,50 +12,61 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "documentpreview.h"
 
 #include <typeinfo>
 
+#include <QDomDocument>
+#include <QDomElement>
 #include <QList>
 #include <QLayout>
 #include <QMap>
 #include <QFileInfo>
 #include <QResizeEvent>
 #include <QCheckBox>
+#include <QMenuBar>
 #include <QStackedWidget>
 #include <QDockWidget>
+#include <QDebug>
+#include <QPushButton>
 #include <QMutex>
-#ifdef HAVE_QTWEBKIT // krazy:exclude=cpp
+#include <QMimeDatabase>
+#include <QMimeType>
+#include <QIcon>
+#ifdef HAVE_WEBENGINEWIDGETS
+#include <QWebEngineView>
+#else // HAVE_WEBENGINEWIDGETS
+#ifdef HAVE_WEBKITWIDGETS
 #include <QWebView>
-#endif // HAVE_QTWEBKIT
+#endif // HAVE_WEBKITWIDGETS
+#endif // HAVE_WEBENGINEWIDGETS
 
-#include <KLocale>
+#include <KLocalizedString>
 #include <KComboBox>
-#include <KIcon>
+#include <KJobWidgets>
 #include <KRun>
-#include <KMimeType>
 #include <KMimeTypeTrader>
 #include <KService>
-#include <kparts/part.h>
-#include <KDebug>
-#include <kio/netaccess.h>
-#include <KPushButton>
+#include <KParts/Part>
+#include <KParts/ReadOnlyPart>
 #include <kio/jobclasses.h>
 #include <kio/job.h>
 #include <kio/jobuidelegate.h>
-#include <KTemporaryFile>
-#include <KMenuBar>
 #include <KToolBar>
 #include <KActionCollection>
+#include <KSharedConfig>
+#include <KConfigGroup>
+#include <kio_version.h>
 
-#include "kbibtexnamespace.h"
+#include "kbibtex.h"
 #include "element.h"
 #include "entry.h"
 #include "file.h"
 #include "fileinfo.h"
+#include "logging_program.h"
 
 ImageLabel::ImageLabel(const QString &text, QWidget *parent, Qt::WindowFlags f)
         : QLabel(text, parent, f)
@@ -92,9 +103,9 @@ class DocumentPreview::DocumentPreviewPrivate
 {
 public:
     struct UrlInfo {
-        KUrl url;
+        QUrl url;
         QString mimeType;
-        KIcon icon;
+        QIcon icon;
     };
 
 private:
@@ -104,7 +115,7 @@ private:
     static const QString configGroupName;
     static const QString onlyLocalFilesCheckConfig;
 
-    KPushButton *externalViewerButton;
+    QPushButton *externalViewerButton;
     QStackedWidget *stackedWidget;
     ImageLabel *message;
     QMap<int, struct UrlInfo> cbxEntryToUrlInfo;
@@ -113,36 +124,40 @@ private:
     static const QString arXivPDFUrlStart;
     bool anyLocal;
 
-    KMenuBar *menuBar;
+    QMenuBar *menuBar;
     KToolBar *toolBar;
     KParts::ReadOnlyPart *okularPart;
-#ifdef HAVE_QTWEBKIT // krazy:exclude=cpp
+#ifdef HAVE_WEBENGINEWIDGETS
+    QWebEngineView *htmlWidget;
+#else // HAVE_WEBENGINEWIDGETS
+#ifdef HAVE_WEBKITWIDGETS
     QWebView *htmlWidget;
-#else // HAVE_QTWEBKIT
+#else // HAVE_WEBKITWIDGETS
     KParts::ReadOnlyPart *htmlPart;
-#endif // HAVE_QTWEBKIT
+#endif // HAVE_WEBKITWIDGETS
+#endif // HAVE_WEBENGINEWIDGETS
     int swpMessage, swpOkular, swpHTML;
 
 public:
     KComboBox *urlComboBox;
-    KPushButton *onlyLocalFilesButton;
+    QPushButton *onlyLocalFilesButton;
     QList<KIO::StatJob *> runningJobs;
     QSharedPointer<const Entry> entry;
-    KUrl baseUrl;
+    QUrl baseUrl;
     bool anyRemote;
 
-    KParts::ReadOnlyPart *locatePart(const QString &desktopFile, QWidget *parentWidget) {
-        KService::Ptr service = KService::serviceByDesktopPath(desktopFile);
-        if (!service.isNull()) {
+    KParts::ReadOnlyPart *locatePart(const QString &mimeType, QWidget *parentWidget) {
+        KService::Ptr service = KMimeTypeTrader::self()->preferredService(mimeType, QStringLiteral("KParts/ReadOnlyPart"));
+        if (service) {
             KParts::ReadOnlyPart *part = service->createInstance<KParts::ReadOnlyPart>(parentWidget, p);
-            connect(part, SIGNAL(completed()), p, SLOT(loadingFinished()));
+            connect(part, static_cast<void(KParts::ReadOnlyPart::*)()>(&KParts::ReadOnlyPart::completed), p, &DocumentPreview::loadingFinished);
             return part;
         } else
-            return NULL;
+            return nullptr;
     }
 
     DocumentPreviewPrivate(DocumentPreview *parent)
-            : p(parent), config(KSharedConfig::openConfig(QLatin1String("kbibtexrc"))), anyLocal(false), entry(NULL), anyRemote(false) {
+            : p(parent), config(KSharedConfig::openConfig(QStringLiteral("kbibtexrc"))), anyLocal(false), entry(nullptr), anyRemote(false) {
         setupGUI();
     }
 
@@ -162,7 +177,7 @@ public:
         QHBoxLayout *innerLayout = new QHBoxLayout();
         layout->addLayout(innerLayout, 0);
 
-        onlyLocalFilesButton = new KPushButton(KIcon("applications-internet"), QString(), p);
+        onlyLocalFilesButton = new QPushButton(QIcon::fromTheme(QStringLiteral("applications-internet")), QString(), p);
         onlyLocalFilesButton->setToolTip(i18n("Toggle between local files only and all documents including remote ones"));
         innerLayout->addWidget(onlyLocalFilesButton, 0);
         onlyLocalFilesButton->setCheckable(true);
@@ -173,14 +188,14 @@ public:
         urlComboBox = new KComboBox(false, p);
         innerLayout->addWidget(urlComboBox, 1);
 
-        externalViewerButton = new KPushButton(KIcon("document-open"), QString(), p);
+        externalViewerButton = new QPushButton(QIcon::fromTheme(QStringLiteral("document-open")), QString(), p);
         externalViewerButton->setToolTip(i18n("Open in external program"));
         innerLayout->addWidget(externalViewerButton, 0);
         sp = externalViewerButton->sizePolicy();
         sp.setVerticalPolicy(QSizePolicy::MinimumExpanding);
         externalViewerButton->setSizePolicy(sp);
 
-        menuBar = new KMenuBar(p);
+        menuBar = new QMenuBar(p);
         menuBar->setBackgroundRole(QPalette::Window);
         menuBar->setVisible(false);
         layout->addWidget(menuBar, 0);
@@ -201,34 +216,54 @@ public:
         message->setAlignment(Qt::AlignCenter);
         message->setWordWrap(true);
         swpMessage = stackedWidget->addWidget(message);
-        connect(message, SIGNAL(linkActivated(QString)), p, SLOT(linkActivated(QString)));
+        connect(message, &QLabel::linkActivated, p, &DocumentPreview::linkActivated);
 
         /// add parts to stackedWidget
-        okularPart = locatePart(QLatin1String("okularPoppler.desktop"), stackedWidget);
-        swpOkular = (okularPart == NULL) ? -1 : stackedWidget->addWidget(okularPart->widget());
-        if (okularPart == NULL || swpOkular < 0) {
-            kWarning() << "No 'KDE 4'-based Okular part for PDF or PostScript document preview available.";
+        okularPart = locatePart(QStringLiteral("application/pdf"), stackedWidget);
+        swpOkular = (okularPart == nullptr) ? -1 : stackedWidget->addWidget(okularPart->widget());
+        if (okularPart == nullptr || swpOkular < 0) {
+            qCWarning(LOG_KBIBTEX_PROGRAM) << "No 'KDE Framworks 5'-based Okular part for PDF or PostScript document preview available.";
         }
-#ifdef HAVE_QTWEBKIT // krazy:exclude=cpp
-        kDebug() << "WebKit is available, using it instead of KHTML for HTML/Web preview.";
+#ifdef HAVE_WEBENGINEWIDGETS
+        qCDebug(LOG_KBIBTEX_PROGRAM) << "WebEngine is available, using it instead of WebKit or HTML KPart (both neither considered nor tested for) for HTML/Web preview.";
+        /// To make DrKonqi handle crashes in Chromium-based QtWebEngine,
+        /// set a certain environment variable. For details, see here:
+        /// https://www.dvratil.cz/2018/10/drkonqi-and-qtwebengine/
+        /// https://phabricator.kde.org/D16004
+        const auto chromiumFlags = qgetenv("QTWEBENGINE_CHROMIUM_FLAGS");
+        if (!chromiumFlags.contains("disable-in-process-stack-traces")) {
+            qputenv("QTWEBENGINE_CHROMIUM_FLAGS", chromiumFlags + " --disable-in-process-stack-traces");
+        }
+        htmlWidget = new QWebEngineView(stackedWidget);
+        swpHTML = stackedWidget->addWidget(htmlWidget);
+        connect(htmlWidget, &QWebEngineView::loadFinished, p, &DocumentPreview::loadingFinished);
+#else // HAVE_WEBENGINEWIDGETS
+#ifdef HAVE_WEBKITWIDGETS
+        qCDebug(LOG_KBIBTEX_PROGRAM) << "WebKit is available, using it instead of WebEngine (missing) or HTML KPart (not considered) for HTML/Web preview.";
         htmlWidget = new QWebView(stackedWidget);
         swpHTML = stackedWidget->addWidget(htmlWidget);
-        connect(htmlWidget, SIGNAL(loadFinished(bool)), p, SLOT(loadingFinished()));
-#else // HAVE_QTWEBKIT
-        kDebug() << "WebKit not is available, using KHTML instead for HTML/Web preview.";
-        htmlPart = locatePart(QLatin1String("khtml.desktop"), stackedWidget);
-        swpHTML = stackedWidget->addWidget(htmlPart->widget());
-#endif // HAVE_QTWEBKIT
+        connect(htmlWidget, &QWebView::loadFinished, p, &DocumentPreview::loadingFinished);
+#else // HAVE_WEBKITWIDGETS
+        htmlPart = locatePart(QStringLiteral("text/html"), stackedWidget);
+        if (htmlPart != nullptr) {
+            qCDebug(LOG_KBIBTEX_PROGRAM) << "HTML KPart is available, using it instead of WebEngine or WebKit (neither available) for HTML/Web preview.";
+            swpHTML = stackedWidget->addWidget(htmlPart->widget());
+        } else {
+            qCDebug(LOG_KBIBTEX_PROGRAM) << "No HTML viewing component is available, disabling HTML/Web preview.";
+            swpHTML = -1;
+        }
+#endif // HAVE_WEBKITWIDGETS
+#endif // HAVE_WEBENGINEWIDGETS
 
         loadState();
 
-        connect(externalViewerButton, SIGNAL(clicked()), p, SLOT(openExternally()));
-        connect(urlComboBox, SIGNAL(activated(int)), p, SLOT(comboBoxChanged(int)));
-        connect(onlyLocalFilesButton, SIGNAL(toggled(bool)), p, SLOT(onlyLocalFilesChanged()));
+        connect(externalViewerButton, &QPushButton::clicked, p, &DocumentPreview::openExternally);
+        connect(urlComboBox, static_cast<void(QComboBox::*)(int)>(&QComboBox::activated), p, &DocumentPreview::comboBoxChanged);
+        connect(onlyLocalFilesButton, &QPushButton::toggled, p, &DocumentPreview::onlyLocalFilesChanged);
     }
 
     bool addUrl(const struct UrlInfo &urlInfo) {
-        bool isLocal = isLocalOrRelative(urlInfo.url);
+        bool isLocal = KBibTeX::isLocalOrRelative(urlInfo.url);
         anyLocal |= isLocal;
 
         if (!onlyLocalFilesButton->isChecked() && !isLocal) return true; ///< ignore URL if only local files are allowed
@@ -237,13 +272,13 @@ public:
             /// create a drop-down list entry if file is a local file
             /// (based on patch by Luis Silva)
             QString fn = urlInfo.url.fileName();
-            QString full = urlInfo.url.pathOrUrl();
-            QString dir = urlInfo.url.directory();
-            QString text = fn.isEmpty() ? full : (dir.isEmpty() ? fn : QString(QLatin1String("%1 [%2]")).arg(fn, dir));
+            QString full = urlInfo.url.url(QUrl::PreferLocalFile);
+            QString dir = urlInfo.url.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash).path();
+            QString text = fn.isEmpty() ? full : (dir.isEmpty() ? fn : QString(QStringLiteral("%1 [%2]")).arg(fn, dir));
             urlComboBox->addItem(urlInfo.icon, text);
         } else {
             /// create a drop-down list entry if file is a remote file
-            urlComboBox->addItem(urlInfo.icon, urlInfo.url.prettyUrl());
+            urlComboBox->addItem(urlInfo.icon, urlInfo.url.toDisplayString());
         }
         urlComboBox->setEnabled(true);
         cbxEntryToUrlInfo.insert(urlComboBox->count() - 1, urlInfo);
@@ -252,7 +287,7 @@ public:
         if (urlComboBox->count() == 1 || ///< first entry in combobox
                 isLocal || ///< local files always preferred over URLs
                 /// prefer arXiv summary URLs over other URLs
-                (!anyLocal && urlInfo.url.host().contains("arxiv.org/abs"))) {
+                (!anyLocal && urlInfo.url.host().contains(QStringLiteral("arxiv.org/abs")))) {
             showUrl(urlInfo);
         }
 
@@ -263,13 +298,18 @@ public:
         p->setCursor(Qt::WaitCursor);
 
         /// reset and clear all controls
-        if (okularPart != NULL)
+        if (swpOkular >= 0 && okularPart != nullptr)
             okularPart->closeUrl();
-#ifdef HAVE_QTWEBKIT // krazy:exclude=cpp
+#ifdef HAVE_WEBENGINEWIDGETS
         htmlWidget->stop();
-#else // HAVE_QTWEBKIT
-        htmlPart->closeUrl();
-#endif // HAVE_QTWEBKIT
+#else // HAVE_WEBENGINEWIDGETS
+#ifdef HAVE_WEBKITWIDGETS
+        htmlWidget->stop();
+#else // HAVE_WEBKITWIDGETS
+        if (swpHTML >= 0 && htmlPart != nullptr)
+            htmlPart->closeUrl();
+#endif // HAVE_WEBKITWIDGETS
+#endif // HAVE_WEBENGINEWIDGETS
         urlComboBox->setEnabled(false);
         urlComboBox->clear();
         cbxEntryToUrlInfo.clear();
@@ -277,9 +317,11 @@ public:
         showMessage(i18n("Refreshing...")); // krazy:exclude=qmethods
 
         /// cancel/kill all running jobs
-        for (QList<KIO::StatJob *>::ConstIterator it = runningJobs.constBegin(); it != runningJobs.constEnd(); ++it)
+        auto it = runningJobs.begin();
+        while (it != runningJobs.end()) {
             (*it)->kill();
-        runningJobs.clear();
+            it = runningJobs.erase(it);
+        }
 
         /// clear flag that memorizes if any local file was referenced
         anyLocal = false;
@@ -287,17 +329,16 @@ public:
 
         /// do not load external reference if widget is hidden
         if (isVisible()) {
-            QList<KUrl> urlList = FileInfo::entryUrls(entry.data(), baseUrl, FileInfo::TestExistenceYes);
-
-            for (QList<KUrl>::ConstIterator it = urlList.constBegin(); it != urlList.constEnd(); ++it) {
-                bool isLocal = isLocalOrRelative(*it);
+            const QList<QUrl> urlList = FileInfo::entryUrls(entry, baseUrl, FileInfo::TestExistenceYes);
+            for (const QUrl &url : urlList) {
+                bool isLocal = KBibTeX::isLocalOrRelative(url);
                 anyRemote |= !isLocal;
                 if (!onlyLocalFilesButton->isChecked() && !isLocal) continue;
 
-                KIO::StatJob *job = KIO::stat(*it, KIO::StatJob::SourceSide, 3, KIO::HideProgressInfo);
+                KIO::StatJob *job = KIO::stat(url, KIO::StatJob::SourceSide, 3, KIO::HideProgressInfo);
                 runningJobs << job;
-                job->ui()->setWindow(p);
-                connect(job, SIGNAL(result(KJob*)), p, SLOT(statFinished(KJob*)));
+                KJobWidgets::setWindow(job, p);
+                connect(job, &KIO::StatJob::result, p, &DocumentPreview::statFinished);
             }
             if (urlList.isEmpty()) {
                 /// Case no URLs associated with this entry.
@@ -322,7 +363,8 @@ public:
         message->setText(msgText);
         if (swpOkular >= 0)
             stackedWidget->widget(swpOkular)->setEnabled(false);
-        stackedWidget->widget(swpHTML)->setEnabled(false);
+        if (swpHTML >= 0)
+            stackedWidget->widget(swpHTML)->setEnabled(false);
         menuBar->setVisible(false);
         toolBar->setVisible(true);
         menuBar->clear();
@@ -340,32 +382,32 @@ public:
         QDomDocument doc = part->domDocument();
         QDomElement docElem = doc.documentElement();
 
-        QDomNodeList toolbarNodes = docElem.elementsByTagName("ToolBar");
+        QDomNodeList toolbarNodes = docElem.elementsByTagName(QStringLiteral("ToolBar"));
         for (int i = 0; i < toolbarNodes.count(); ++i) {
             QDomNodeList toolbarItems = toolbarNodes.at(i).childNodes();
             for (int j = 0; j < toolbarItems.count(); ++j) {
                 QDomNode toolbarItem = toolbarItems.at(j);
-                if (toolbarItem.nodeName() == QLatin1String("Action")) {
-                    QString actionName = toolbarItem.attributes().namedItem(QLatin1String("name")).nodeValue();
+                if (toolbarItem.nodeName() == QStringLiteral("Action")) {
+                    QString actionName = toolbarItem.attributes().namedItem(QStringLiteral("name")).nodeValue();
                     toolBar->addAction(part->actionCollection()->action(actionName));
-                } else if (toolbarItem.nodeName() == QLatin1String("Separator")) {
+                } else if (toolbarItem.nodeName() == QStringLiteral("Separator")) {
                     toolBar->addSeparator();
                 }
             }
         }
 
 
-        QDomNodeList menubarNodes = docElem.elementsByTagName("MenuBar");
+        QDomNodeList menubarNodes = docElem.elementsByTagName(QStringLiteral("MenuBar"));
         for (int i = 0; i < menubarNodes.count(); ++i) {
             QDomNodeList menubarNode = menubarNodes.at(i).childNodes();
             for (int j = 0; j < menubarNode.count(); ++j) {
                 QDomNode menubarItem = menubarNode.at(j);
-                if (menubarItem.nodeName() == QLatin1String("Menu")) {
+                if (menubarItem.nodeName() == QStringLiteral("Menu")) {
                     QDomNodeList menuNode = menubarItem.childNodes();
                     QString text;
                     for (int k = 0; k < menuNode.count(); ++k) {
                         QDomNode menuItem = menuNode.at(k);
-                        if (menuItem.nodeName() == QLatin1String("text")) {
+                        if (menuItem.nodeName() == QStringLiteral("text")) {
                             text = menuItem.firstChild().toText().data();
                             break;
                         }
@@ -374,10 +416,10 @@ public:
 
                     for (int k = 0; k < menuNode.count(); ++k) {
                         QDomNode menuItem = menuNode.at(k);
-                        if (menuItem.nodeName() == QLatin1String("Action")) {
-                            QString actionName = menuItem.attributes().namedItem(QLatin1String("name")).nodeValue();
+                        if (menuItem.nodeName() == QStringLiteral("Action")) {
+                            QString actionName = menuItem.attributes().namedItem(QStringLiteral("name")).nodeValue();
                             menu->addAction(part->actionCollection()->action(actionName));
-                        } else if (menuItem.nodeName() == QLatin1String("Separator")) {
+                        } else if (menuItem.nodeName() == QStringLiteral("Separator")) {
                             menu->addSeparator();
                         }
                     }
@@ -385,19 +427,18 @@ public:
             }
         }
 
-        QDomNodeList actionPropertiesList = docElem.elementsByTagName("ActionProperties");
+        QDomNodeList actionPropertiesList = docElem.elementsByTagName(QStringLiteral("ActionProperties"));
         for (int i = 0; i < actionPropertiesList.count(); ++i) {
             QDomNodeList actionProperties = actionPropertiesList.at(i).childNodes();
             for (int j = 0; j < actionProperties.count(); ++j) {
                 QDomNode actionNode = actionProperties.at(j);
-                if (actionNode.nodeName() == QLatin1String("Action")) {
-                    const QString actionName = actionNode.attributes().namedItem("name").toAttr().nodeValue();
-                    const QString actionShortcut = actionNode.attributes().namedItem("shortcut").toAttr().value();
+                if (actionNode.nodeName() == QStringLiteral("Action")) {
+                    const QString actionName = actionNode.attributes().namedItem(QStringLiteral("name")).toAttr().nodeValue();
+                    const QString actionShortcut = actionNode.attributes().namedItem(QStringLiteral("shortcut")).toAttr().value();
                     QAction *action = part->actionCollection()->action(actionName);
-                    if (action != NULL) {
+                    if (action != nullptr) {
                         action->setShortcut(QKeySequence(actionShortcut));
-                    } else
-                        kDebug() << "Could not locate an action with name " << actionName << "for shortcut" << actionShortcut;
+                    }
                 }
             }
         }
@@ -413,20 +454,26 @@ public:
         menuBar->clear();
         toolBar->clear();
 
-        if (part == okularPart && swpOkular >= 0) {
+        if (okularPart != nullptr && part == okularPart && swpOkular >= 0) {
             stackedWidget->setCurrentIndex(swpOkular);
             stackedWidget->widget(swpOkular)->setEnabled(true);
             setupToolMenuBarForPart(okularPart);
-#ifdef HAVE_QTWEBKIT // krazy:exclude=cpp
+#ifdef HAVE_WEBENGINEWIDGETS
         } else if (widget == htmlWidget) {
             stackedWidget->setCurrentIndex(swpHTML);
             stackedWidget->widget(swpHTML)->setEnabled(true);
-#else // HAVE_QTWEBKIT
-        } else if (part == htmlPart) {
+#else // HAVE_WEBENGINEWIDGETS
+#ifdef HAVE_WEBKITWIDGETS
+        } else if (widget == htmlWidget) {
+            stackedWidget->setCurrentIndex(swpHTML);
+            stackedWidget->widget(swpHTML)->setEnabled(true);
+#else // HAVE_WEBKITWIDGETS
+        } else if (htmlPart != nullptr && part == htmlPart && swpHTML >= 0) {
             stackedWidget->setCurrentIndex(swpHTML);
             stackedWidget->widget(swpHTML)->setEnabled(true);
             setupToolMenuBarForPart(htmlPart);
-#endif // HAVE_QTWEBKIT
+#endif // HAVE_WEBKITWIDGETS
+#endif // HAVE_WEBENGINEWIDGETS
         } else if (widget == message) {
             stackedWidget->setCurrentIndex(swpMessage);
         } else
@@ -434,44 +481,55 @@ public:
     }
 
     bool showUrl(const struct UrlInfo &urlInfo) {
-        static const QStringList okularMimetypes = QStringList() << QLatin1String("application/x-pdf") << QLatin1String("application/pdf") << QLatin1String("application/x-gzpdf") << QLatin1String("application/x-bzpdf") << QLatin1String("application/x-wwf") << QLatin1String("image/vnd.djvu") << QLatin1String("application/postscript") << QLatin1String("image/x-eps") << QLatin1String("application/x-gzpostscript") << QLatin1String("application/x-bzpostscript") << QLatin1String("image/x-gzeps") << QLatin1String("image/x-bzeps");
-        static const QStringList htmlMimetypes = QStringList() << QLatin1String("text/html") << QLatin1String("application/xml") << QLatin1String("application/xhtml+xml");
-        static const QStringList imageMimetypes = QStringList() << QLatin1String("image/jpeg") << QLatin1String("image/png") << QLatin1String("image/gif") << QLatin1String("image/tiff");
+        static const QStringList okularMimetypes = QStringList() << QStringLiteral("application/x-pdf") << QStringLiteral("application/pdf") << QStringLiteral("application/x-gzpdf") << QStringLiteral("application/x-bzpdf") << QStringLiteral("application/x-wwf") << QStringLiteral("image/vnd.djvu") << QStringLiteral("image/vnd.djvu+multipage") << QStringLiteral("application/postscript") << QStringLiteral("image/x-eps") << QStringLiteral("application/x-gzpostscript") << QStringLiteral("application/x-bzpostscript") << QStringLiteral("image/x-gzeps") << QStringLiteral("image/x-bzeps");
+        static const QStringList htmlMimetypes = QStringList() << QStringLiteral("text/html") << QStringLiteral("application/xml") << QStringLiteral("application/xhtml+xml");
+        static const QStringList imageMimetypes = QStringList() << QStringLiteral("image/jpeg") << QStringLiteral("image/png") << QStringLiteral("image/gif") << QStringLiteral("image/tiff");
 
-        stackedWidget->widget(swpHTML)->setEnabled(false);
-        if (swpOkular >= 0 && okularPart != NULL) {
+        if (swpHTML >= 0)
+            stackedWidget->widget(swpHTML)->setEnabled(false);
+        if (swpOkular >= 0 && okularPart != nullptr) {
             stackedWidget->widget(swpOkular)->setEnabled(false);
             okularPart->closeUrl();
         }
-#ifdef HAVE_QTWEBKIT // krazy:exclude=cpp
+#ifdef HAVE_WEBENGINEWIDGETS
         htmlWidget->stop();
-#else // HAVE_QTWEBKIT
-        htmlPart->closeUrl();
-#endif // HAVE_QTWEBKIT
+#else // HAVE_WEBENGINEWIDGETS
+#ifdef HAVE_WEBKITWIDGETS
+        htmlWidget->stop();
+#else // HAVE_WEBKITWIDGETS
+        if (swpHTML >= 0 && htmlPart != nullptr)
+            htmlPart->closeUrl();
+#endif // HAVE_WEBKITWIDGETS
+#endif // HAVE_WEBENGINEWIDGETS
 
-        if (okularPart != NULL && okularMimetypes.contains(urlInfo.mimeType)) {
+        if (swpOkular >= 0 && okularPart != nullptr && okularMimetypes.contains(urlInfo.mimeType)) {
             p->setCursor(Qt::BusyCursor);
             showMessage(i18n("Loading...")); // krazy:exclude=qmethods
             return okularPart->openUrl(urlInfo.url);
         } else if (htmlMimetypes.contains(urlInfo.mimeType)) {
             p->setCursor(Qt::BusyCursor);
             showMessage(i18n("Loading...")); // krazy:exclude=qmethods
-#ifdef HAVE_QTWEBKIT // krazy:exclude=cpp
+#ifdef HAVE_WEBENGINEWIDGETS
             htmlWidget->load(urlInfo.url);
             return true;
-#else // HAVE_QTWEBKIT
-            return htmlPart->openUrl(urlInfo.url);
-#endif // HAVE_QTWEBKIT
+#else // HAVE_WEBENGINEWIDGETS
+#ifdef HAVE_WEBKITWIDGETS
+            htmlWidget->load(urlInfo.url);
+            return true;
+#else // HAVE_WEBKITWIDGETS
+            return (swpHTML >= 0 && htmlPart != nullptr) ? htmlPart->openUrl(urlInfo.url) : false;
+#endif // HAVE_WEBKITWIDGETS
+#endif // HAVE_WEBENGINEWIDGETS
         } else if (imageMimetypes.contains(urlInfo.mimeType)) {
             p->setCursor(Qt::BusyCursor);
-            message->setPixmap(QPixmap(urlInfo.url.pathOrUrl()));
-            showPart(NULL, message);
+            message->setPixmap(QPixmap(urlInfo.url.url(QUrl::PreferLocalFile)));
+            showPart(nullptr, message);
             p->unsetCursor();
             return true;
         } else {
             QString additionalInformation;
-            if (urlInfo.mimeType == QLatin1String("application/pdf"))
-                additionalInformation = i18nc("Additional information in case there is not KPart available for mime type 'application/pdf'", "<br/><br/>Please install <a href=\"https://userbase.kde.org/Okular\">Okular</a> for KDE&nbsp;4 to make use of its PDF viewing component.<br/>Okular for KDE Frameworks&nbsp;5 will not work.");
+            if (urlInfo.mimeType == QStringLiteral("application/pdf"))
+                additionalInformation = i18nc("Additional information in case there is not KPart available for mime type 'application/pdf'", "<br/><br/>Please install <a href=\"https://userbase.kde.org/Okular\">Okular</a> for KDE Frameworks&nbsp;5 to make use of its PDF viewing component.<br/>Okular for KDE&nbsp;4 will not work.");
             showMessage(i18nc("First parameter is mime type, second parameter is optional information (may be empty)", "<qt>Don't know how to show mimetype '%1'.%2</qt>", urlInfo.mimeType, additionalInformation)); // krazy:exclude=qmethods
         }
 
@@ -479,46 +537,53 @@ public:
     }
 
     void openExternally() {
-        KUrl url(cbxEntryToUrlInfo[urlComboBox->currentIndex()].url);
+        QUrl url(cbxEntryToUrlInfo[urlComboBox->currentIndex()].url);
         /// Guess mime type for url to open
-        KMimeType::Ptr mimeType = FileInfo::mimeTypeForUrl(url);
-        const QString mimeTypeName = mimeType->name();
+        QMimeType mimeType = FileInfo::mimeTypeForUrl(url);
+        const QString mimeTypeName = mimeType.name();
         /// Ask KDE subsystem to open url in viewer matching mime type
+#if KIO_VERSION < 0x051f00 // < 5.31.0
         KRun::runUrl(url, mimeTypeName, p, false, false);
+#else // KIO_VERSION < 0x051f00 // >= 5.31.0
+        KRun::runUrl(url, mimeTypeName, p, KRun::RunFlags());
+#endif // KIO_VERSION < 0x051f00
     }
 
-    UrlInfo urlMetaInfo(const KUrl &url) {
+    UrlInfo urlMetaInfo(const QUrl &url) {
         UrlInfo result;
         result.url = url;
 
-        if (!isLocalOrRelative(url) && url.fileName().isEmpty()) {
+        if (!KBibTeX::isLocalOrRelative(url) && url.fileName().isEmpty()) {
             /// URLs not pointing to a specific file should be opened with a web browser component
-            result.icon = KIcon("text-html");
-            result.mimeType = QLatin1String("text/html");
+            result.icon = QIcon::fromTheme(QStringLiteral("text-html"));
+            result.mimeType = QStringLiteral("text/html");
             return result;
         }
 
-        int accuracy = 0;
-        KMimeType::Ptr mimeTypePtr = KMimeType::findByUrl(url, 0, isLocalOrRelative(url), true, &accuracy);
+        QMimeType mimeType = FileInfo::mimeTypeForUrl(url);
+        // FIXME accuracy, necessary:
+        /*
         if (accuracy < 50) {
-            mimeTypePtr = KMimeType::findByPath(url.fileName(), 0, true, &accuracy);
+        QMimeDatabase db;
+            mimeType = db.mimeTypeForFile(url.fileName());
         }
-        result.mimeType = mimeTypePtr->name();
-        result.icon = KIcon(mimeTypePtr->iconName());
+        */
+        result.mimeType = mimeType.name();
+        result.icon = QIcon::fromTheme(mimeType.iconName());
 
-        if (result.mimeType == QLatin1String("application/octet-stream")) {
+        if (result.mimeType == QStringLiteral("application/octet-stream")) {
             /// application/octet-stream is a fall-back if KDE did not know better
-            result.icon = KIcon("text-html");
-            result.mimeType = QLatin1String("text/html");
-        } else if ((result.mimeType.isEmpty() || result.mimeType == QLatin1String("inode/directory")) && (result.url.protocol() == QLatin1String("http") || result.url.protocol() == QLatin1String("https"))) {
+            result.icon = QIcon::fromTheme(QStringLiteral("text-html"));
+            result.mimeType = QStringLiteral("text/html");
+        } else if ((result.mimeType.isEmpty() || result.mimeType == QStringLiteral("inode/directory")) && (result.url.scheme() == QStringLiteral("http") || result.url.scheme() == QStringLiteral("https"))) {
             /// directory via http means normal webpage (not browsable directory)
-            result.icon = KIcon("text-html");
-            result.mimeType = QLatin1String("text/html");
+            result.icon = QIcon::fromTheme(QStringLiteral("text-html"));
+            result.mimeType = QStringLiteral("text/html");
         }
 
-        if (url.pathOrUrl().startsWith(arXivPDFUrlStart)) {
-            result.icon = KIcon("application-pdf");
-            result.mimeType = QLatin1String("application/pdf");
+        if (url.url(QUrl::PreferLocalFile).startsWith(arXivPDFUrlStart)) {
+            result.icon = QIcon::fromTheme(QStringLiteral("application-pdf"));
+            result.mimeType = QStringLiteral("application/pdf");
         }
 
         return result;
@@ -532,7 +597,7 @@ public:
         /// get dock where this widget is inside
         /// static cast is save as constructor requires parent to be QDockWidget
         QDockWidget *pp = static_cast<QDockWidget *>(p->parent());
-        return pp != NULL && !pp->isHidden();
+        return pp != nullptr && !pp->isHidden();
     }
 
     void loadState() {
@@ -547,14 +612,14 @@ public:
     }
 };
 
-const QString DocumentPreview::DocumentPreviewPrivate::arXivPDFUrlStart = QLatin1String("http://arxiv.org/pdf/");
-const QString DocumentPreview::DocumentPreviewPrivate::configGroupName = QLatin1String("URL Preview");
-const QString DocumentPreview::DocumentPreviewPrivate::onlyLocalFilesCheckConfig = QLatin1String("OnlyLocalFiles");
+const QString DocumentPreview::DocumentPreviewPrivate::arXivPDFUrlStart = QStringLiteral("http://arxiv.org/pdf/");
+const QString DocumentPreview::DocumentPreviewPrivate::configGroupName = QStringLiteral("URL Preview");
+const QString DocumentPreview::DocumentPreviewPrivate::onlyLocalFilesCheckConfig = QStringLiteral("OnlyLocalFiles");
 
 DocumentPreview::DocumentPreview(QDockWidget *parent)
         : QWidget(parent), d(new DocumentPreviewPrivate(this))
 {
-    connect(parent, SIGNAL(visibilityChanged(bool)), this, SLOT(visibilityChanged(bool)));
+    connect(parent, &QDockWidget::visibilityChanged, this, &DocumentPreview::visibilityChanged);
 }
 
 DocumentPreview::~DocumentPreview()
@@ -562,7 +627,7 @@ DocumentPreview::~DocumentPreview()
     delete d;
 }
 
-void DocumentPreview::setElement(QSharedPointer<Element> element, File *)
+void DocumentPreview::setElement(QSharedPointer<Element> element, const File *)
 {
     d->entry = element.dynamicCast<const Entry>();
     d->update();
@@ -573,7 +638,7 @@ void DocumentPreview::openExternally()
     d->openExternally();
 }
 
-void DocumentPreview::setBibTeXUrl(const KUrl &url)
+void DocumentPreview::setBibTeXUrl(const QUrl &url)
 {
     d->baseUrl = url;
 }
@@ -599,12 +664,12 @@ void DocumentPreview::statFinished(KJob *kjob)
     KIO::StatJob *job = static_cast<KIO::StatJob *>(kjob);
     d->runningJobs.removeOne(job);
     if (!job->error()) {
-        const KUrl url = job->mostLocalUrl();
+        const QUrl url = job->mostLocalUrl();
         DocumentPreviewPrivate::UrlInfo urlInfo = d->urlMetaInfo(url);
         setCursor(d->runningJobs.isEmpty() ? Qt::ArrowCursor : Qt::BusyCursor);
         d->addUrl(urlInfo);
     } else {
-        kDebug() << job->error() << job->errorString();
+        qCWarning(LOG_KBIBTEX_PROGRAM) << job->error() << job->errorString();
     }
 
     if (d->runningJobs.isEmpty()) {
@@ -633,16 +698,20 @@ void DocumentPreview::loadingFinished()
 
 void DocumentPreview::linkActivated(const QString &link)
 {
-    if (link == QLatin1String("disableonlylocalfiles"))
+    if (link == QStringLiteral("disableonlylocalfiles"))
         d->onlyLocalFilesButton->setChecked(true);
-    else if (link.startsWith(QLatin1String("http://")) || link.startsWith(QLatin1String("https://"))) {
-        const KUrl urlToOpen = KUrl::fromUserInput(link);
+    else if (link.startsWith(QStringLiteral("http://")) || link.startsWith(QStringLiteral("https://"))) {
+        const QUrl urlToOpen = QUrl::fromUserInput(link);
         if (urlToOpen.isValid()) {
             /// Guess mime type for url to open
-            KMimeType::Ptr mimeType = FileInfo::mimeTypeForUrl(urlToOpen);
-            const QString mimeTypeName = mimeType->name();
+            QMimeType mimeType = FileInfo::mimeTypeForUrl(urlToOpen);
+            const QString mimeTypeName = mimeType.name();
             /// Ask KDE subsystem to open url in viewer matching mime type
+#if KIO_VERSION < 0x051f00 // < 5.31.0
             KRun::runUrl(urlToOpen, mimeTypeName, this, false, false);
+#else // KIO_VERSION < 0x051f00 // >= 5.31.0
+            KRun::runUrl(urlToOpen, mimeTypeName, this, KRun::RunFlags());
+#endif // KIO_VERSION < 0x051f00
         }
     }
 }
