@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,24 +12,26 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "searchresults.h"
 
 #include <QGridLayout>
 #include <QLabel>
+#include <QPushButton>
+#include <QAction>
 
-#include <KLocale>
-#include <KPushButton>
-#include <KDebug>
-#include <KAction>
+#include <KLocalizedString>
+#include <KIconLoader>
 
 #include "file.h"
 #include "clipboard.h"
 #include "fileview.h"
-#include "filemodel.h"
+#include "filedelegate.h"
+#include "models/filemodel.h"
 #include "idsuggestions.h"
+#include "logging_program.h"
 
 class SearchResults::SearchResultsPrivate
 {
@@ -42,18 +44,18 @@ public:
     File *file;
     QWidget *widgetCannotImport;
     QLabel *labelCannotImportMsg;
-    KPushButton *buttonImport;
+    QPushButton *buttonImport;
     FileView *resultList, *mainEditor;
-    KAction *actionViewCurrent, *actionImportSelected, *actionCopySelected;
+    QAction *actionViewCurrent, *actionImportSelected, *actionCopySelected;
 
     SearchResultsPrivate(MDIWidget *mdiWidget, SearchResults *parent)
-        : /* UNUSED p(parent),*/ m(mdiWidget), file(new File()), mainEditor(NULL) {
+        : /* UNUSED p(parent),*/ m(mdiWidget), file(new File()), mainEditor(nullptr) {
         QGridLayout *layout = new QGridLayout(parent);
         layout->setMargin(0);
         layout->setColumnStretch(0, 1);
         layout->setColumnStretch(1, 0);
 
-        resultList = new FileView(QLatin1String("SearchResults"), parent);
+        resultList = new FileView(QStringLiteral("SearchResults"), parent);
         resultList->setItemDelegate(new FileDelegate(resultList));
         resultList->setReadOnly(true);
         resultList->setFrameShadow(QFrame::Sunken);
@@ -73,14 +75,14 @@ public:
         layoutCannotImport->addStretch(10);
         QLabel *label = new QLabel(widgetCannotImport);
         label->setAlignment(Qt::AlignCenter | Qt::AlignVCenter);
-        label->setPixmap(KIconLoader::global()->loadIcon("dialog-warning", KIconLoader::Dialog, KIconLoader::SizeSmall));
+        label->setPixmap(KIconLoader::global()->loadIcon(QStringLiteral("dialog-warning"), KIconLoader::Dialog, KIconLoader::SizeSmall));
         layoutCannotImport->addWidget(label);
         labelCannotImportMsg = new QLabel(widgetCannotImport);
         labelCannotImportMsg->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
         layoutCannotImport->addWidget(labelCannotImportMsg);
         /// see also updateCannotImportMessage()
 
-        buttonImport = new KPushButton(KIcon("svn-update"), i18n("Import"), parent);
+        buttonImport = new QPushButton(QIcon::fromTheme(QStringLiteral("svn-update")), i18n("Import"), parent);
         layout->addWidget(buttonImport, 1, 1, 1, 1);
         buttonImport->setEnabled(false);
 
@@ -90,24 +92,24 @@ public:
         model->setSourceModel(fileModel);
         resultList->setModel(model);
 
-        actionViewCurrent = new KAction(KIcon("document-preview"), i18n("View Element"), parent);
+        actionViewCurrent = new QAction(QIcon::fromTheme(QStringLiteral("document-preview")), i18n("View Element"), parent);
         resultList->addAction(actionViewCurrent);
         actionViewCurrent->setEnabled(false);
-        connect(actionViewCurrent, SIGNAL(triggered()), resultList, SLOT(viewCurrentElement()));
+        connect(actionViewCurrent, &QAction::triggered, resultList, &FileView::viewCurrentElement);
 
-        actionImportSelected = new KAction(KIcon("svn-update"), i18n("Import"), parent);
+        actionImportSelected = new QAction(QIcon::fromTheme(QStringLiteral("svn-update")), i18n("Import"), parent);
         resultList->addAction(actionImportSelected);
         actionImportSelected->setEnabled(false);
-        connect(actionImportSelected, SIGNAL(triggered()), parent, SLOT(importSelected()));
+        connect(actionImportSelected, &QAction::triggered, parent, &SearchResults::importSelected);
 
-        actionCopySelected = new KAction(KIcon("edit-copy"), i18n("Copy"), parent);
+        actionCopySelected = new QAction(QIcon::fromTheme(QStringLiteral("edit-copy")), i18n("Copy"), parent);
         resultList->addAction(actionCopySelected);
         actionCopySelected->setEnabled(false);
-        connect(actionCopySelected, SIGNAL(triggered()), clipboard, SLOT(copy()));
+        connect(actionCopySelected, &QAction::triggered, clipboard, &Clipboard::copy);
 
-        connect(resultList, SIGNAL(doubleClicked(QModelIndex)), resultList, SLOT(viewCurrentElement()));
-        connect(resultList, SIGNAL(selectedElementsChanged()), parent, SLOT(updateGUI()));
-        connect(buttonImport, SIGNAL(clicked()), parent, SLOT(importSelected()));
+        connect(resultList, &FileView::doubleClicked, resultList, &FileView::viewCurrentElement);
+        connect(resultList, &FileView::selectedElementsChanged, parent, &SearchResults::updateGUI);
+        connect(buttonImport, &QPushButton::clicked, parent, &SearchResults::importSelected);
 
         updateCannotImportMessage();
     }
@@ -117,8 +119,7 @@ public:
     }
 
     void clear() {
-        file->clear();
-        resultList->fileModel()->reset();
+        resultList->fileModel()->clear();
     }
 
     bool insertElement(QSharedPointer<Element> element) {
@@ -140,13 +141,13 @@ public:
 
     void updateCannotImportMessage() {
         const bool isEmpty = resultList->model()->rowCount() == 0;
-        if (mainEditor == NULL && isEmpty)
+        if (mainEditor == nullptr && isEmpty)
             labelCannotImportMsg->setText(i18n("No results to import and no bibliography open to import to."));
-        else if (mainEditor == NULL)
+        else if (mainEditor == nullptr)
             labelCannotImportMsg->setText(i18n("No bibliography open to import to."));
         else if (isEmpty)
             labelCannotImportMsg->setText(i18n("No results to import."));
-        widgetCannotImport->setVisible(mainEditor == NULL || isEmpty);
+        widgetCannotImport->setVisible(mainEditor == nullptr || isEmpty);
     }
 };
 
@@ -184,23 +185,25 @@ void SearchResults::documentSwitched(FileView *oldEditor, FileView *newEditor)
 void SearchResults::updateGUI()
 {
     d->updateCannotImportMessage();
-    d->buttonImport->setEnabled(d->mainEditor != NULL && !d->resultList->selectedElements().isEmpty());
+    d->buttonImport->setEnabled(d->mainEditor != nullptr && !d->resultList->selectedElements().isEmpty());
     d->actionImportSelected->setEnabled(d->buttonImport->isEnabled());
     d->actionCopySelected->setEnabled(!d->resultList->selectedElements().isEmpty());
-    d->actionViewCurrent->setEnabled(d->resultList->currentElement() != NULL);
+    d->actionViewCurrent->setEnabled(d->resultList->currentElement() != nullptr);
 }
 
 void SearchResults::importSelected()
 {
-    Q_ASSERT_X(d->mainEditor != NULL, "SearchResults::importSelected", "d->mainEditor is NULL");
+    Q_ASSERT_X(d->mainEditor != nullptr, "SearchResults::importSelected", "d->mainEditor is NULL");
 
     FileModel *targetModel = d->mainEditor->fileModel();
     FileModel *sourceModel = d->resultList->fileModel();
-    QList<QModelIndex> selList = d->resultList->selectionModel()->selectedRows();
-    for (QList<QModelIndex>::ConstIterator it = selList.constBegin(); it != selList.constEnd(); ++it) {
+    if (targetModel == nullptr || sourceModel == nullptr) return; ///< either source or target model is invalid
+
+    const QModelIndexList selList = d->resultList->selectionModel()->selectedRows();
+    for (const QModelIndex &modelIndex : selList) {
         /// Map from visible row to 'real' row
         /// that may be hidden through sorting
-        int row = d->resultList->sortFilterProxyModel()->mapToSource(*it).row();
+        int row = d->resultList->sortFilterProxyModel()->mapToSource(modelIndex).row();
         /// Should only be an Entry,
         /// everthing else is unexpected
         QSharedPointer<Entry> entry = sourceModel->element(row).dynamicCast<Entry>();
@@ -210,7 +213,7 @@ void SearchResults::importSelected()
             QSharedPointer<Entry> clone(new Entry(*entry));
             targetModel->insertRow(clone, targetModel->rowCount());
         } else
-            kWarning() << "Trying to import something that isn't an Entry";
+            qCWarning(LOG_KBIBTEX_PROGRAM) << "Trying to import something that isn't an Entry";
     }
 
     if (!selList.isEmpty())
