@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,7 +12,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "valuelist.h"
@@ -26,20 +26,20 @@
 #include <QScrollBar>
 #include <QTimer>
 #include <QSortFilterProxyModel>
+#include <QAction>
 
 #include <KLineEdit>
 #include <KComboBox>
-#include <KStandardDirs>
 #include <KConfigGroup>
-#include <KLocale>
-#include <KAction>
+#include <KLocalizedString>
 #include <KToggleAction>
+#include <KSharedConfig>
 
 #include "bibtexfields.h"
 #include "entry.h"
 #include "fileview.h"
 #include "valuelistmodel.h"
-#include "filemodel.h"
+#include "models/filemodel.h"
 
 class ValueList::ValueListPrivate
 {
@@ -59,16 +59,16 @@ public:
     KComboBox *comboboxFieldNames;
     KLineEdit *lineeditFilter;
     const int countWidth;
-    KAction *assignSelectionAction;
-    KAction *removeSelectionAction;
+    QAction *assignSelectionAction;
+    QAction *removeSelectionAction;
     KToggleAction *showCountColumnAction;
     KToggleAction *sortByCountAction;
 
     ValueListPrivate(ValueList *parent)
-            : p(parent), config(KSharedConfig::openConfig(QLatin1String("kbibtexrc"))), configGroupName(QLatin1String("Value List Docklet")),
-          configKeyFieldName(QLatin1String("FieldName")), configKeyShowCountColumn(QLatin1String("ShowCountColumn")),
-          configKeySortByCountAction(QLatin1String("SortByCountAction")), configKeyHeaderState(QLatin1String("HeaderState")),
-          fileView(NULL), model(NULL), sortingModel(NULL), countWidth(8 + parent->fontMetrics().width(i18n("Count"))) {
+            : p(parent), config(KSharedConfig::openConfig(QStringLiteral("kbibtexrc"))), configGroupName(QStringLiteral("Value List Docklet")),
+          configKeyFieldName(QStringLiteral("FieldName")), configKeyShowCountColumn(QStringLiteral("ShowCountColumn")),
+          configKeySortByCountAction(QStringLiteral("SortByCountAction")), configKeyHeaderState(QStringLiteral("HeaderState")),
+          fileView(nullptr), model(nullptr), sortingModel(nullptr), countWidth(8 + parent->fontMetrics().width(i18n("Count"))) {
         setupGUI();
         initialize();
     }
@@ -87,8 +87,8 @@ public:
 
         lineeditFilter = new KLineEdit(p);
         layout->addWidget(lineeditFilter);
-        lineeditFilter->setClearButtonShown(true);
-        lineeditFilter->setClickMessage(i18n("Filter value list"));
+        lineeditFilter->setClearButtonEnabled(true);
+        lineeditFilter->setPlaceholderText(i18n("Filter value list"));
 
         treeviewFieldValues = new QTreeView(p);
         layout->addWidget(treeviewFieldValues);
@@ -100,44 +100,45 @@ public:
         treeviewFieldValues->setRootIsDecorated(false);
         treeviewFieldValues->setSelectionMode(QTreeView::ExtendedSelection);
         treeviewFieldValues->setAlternatingRowColors(true);
+        treeviewFieldValues->header()->setSectionResizeMode(QHeaderView::Fixed);
 
         treeviewFieldValues->setContextMenuPolicy(Qt::ActionsContextMenu);
         /// create context menu item to start renaming
-        KAction *action = new KAction(KIcon("edit-rename"), i18n("Replace all occurrences"), p);
-        connect(action, SIGNAL(triggered()), p, SLOT(startItemRenaming()));
+        QAction *action = new QAction(QIcon::fromTheme(QStringLiteral("edit-rename")), i18n("Replace all occurrences"), p);
+        connect(action, &QAction::triggered, p, &ValueList::startItemRenaming);
         treeviewFieldValues->addAction(action);
         /// create context menu item to delete value
-        action = new KAction(KIcon("edit-table-delete-row"), i18n("Delete all occurrences"), p);
-        connect(action, SIGNAL(triggered()), p, SLOT(deleteAllOccurrences()));
+        action = new QAction(QIcon::fromTheme(QStringLiteral("edit-table-delete-row")), i18n("Delete all occurrences"), p);
+        connect(action, &QAction::triggered, p, &ValueList::deleteAllOccurrences);
         treeviewFieldValues->addAction(action);
         /// create context menu item to search for multiple selections
-        action = new KAction(KIcon("edit-find"), i18n("Search for selected values"), p);
-        connect(action, SIGNAL(triggered()), p, SLOT(searchSelection()));
+        action = new QAction(QIcon::fromTheme(QStringLiteral("edit-find")), i18n("Search for selected values"), p);
+        connect(action, &QAction::triggered, p, &ValueList::searchSelection);
         treeviewFieldValues->addAction(action);
         /// create context menu item to assign value to selected bibliography elements
-        assignSelectionAction = new KAction(KIcon("emblem-new"), i18n("Add value to selected entries"), p);
-        connect(assignSelectionAction, SIGNAL(triggered()), p, SLOT(assignSelection()));
+        assignSelectionAction = new QAction(QIcon::fromTheme(QStringLiteral("emblem-new")), i18n("Add value to selected entries"), p);
+        connect(assignSelectionAction, &QAction::triggered, p, &ValueList::assignSelection);
         treeviewFieldValues->addAction(assignSelectionAction);
         /// create context menu item to remove value from selected bibliography elements
-        removeSelectionAction = new KAction(KIcon("list-remove"), i18n("Remove value from selected entries"), p);
-        connect(removeSelectionAction, SIGNAL(triggered()), p, SLOT(removeSelection()));
+        removeSelectionAction = new QAction(QIcon::fromTheme(QStringLiteral("list-remove")), i18n("Remove value from selected entries"), p);
+        connect(removeSelectionAction, &QAction::triggered, p, &ValueList::removeSelection);
         treeviewFieldValues->addAction(removeSelectionAction);
 
         p->setEnabled(false);
 
-        connect(comboboxFieldNames, SIGNAL(activated(int)), p, SLOT(fieldNamesChanged(int)));
-        connect(comboboxFieldNames, SIGNAL(activated(int)), lineeditFilter, SLOT(clear()));
-        connect(treeviewFieldValues, SIGNAL(activated(QModelIndex)), p, SLOT(listItemActivated(QModelIndex)));
-        connect(delegate, SIGNAL(closeEditor(QWidget*)), treeviewFieldValues, SLOT(reset()));
+        connect(comboboxFieldNames, static_cast<void(QComboBox::*)(int)>(&QComboBox::activated), p, &ValueList::fieldNamesChanged);
+        connect(comboboxFieldNames, static_cast<void(QComboBox::*)(int)>(&QComboBox::activated), lineeditFilter, &KLineEdit::clear);
+        connect(treeviewFieldValues, &QTreeView::activated, p, &ValueList::listItemActivated);
+        connect(delegate, &ValueListDelegate::closeEditor, treeviewFieldValues, &QTreeView::reset);
 
         /// add context menu to header
         treeviewFieldValues->header()->setContextMenuPolicy(Qt::ActionsContextMenu);
         showCountColumnAction = new KToggleAction(i18n("Show Count Column"), treeviewFieldValues);
-        connect(showCountColumnAction, SIGNAL(triggered()), p, SLOT(showCountColumnToggled()));
+        connect(showCountColumnAction, &QAction::triggered, p, &ValueList::showCountColumnToggled);
         treeviewFieldValues->header()->addAction(showCountColumnAction);
 
         sortByCountAction = new KToggleAction(i18n("Sort by Count"), treeviewFieldValues);
-        connect(sortByCountAction, SIGNAL(triggered()), p, SLOT(sortByCountToggled()));
+        connect(sortByCountAction, &QAction::triggered, p, &ValueList::sortByCountToggled);
         treeviewFieldValues->header()->addAction(sortByCountAction);
     }
 
@@ -153,10 +154,10 @@ public:
 
         lineeditFilter->clear();
         comboboxFieldNames->clear();
-        foreach (const FieldDescription *fd, *bibtexFields) {
-            if (!fd->upperCamelCaseAlt.isEmpty()) continue; /// keep only "single" fields and not combined ones like "Author or Editor"
-            if (fd->upperCamelCase.startsWith('^')) continue; /// skip "type" and "id"
-            comboboxFieldNames->addItem(fd->label, fd->upperCamelCase);
+        for (const auto &fd : const_cast<const BibTeXFields &>(*bibtexFields)) {
+            if (!fd.upperCamelCaseAlt.isEmpty()) continue; /// keep only "single" fields and not combined ones like "Author or Editor"
+            if (fd.upperCamelCase.startsWith('^')) continue; /// skip "type" and "id"
+            comboboxFieldNames->addItem(fd.label, fd.upperCamelCase);
         }
         /// Sort the combo box locale-aware. Thus we need a SortFilterProxyModel
         QSortFilterProxyModel *proxy = new QSortFilterProxyModel(comboboxFieldNames);
@@ -179,7 +180,7 @@ public:
         QByteArray headerState = configGroup.readEntry(configKeyHeaderState, QByteArray());
         treeviewFieldValues->header()->restoreState(headerState);
 
-        connect(treeviewFieldValues->header(), SIGNAL(sortIndicatorChanged(int,Qt::SortOrder)), p, SLOT(columnsChanged()));
+        connect(treeviewFieldValues->header(), &QHeaderView::sortIndicatorChanged, p, &ValueList::columnsChanged);
     }
 
     void update() {
@@ -187,13 +188,13 @@ public:
         if (text.isEmpty()) text = comboboxFieldNames->currentText();
 
         delegate->setFieldName(text);
-        model = fileView == NULL ? NULL : fileView->valueListModel(text);
+        model = fileView == nullptr ? nullptr : fileView->valueListModel(text);
         QAbstractItemModel *usedModel = model;
-        if (usedModel != NULL) {
+        if (usedModel != nullptr) {
             model->setShowCountColumn(showCountColumnAction->isChecked());
             model->setSortBy(sortByCountAction->isChecked() ? ValueListModel::SortByCount : ValueListModel::SortByText);
 
-            if (sortingModel != NULL) delete sortingModel;
+            if (sortingModel != nullptr) delete sortingModel;
             sortingModel = new QSortFilterProxyModel(p);
             sortingModel->setSourceModel(model);
             if (treeviewFieldValues->header()->isSortIndicatorShown())
@@ -204,12 +205,11 @@ public:
             sortingModel->setFilterKeyColumn(0);
             sortingModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
             sortingModel->setFilterRole(ValueListModel::SearchTextRole);
-            connect(lineeditFilter, SIGNAL(textEdited(QString)), sortingModel, SLOT(setFilterFixedString(QString)));
+            connect(lineeditFilter, &KLineEdit::textEdited, sortingModel, &QSortFilterProxyModel::setFilterFixedString);
             sortingModel->setSortLocaleAware(true);
             usedModel = sortingModel;
         }
         treeviewFieldValues->setModel(usedModel);
-        treeviewFieldValues->header()->setResizeMode(QHeaderView::Fixed);
 
         KConfigGroup configGroup(config, configGroupName);
         configGroup.writeEntry(configKeyFieldName, text);
@@ -229,7 +229,7 @@ public:
 ValueList::ValueList(QWidget *parent)
         : QWidget(parent), d(new ValueListPrivate(this))
 {
-    QTimer::singleShot(500, this, SLOT(delayedResize()));
+    QTimer::singleShot(500, this, &ValueList::delayedResize);
 }
 
 ValueList::~ValueList()
@@ -239,22 +239,22 @@ ValueList::~ValueList()
 
 void ValueList::setFileView(FileView *fileView)
 {
-    if (d->fileView != NULL)
-        disconnect(d->fileView, SIGNAL(selectedElementsChanged()), this, SLOT(editorSelectionChanged()));
+    if (d->fileView != nullptr)
+        disconnect(d->fileView, &FileView::selectedElementsChanged, this, &ValueList::editorSelectionChanged);
     d->fileView = fileView;
-    if (d->fileView != NULL) {
-        connect(d->fileView, SIGNAL(selectedElementsChanged()), this, SLOT(editorSelectionChanged()));
-        connect(d->fileView, SIGNAL(destroyed()), this, SLOT(editorDestroyed()));
+    if (d->fileView != nullptr) {
+        connect(d->fileView, &FileView::selectedElementsChanged, this, &ValueList::editorSelectionChanged);
+        connect(d->fileView, &FileView::destroyed, this, &ValueList::editorDestroyed);
     }
     editorSelectionChanged();
     update();
-    resizeEvent(NULL);
+    resizeEvent(nullptr);
 }
 
 void ValueList::update()
 {
     d->update();
-    setEnabled(d->fileView != NULL);
+    setEnabled(d->fileView != nullptr);
 }
 
 void ValueList::resizeEvent(QResizeEvent *)
@@ -291,7 +291,8 @@ void ValueList::searchSelection()
     SortFilterFileModel::FilterQuery fq;
     fq.combination = SortFilterFileModel::EveryTerm;
     fq.field = fieldText;
-    foreach (const QModelIndex &index, d->treeviewFieldValues->selectionModel()->selectedIndexes()) {
+    const auto selectedIndexes = d->treeviewFieldValues->selectionModel()->selectedIndexes();
+    for (const QModelIndex &index : selectedIndexes) {
         if (index.column() == 0) {
             QString itemText = index.data(ValueListModel::SearchTextRole).toString();
             fq.terms << itemText;
@@ -317,7 +318,7 @@ void ValueList::assignSelection() {
 
     /// Go through all selected elements in current editor
     const QList<QSharedPointer<Element> > &selection = d->fileView->selectedElements();
-    foreach (const QSharedPointer<Element> &element, selection) {
+    for (const auto &element : selection) {
         /// Only entries (not macros or comments) are of interest
         QSharedPointer<Entry> entry = element.dynamicCast<Entry>();
         if (!entry.isNull()) {
@@ -328,14 +329,15 @@ void ValueList::assignSelection() {
                 /// Fields for which multiple values are valid
                 bool valueItemAlreadyContained = false; ///< add only if to-be-assigned value is not yet contained
                 Value entrysValueForField = entry->value(field);
-                foreach (const QSharedPointer<ValueItem> &containedValueItem, entrysValueForField) {
+                for (const auto &containedValueItem : const_cast<const Value &>(entrysValueForField)) {
                     valueItemAlreadyContained |= PlainTextValue::text(containedValueItem) == toBeAssignedValueText;
                     if (valueItemAlreadyContained) break;
                 }
 
                 if (!valueItemAlreadyContained) {
                     /// Add each ValueItem from the to-be-assigned value to the entry's value for this field
-                    foreach (const QSharedPointer<ValueItem> &newValueItem, toBeAssignedValue) {
+                    entrysValueForField.reserve(toBeAssignedValue.size());
+                    for (const auto &newValueItem : toBeAssignedValue) {
                         entrysValueForField.append(newValueItem);
                     }
                     /// "Write back" value to field in entry
@@ -375,7 +377,7 @@ void ValueList::removeSelection() {
 
     /// Go through all selected elements in current editor
     const QList<QSharedPointer<Element> > &selection = d->fileView->selectedElements();
-    foreach (const QSharedPointer<Element> &element, selection) {
+    for (const auto &element : selection) {
         /// Only entries (not macros or comments) are of interest
         QSharedPointer<Entry> entry = element.dynamicCast<Entry>();
         if (!entry.isNull()) {
@@ -428,10 +430,10 @@ void ValueList::deleteAllOccurrences()
 
 void ValueList::showCountColumnToggled()
 {
-    if (d->model != NULL)
+    if (d->model != nullptr)
         d->model->setShowCountColumn(d->showCountColumnAction->isChecked());
     if (d->showCountColumnAction->isChecked())
-        resizeEvent(NULL);
+        resizeEvent(nullptr);
 
     d->sortByCountAction->setEnabled(!d->showCountColumnAction->isChecked());
 
@@ -442,7 +444,7 @@ void ValueList::showCountColumnToggled()
 
 void ValueList::sortByCountToggled()
 {
-    if (d->model != NULL)
+    if (d->model != nullptr)
         d->model->setSortBy(d->sortByCountAction->isChecked() ? ValueListModel::SortByCount : ValueListModel::SortByText);
 
     KConfigGroup configGroup(d->config, d->configGroupName);
@@ -452,7 +454,7 @@ void ValueList::sortByCountToggled()
 
 void ValueList::delayedResize()
 {
-    resizeEvent(NULL);
+    resizeEvent(nullptr);
 }
 
 void ValueList::columnsChanged()
@@ -462,11 +464,11 @@ void ValueList::columnsChanged()
     configGroup.writeEntry(d->configKeyHeaderState, headerState);
     d->config->sync();
 
-    resizeEvent(NULL);
+    resizeEvent(nullptr);
 }
 
 void ValueList::editorSelectionChanged() {
-    const bool selectedElements = d->fileView == NULL ? false : d->fileView->selectedElements().count() > 0;
+    const bool selectedElements = d->fileView == nullptr ? false : d->fileView->selectedElements().count() > 0;
     d->assignSelectionAction->setEnabled(selectedElements);
     d->removeSelectionAction->setEnabled(selectedElements);
 }
@@ -474,7 +476,7 @@ void ValueList::editorSelectionChanged() {
 void ValueList::editorDestroyed() {
     /// Reset internal variable to NULL to avoid
     /// accessing invalid pointer/data later
-    d->fileView = NULL;
+    d->fileView = nullptr;
     editorSelectionChanged();
 }
 
