@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2018 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,33 +12,35 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "openfileinfo.h"
 
 #include <QString>
-#include <QLatin1String>
 #include <QTimer>
 #include <QFileInfo>
 #include <QWidget>
+#include <QUrl>
+#include <QDebug>
+#include <QApplication>
 
+#include <KLocalizedString>
 #include <KConfig>
 #include <KConfigGroup>
-#include <KDebug>
+#include <KSharedConfig>
 #include <KMimeTypeTrader>
-#include <KUrl>
-#include <kparts/part.h>
-#include <KIO/NetAccess>
+#include <KParts/Part>
+#include <KParts/ReadOnlyPart>
+#include <KParts/ReadWritePart>
 
 #include "fileimporterpdf.h"
-#include "openfileinfoadaptor.h"
+#include "logging_program.h"
 
 class OpenFileInfo::OpenFileInfoPrivate
 {
 private:
     static int globalCounter;
-    static int fileIdCounter;
     int m_counter;
 
 public:
@@ -55,23 +57,21 @@ public:
     StatusFlags flags;
     OpenFileInfoManager *openFileInfoManager;
     QString mimeType;
-    KUrl url;
+    QUrl url;
 
-    int fileId;
-
-    OpenFileInfoPrivate(OpenFileInfoManager *openFileInfoManager, const KUrl &url, const QString &mimeType, OpenFileInfo *p)
-        :  m_counter(-1), p(p), part(NULL), internalServicePtr(KService::Ptr()), internalWidgetParent(NULL), flags(0), fileId(fileIdCounter++) {
+    OpenFileInfoPrivate(OpenFileInfoManager *openFileInfoManager, const QUrl &url, const QString &mimeType, OpenFileInfo *p)
+        :  m_counter(-1), p(p), part(nullptr), internalServicePtr(KService::Ptr()), internalWidgetParent(nullptr), flags(nullptr) {
         this->openFileInfoManager = openFileInfoManager;
         this->url = url;
-        if (this->url.scheme().isEmpty())
-            kWarning() << "No scheme specified for URL" << this->url.pathOrUrl();
+        if (this->url.isValid() && this->url.scheme().isEmpty())
+            qCWarning(LOG_KBIBTEX_PROGRAM) << "No scheme specified for URL" << this->url.toDisplayString();
         this->mimeType = mimeType;
     }
 
     ~OpenFileInfoPrivate() {
-        if (part != NULL) {
+        if (part != nullptr) {
             KParts::ReadWritePart *rwp = qobject_cast<KParts::ReadWritePart *>(part);
-            if (rwp != NULL)
+            if (rwp != nullptr)
                 rwp->closeUrl(true);
             delete part;
         }
@@ -79,47 +79,59 @@ public:
 
     KParts::ReadOnlyPart *createPart(QWidget *newWidgetParent, KService::Ptr newServicePtr = KService::Ptr()) {
         if (!p->flags().testFlag(OpenFileInfo::Open)) {
-            kWarning() << "Cannot create part for a file which is not open";
-            return NULL;
+            qCWarning(LOG_KBIBTEX_PROGRAM) << "Cannot create part for a file which is not open";
+            return nullptr;
         }
 
-        Q_ASSERT_X(internalWidgetParent == NULL || internalWidgetParent == newWidgetParent, "KParts::ReadOnlyPart *OpenFileInfo::OpenFileInfoPrivate::createPart(QWidget *newWidgetParent, KService::Ptr newServicePtr = KService::Ptr())", "internal widget should be either NULL or the same one as supplied as \"newWidgetParent\"");
+        Q_ASSERT_X(internalWidgetParent == nullptr || internalWidgetParent == newWidgetParent, "KParts::ReadOnlyPart *OpenFileInfo::OpenFileInfoPrivate::createPart(QWidget *newWidgetParent, KService::Ptr newServicePtr = KService::Ptr())", "internal widget should be either NULL or the same one as supplied as \"newWidgetParent\"");
 
         /** use cached part for this parent if possible */
         if (internalWidgetParent == newWidgetParent && (newServicePtr == KService::Ptr() || internalServicePtr == newServicePtr)) {
-            Q_ASSERT_X(part != NULL, "KParts::ReadOnlyPart *OpenFileInfo::OpenFileInfoPrivate::createPart(QWidget *newWidgetParent, KService::Ptr newServicePtr = KService::Ptr())", "Part is NULL");
+            Q_ASSERT_X(part != nullptr, "KParts::ReadOnlyPart *OpenFileInfo::OpenFileInfoPrivate::createPart(QWidget *newWidgetParent, KService::Ptr newServicePtr = KService::Ptr())", "Part is NULL");
             return part;
-        } else if (part != NULL) {
+        } else if (part != nullptr) {
             KParts::ReadWritePart *rwp = qobject_cast<KParts::ReadWritePart *>(part);
-            if (rwp != NULL)
+            if (rwp != nullptr)
                 rwp->closeUrl(true);
             part->deleteLater();
-            part = NULL;
+            part = nullptr;
         }
 
         /// reset to invalid values in case something goes wrong
         internalServicePtr = KService::Ptr();
-        internalWidgetParent = NULL;
+        internalWidgetParent = nullptr;
 
-        if (newServicePtr.isNull()) {
+        if (!newServicePtr) {
             /// no valid KService has been passed
             /// try to find a read-write part to open file
             newServicePtr = p->defaultService();
         }
-        if (newServicePtr.isNull()) {
-            kError() << "Cannot find service to handle mimetype " << mimeType << endl;
-            return NULL;
+        if (!newServicePtr) {
+            qCDebug(LOG_KBIBTEX_PROGRAM) << "PATH=" << getenv("PATH");
+            qCDebug(LOG_KBIBTEX_PROGRAM) << "LD_LIBRARY_PATH=" << getenv("LD_LIBRARY_PATH");
+            qCDebug(LOG_KBIBTEX_PROGRAM) << "XDG_DATA_DIRS=" << getenv("XDG_DATA_DIRS");
+            qCDebug(LOG_KBIBTEX_PROGRAM) << "QT_PLUGIN_PATH=" << getenv("QT_PLUGIN_PATH");
+            qCDebug(LOG_KBIBTEX_PROGRAM) << "KDEDIRS=" << getenv("KDEDIRS");
+            qCCritical(LOG_KBIBTEX_PROGRAM) << "Cannot find service to handle mimetype " << mimeType << endl;
+            return nullptr;
         }
 
-        part = newServicePtr->createInstance<KParts::ReadWritePart>(newWidgetParent, (QObject *)newWidgetParent);
-        if (part == NULL) {
+        QString errorString;
+        part = newServicePtr->createInstance<KParts::ReadWritePart>(newWidgetParent, (QObject *)newWidgetParent, QVariantList(), &errorString);
+        if (part == nullptr) {
+            qCDebug(LOG_KBIBTEX_PROGRAM) << "PATH=" << getenv("PATH");
+            qCDebug(LOG_KBIBTEX_PROGRAM) << "LD_LIBRARY_PATH=" << getenv("LD_LIBRARY_PATH");
+            qCDebug(LOG_KBIBTEX_PROGRAM) << "XDG_DATA_DIRS=" << getenv("XDG_DATA_DIRS");
+            qCDebug(LOG_KBIBTEX_PROGRAM) << "QT_PLUGIN_PATH=" << getenv("QT_PLUGIN_PATH");
+            qCDebug(LOG_KBIBTEX_PROGRAM) << "KDEDIRS=" << getenv("KDEDIRS");
+            qCWarning(LOG_KBIBTEX_PROGRAM) << "Could not instantiate read-write part for service" << newServicePtr->name() << "(mimeType=" << mimeType << ", library=" << newServicePtr->library() << ", error msg=" << errorString << ")";
             /// creating a read-write part failed, so maybe it is read-only (like Okular's PDF viewer)?
-            part = newServicePtr->createInstance<KParts::ReadOnlyPart>(newWidgetParent, (QObject *)newWidgetParent);
+            part = newServicePtr->createInstance<KParts::ReadOnlyPart>(newWidgetParent, (QObject *)newWidgetParent, QVariantList(), &errorString);
         }
-        if (part == NULL) {
+        if (part == nullptr) {
             /// still cannot create part, must be error
-            kError() << "Cannot find part for mimetype " << mimeType << endl;
-            return NULL;
+            qCCritical(LOG_KBIBTEX_PROGRAM) << "Could not instantiate part for service" << newServicePtr->name() << "(mimeType=" << mimeType << ", library=" << newServicePtr->library() << ", error msg=" << errorString << ")";
+            return nullptr;
         }
 
         if (url.isValid()) {
@@ -130,14 +142,14 @@ public:
             p->addFlags(OpenFileInfo::HasName);
         } else {
             /// initialize part with empty document
-            part->openUrl(KUrl());
+            part->openUrl(QUrl());
         }
         p->addFlags(OpenFileInfo::Open);
 
         internalServicePtr = newServicePtr;
         internalWidgetParent = newWidgetParent;
 
-        Q_ASSERT_X(part != NULL, "KParts::ReadOnlyPart *OpenFileInfo::OpenFileInfoPrivate::createPart(QWidget *newWidgetParent, KService::Ptr newServicePtr = KService::Ptr())", "Creation of part failed, is NULL"); /// test should not be necessary, but just to be save ...
+        Q_ASSERT_X(part != nullptr, "KParts::ReadOnlyPart *OpenFileInfo::OpenFileInfoPrivate::createPart(QWidget *newWidgetParent, KService::Ptr newServicePtr = KService::Ptr())", "Creation of part failed, is NULL"); /// test should not be necessary, but just to be save ...
         return part;
     }
 
@@ -145,26 +157,25 @@ public:
         if (!url.isValid() && m_counter < 0)
             m_counter = ++globalCounter;
         else if (url.isValid())
-            kWarning() << "This function should not be called if URL is valid";
+            qCWarning(LOG_KBIBTEX_PROGRAM) << "This function should not be called if URL is valid";
         return m_counter;
     }
 
 };
 
 int OpenFileInfo::OpenFileInfoPrivate::globalCounter = 0;
-int OpenFileInfo::OpenFileInfoPrivate::fileIdCounter = OpenFileInfo::FileIdStart;
-const QString OpenFileInfo::OpenFileInfoPrivate::dateTimeFormat = QLatin1String("yyyy-MM-dd-hh-mm-ss-zzz");
-const QString OpenFileInfo::OpenFileInfoPrivate::keyLastAccess = QLatin1String("LastAccess");
-const QString OpenFileInfo::OpenFileInfoPrivate::keyURL = QLatin1String("URL");
+const QString OpenFileInfo::OpenFileInfoPrivate::dateTimeFormat = QStringLiteral("yyyy-MM-dd-hh-mm-ss-zzz");
+const QString OpenFileInfo::OpenFileInfoPrivate::keyLastAccess = QStringLiteral("LastAccess");
+const QString OpenFileInfo::OpenFileInfoPrivate::keyURL = QStringLiteral("URL");
 
-OpenFileInfo::OpenFileInfo(OpenFileInfoManager *openFileInfoManager, const KUrl &url)
-        : d(new OpenFileInfoPrivate(openFileInfoManager, url, FileInfo::mimeTypeForUrl(url)->name(), this))
+OpenFileInfo::OpenFileInfo(OpenFileInfoManager *openFileInfoManager, const QUrl &url)
+        : d(new OpenFileInfoPrivate(openFileInfoManager, url, FileInfo::mimeTypeForUrl(url).name(), this))
 {
-    new OpenFileInfoAdaptor(this, openFileInfoManager);
+    // nothing
 }
 
 OpenFileInfo::OpenFileInfo(OpenFileInfoManager *openFileInfoManager, const QString &mimeType)
-        : d(new OpenFileInfoPrivate(openFileInfoManager, KUrl(), mimeType, this))
+        : d(new OpenFileInfoPrivate(openFileInfoManager, QUrl(), mimeType, this))
 {
     // nothing
 }
@@ -174,17 +185,17 @@ OpenFileInfo::~OpenFileInfo()
     delete d;
 }
 
-void OpenFileInfo::setUrl(const KUrl &url)
+void OpenFileInfo::setUrl(const QUrl &url)
 {
-    Q_ASSERT_X(url.isValid(), "void OpenFileInfo::setUrl(const KUrl&)", "URL is not valid");
+    Q_ASSERT_X(url.isValid(), "void OpenFileInfo::setUrl(const QUrl&)", "URL is not valid");
     d->url = url;
     if (d->url.scheme().isEmpty())
-        kWarning() << "No scheme specified for URL" << d->url.pathOrUrl();
-    d->mimeType = FileInfo::mimeTypeForUrl(url)->name();
+        qCWarning(LOG_KBIBTEX_PROGRAM) << "No scheme specified for URL" << d->url.toDisplayString();
+    d->mimeType = FileInfo::mimeTypeForUrl(url).name();
     addFlags(OpenFileInfo::HasName);
 }
 
-KUrl OpenFileInfo::url() const
+QUrl OpenFileInfo::url() const
 {
     return d->url;
 }
@@ -192,7 +203,7 @@ KUrl OpenFileInfo::url() const
 bool OpenFileInfo::isModified() const
 {
     KParts::ReadWritePart *rwPart = qobject_cast< KParts::ReadWritePart *>(d->part);
-    if (rwPart == NULL)
+    if (rwPart == nullptr)
         return false;
     else
         return rwPart->isModified();
@@ -201,7 +212,7 @@ bool OpenFileInfo::isModified() const
 bool OpenFileInfo::save()
 {
     KParts::ReadWritePart *rwPart = qobject_cast< KParts::ReadWritePart *>(d->part);
-    if (rwPart == NULL)
+    if (rwPart == nullptr)
         return true;
     else
         return rwPart->save();
@@ -209,16 +220,16 @@ bool OpenFileInfo::save()
 
 bool OpenFileInfo::close()
 {
-    if (d->part == NULL) {
+    if (d->part == nullptr) {
         /// if there is no part, closing always "succeeds"
         return true;
     }
 
     KParts::ReadWritePart *rwp = qobject_cast<KParts::ReadWritePart *>(d->part);
-    if (rwp == NULL || rwp->closeUrl(true)) {
+    if (rwp == nullptr || rwp->closeUrl(true)) {
         d->part->deleteLater();
-        d->part = NULL;
-        d->internalWidgetParent = NULL;
+        d->part = nullptr;
+        d->internalWidgetParent = nullptr;
         return true;
     }
     return false;
@@ -240,11 +251,13 @@ QString OpenFileInfo::shortCaption() const
 QString OpenFileInfo::fullCaption() const
 {
     if (d->url.isValid())
-        return d->url.pathOrUrl();
+        return d->url.url(QUrl::PreferLocalFile);
     else
         return shortCaption();
 }
 
+/// Clazy warns: "Missing reference on non-trivial type" for argument 'servicePtr',
+/// but type 'KService::Ptr' is actually a pointer (QExplicitlySharedDataPointer).
 KParts::ReadOnlyPart *OpenFileInfo::part(QWidget *parent, KService::Ptr servicePtr)
 {
     return d->createPart(parent, servicePtr);
@@ -300,18 +313,69 @@ void OpenFileInfo::setLastAccess(const QDateTime &dateTime)
 
 KService::List OpenFileInfo::listOfServices()
 {
-    KService::List result = KMimeTypeTrader::self()->query(mimeType(), QLatin1String("KParts/ReadWritePart"));
-    if (result.isEmpty())
-        result = KMimeTypeTrader::self()->query(mimeType(), QLatin1String("KParts/ReadOnlyPart"));
+    const QString mt = mimeType();
+    /// First, try to locate KPart that can both read and write the queried MIME type
+    KService::List result = KMimeTypeTrader::self()->query(mt, QStringLiteral("KParts/ReadWritePart"));
+    if (result.isEmpty()) {
+        /// Second, if no 'writing' KPart was found, try to locate KPart that can at least read the queried MIME type
+        result = KMimeTypeTrader::self()->query(mt, QStringLiteral("KParts/ReadOnlyPart"));
+        if (result.isEmpty()) {
+            /// If not even a 'reading' KPart was found, something is off, so warn the user and stop here
+            qCWarning(LOG_KBIBTEX_PROGRAM) << "Could not find any KPart that reads or writes mimetype" << mt;
+            return result;
+        }
+    }
+
+    /// Always include KBibTeX's KPart in list of services:
+    /// First, check if KBibTeX's KPart is already in list as returned by
+    /// KMimeTypeTrader::self()->query(..)
+    bool listIncludesKBibTeXPart = false;
+    for (KService::List::ConstIterator it = result.constBegin(); it != result.constEnd(); ++it) {
+        qCDebug(LOG_KBIBTEX_PROGRAM) << "Found library for" << mt << ":" << (*it)->library();
+        listIncludesKBibTeXPart |= (*it)->library() == QStringLiteral("kbibtexpart");
+    }
+    /// Then, if KBibTeX's KPart is not in the list, try to located it by desktop name
+    if (!listIncludesKBibTeXPart) {
+        KService::Ptr kbibtexpartByDesktopName = KService::serviceByDesktopName(QStringLiteral("kbibtexpart"));
+        if (kbibtexpartByDesktopName != nullptr) {
+            result << kbibtexpartByDesktopName;
+            qCDebug(LOG_KBIBTEX_PROGRAM) << "Adding library for" << mt << ":" << kbibtexpartByDesktopName->library();
+        } else {
+            qCDebug(LOG_KBIBTEX_PROGRAM) << "Could not locate KBibTeX's KPart neither by MIME type search, nor by desktop name";
+        }
+    }
+
     return result;
 }
 
 KService::Ptr OpenFileInfo::defaultService()
 {
     const QString mt = mimeType();
-    KService::Ptr result = KMimeTypeTrader::self()->preferredService(mt, QLatin1String("KParts/ReadWritePart"));
-    if (result.isNull())
-        result = KMimeTypeTrader::self()->preferredService(mt, QLatin1String("KParts/ReadOnlyPart"));
+    KService::Ptr result;
+    if (mt == QStringLiteral("application/pdf") || mt == QStringLiteral("text/x-bibtex")) {
+        /// If either a BibTeX file or a PDF file is to be opened, enforce using
+        /// KBibTeX's part over anything else.
+        /// KBibTeX has a FileImporterPDF which allows it to load .pdf file
+        /// that got generated with KBibTeX and contain the original
+        /// .bib file as an 'attachment'.
+        /// This importer does not work with any other .pdf files!!!
+        result = KService::serviceByDesktopName(QStringLiteral("kbibtexpart"));
+    }
+    if (result == nullptr) {
+        /// First, try to locate KPart that can both read and write the queried MIME type
+        result = KMimeTypeTrader::self()->preferredService(mt, QStringLiteral("KParts/ReadWritePart"));
+        if (result == nullptr) {
+            /// Second, if no 'writing' KPart was found, try to locate KPart that can at least read the queried MIME type
+            result = KMimeTypeTrader::self()->preferredService(mt, QStringLiteral("KParts/ReadOnlyPart"));
+            if (result == nullptr && mt == QStringLiteral("text/x-bibtex"))
+                /// Third, if MIME type is for BibTeX files, try loading KBibTeX part via desktop name
+                result = KService::serviceByDesktopName(QStringLiteral("kbibtexpart"));
+        }
+    }
+    if (result != nullptr)
+        qCDebug(LOG_KBIBTEX_PROGRAM) << "Using service" << result->name() << "(" << result->comment() << ") for mime type" << mt << "through library" << result->library();
+    else
+        qCWarning(LOG_KBIBTEX_PROGRAM) << "Could not find service for mime type" << mt;
     return result;
 }
 
@@ -320,11 +384,6 @@ KService::Ptr OpenFileInfo::currentService()
     return d->internalServicePtr;
 }
 
-int OpenFileInfo::fileId() const
-{
-    return d->fileId;
-}
-
 class OpenFileInfoManager::OpenFileInfoManagerPrivate
 {
 private:
@@ -332,7 +391,6 @@ private:
     static const QString configGroupNameFavorites;
     static const QString configGroupNameOpen;
     static const int maxNumRecentlyUsedFiles, maxNumFavoriteFiles, maxNumOpenFiles;
-    QWidget *widget;
 
 public:
     OpenFileInfoManager *p;
@@ -340,8 +398,8 @@ public:
     OpenFileInfoManager::OpenFileInfoList openFileInfoList;
     OpenFileInfo *currentFileInfo;
 
-    OpenFileInfoManagerPrivate(OpenFileInfoManager *parent, QWidget *w)
-            : widget(w), p(parent), currentFileInfo(NULL) {
+    OpenFileInfoManagerPrivate(OpenFileInfoManager *parent)
+            : p(parent), currentFileInfo(nullptr) {
         // nothing
     }
 
@@ -373,30 +431,29 @@ public:
     }
 
     void readConfig(OpenFileInfo::StatusFlag statusFlag, const QString &configGroupName, int maxNumFiles) {
-        KSharedConfigPtr config = KSharedConfig::openConfig("kbibtexrc");
+        KSharedConfigPtr config = KSharedConfig::openConfig(QStringLiteral("kbibtexrc"));
 
         bool isFirst = true;
         KConfigGroup cg(config, configGroupName);
         for (int i = 0; i < maxNumFiles; ++i) {
-            KUrl fileUrl = KUrl::fromUserInput(cg.readEntry(QString(QLatin1String("%1-%2")).arg(OpenFileInfo::OpenFileInfoPrivate::keyURL).arg(i), QString()));
+            QUrl fileUrl = QUrl(cg.readEntry(QString(QStringLiteral("%1-%2")).arg(OpenFileInfo::OpenFileInfoPrivate::keyURL).arg(i), ""));
             if (!fileUrl.isValid()) break;
             if (fileUrl.scheme().isEmpty())
-                fileUrl.setScheme(QLatin1String("file"));
+                fileUrl.setScheme(QStringLiteral("file"));
 
             /// For local files, test if they exist; ignore local files that do not exist
             if (fileUrl.isLocalFile()) {
-                if (!QFileInfo(fileUrl.pathOrUrl()).exists())
+                if (!QFileInfo::exists(fileUrl.toLocalFile()))
                     continue;
-            } else if (!KIO::NetAccess::exists(fileUrl, KIO::NetAccess::SourceSide, widget))
-                continue;
+            }
 
             OpenFileInfo *ofi = p->contains(fileUrl);
-            if (ofi == NULL) {
+            if (ofi == nullptr) {
                 ofi = p->open(fileUrl);
             }
             ofi->addFlags(statusFlag);
             ofi->addFlags(OpenFileInfo::HasName);
-            ofi->setLastAccess(QDateTime::fromString(cg.readEntry(QString(QLatin1String("%1-%2")).arg(OpenFileInfo::OpenFileInfoPrivate::keyLastAccess).arg(i), ""), OpenFileInfo::OpenFileInfoPrivate::dateTimeFormat));
+            ofi->setLastAccess(QDateTime::fromString(cg.readEntry(QString(QStringLiteral("%1-%2")).arg(OpenFileInfo::OpenFileInfoPrivate::keyLastAccess).arg(i), ""), OpenFileInfo::OpenFileInfoPrivate::dateTimeFormat));
             if (isFirst) {
                 isFirst = false;
                 if (statusFlag == OpenFileInfo::Open)
@@ -406,7 +463,7 @@ public:
     }
 
     void writeConfig(OpenFileInfo::StatusFlag statusFlag, const QString &configGroupName, int maxNumFiles) {
-        KSharedConfigPtr config = KSharedConfig::openConfig("kbibtexrc");
+        KSharedConfigPtr config = KSharedConfig::openConfig(QStringLiteral("kbibtexrc"));
         KConfigGroup cg(config, configGroupName);
         OpenFileInfoManager::OpenFileInfoList list = p->filteredItems(statusFlag);
 
@@ -414,30 +471,36 @@ public:
         for (OpenFileInfoManager::OpenFileInfoList::ConstIterator it = list.constBegin(); i < maxNumFiles && it != list.constEnd(); ++it, ++i) {
             OpenFileInfo *ofi = *it;
 
-            cg.writeEntry(QString(QLatin1String("%1-%2")).arg(OpenFileInfo::OpenFileInfoPrivate::keyURL).arg(i), ofi->url().pathOrUrl());
-            cg.writeEntry(QString(QLatin1String("%1-%2")).arg(OpenFileInfo::OpenFileInfoPrivate::keyLastAccess).arg(i), ofi->lastAccess().toString(OpenFileInfo::OpenFileInfoPrivate::dateTimeFormat));
+            cg.writeEntry(QString(QStringLiteral("%1-%2")).arg(OpenFileInfo::OpenFileInfoPrivate::keyURL).arg(i), ofi->url().url(QUrl::PreferLocalFile));
+            cg.writeEntry(QString(QStringLiteral("%1-%2")).arg(OpenFileInfo::OpenFileInfoPrivate::keyLastAccess).arg(i), ofi->lastAccess().toString(OpenFileInfo::OpenFileInfoPrivate::dateTimeFormat));
         }
         for (; i < maxNumFiles; ++i) {
-            cg.deleteEntry(QString(QLatin1String("%1-%2")).arg(OpenFileInfo::OpenFileInfoPrivate::keyURL).arg(i));
-            cg.deleteEntry(QString(QLatin1String("%1-%2")).arg(OpenFileInfo::OpenFileInfoPrivate::keyLastAccess).arg(i));
+            cg.deleteEntry(QString(QStringLiteral("%1-%2")).arg(OpenFileInfo::OpenFileInfoPrivate::keyURL).arg(i));
+            cg.deleteEntry(QString(QStringLiteral("%1-%2")).arg(OpenFileInfo::OpenFileInfoPrivate::keyLastAccess).arg(i));
         }
         config->sync();
     }
 };
 
-const QString OpenFileInfoManager::OpenFileInfoManagerPrivate::configGroupNameRecentlyUsed = QLatin1String("DocumentList-RecentlyUsed");
-const QString OpenFileInfoManager::OpenFileInfoManagerPrivate::configGroupNameFavorites = QLatin1String("DocumentList-Favorites");
-const QString OpenFileInfoManager::OpenFileInfoManagerPrivate::configGroupNameOpen = QLatin1String("DocumentList-Open");
+const QString OpenFileInfoManager::OpenFileInfoManagerPrivate::configGroupNameRecentlyUsed = QStringLiteral("DocumentList-RecentlyUsed");
+const QString OpenFileInfoManager::OpenFileInfoManagerPrivate::configGroupNameFavorites = QStringLiteral("DocumentList-Favorites");
+const QString OpenFileInfoManager::OpenFileInfoManagerPrivate::configGroupNameOpen = QStringLiteral("DocumentList-Open");
 const int OpenFileInfoManager::OpenFileInfoManagerPrivate::maxNumFavoriteFiles = 256;
 const int OpenFileInfoManager::OpenFileInfoManagerPrivate::maxNumRecentlyUsedFiles = 8;
 const int OpenFileInfoManager::OpenFileInfoManagerPrivate::maxNumOpenFiles = 16;
 
+OpenFileInfoManager *OpenFileInfoManager::singleton = nullptr;
 
-OpenFileInfoManager::OpenFileInfoManager(QWidget *widget)
-        : QObject(widget), d(new OpenFileInfoManagerPrivate(this, widget))
+OpenFileInfoManager::OpenFileInfoManager(QObject *parent)
+        : QObject(parent), d(new OpenFileInfoManagerPrivate(this))
 {
-    new OpenFileInfoManagerAdaptor(this);
-    QTimer::singleShot(300, this, SLOT(delayedReadConfig()));
+    QTimer::singleShot(300, this, &OpenFileInfoManager::delayedReadConfig);
+}
+
+OpenFileInfoManager *OpenFileInfoManager::instance() {
+    if (singleton == nullptr)
+        singleton = new OpenFileInfoManager(QCoreApplication::instance());
+    return singleton;
 }
 
 OpenFileInfoManager::~OpenFileInfoManager()
@@ -448,60 +511,59 @@ OpenFileInfoManager::~OpenFileInfoManager()
 OpenFileInfo *OpenFileInfoManager::createNew(const QString &mimeType)
 {
     OpenFileInfo *result = new OpenFileInfo(this, mimeType);
-    connect(result, SIGNAL(flagsChanged(OpenFileInfo::StatusFlags)), this, SIGNAL(flagsChanged(OpenFileInfo::StatusFlags)));
+    connect(result, &OpenFileInfo::flagsChanged, this, &OpenFileInfoManager::flagsChanged);
     d->openFileInfoList << result;
     result->setLastAccess();
     return result;
 }
 
-OpenFileInfo *OpenFileInfoManager::open(const KUrl &url)
+OpenFileInfo *OpenFileInfoManager::open(const QUrl &url)
 {
-    Q_ASSERT_X(url.isValid(), "OpenFileInfo *OpenFileInfoManager::open(const KUrl&)", "URL is not valid");
+    Q_ASSERT_X(url.isValid(), "OpenFileInfo *OpenFileInfoManager::open(const QUrl&)", "URL is not valid");
 
     OpenFileInfo *result = contains(url);
-    if (result == NULL) {
+    if (result == nullptr) {
         /// file not yet open
         result = new OpenFileInfo(this, url);
-        connect(result, SIGNAL(flagsChanged(OpenFileInfo::StatusFlags)), this, SIGNAL(flagsChanged(OpenFileInfo::StatusFlags)));
+        connect(result, &OpenFileInfo::flagsChanged, this, &OpenFileInfoManager::flagsChanged);
         d->openFileInfoList << result;
     } /// else: file was already open, re-use and return existing OpenFileInfo pointer
     result->setLastAccess();
     return result;
 }
 
-OpenFileInfo *OpenFileInfoManager::contains(const KUrl &url) const
+OpenFileInfo *OpenFileInfoManager::contains(const QUrl &url) const
 {
-    if (!url.isValid()) return NULL; /// can only be unnamed file
+    if (!url.isValid()) return nullptr; /// can only be unnamed file
 
-    for (OpenFileInfoList::ConstIterator it = d->openFileInfoList.constBegin(); it != d->openFileInfoList.constEnd(); ++it) {
-        OpenFileInfo *ofi = *it;
-        if (ofi->url().equals(url))
+    for (auto *ofi : const_cast<const OpenFileInfoManager::OpenFileInfoList &>(d->openFileInfoList)) {
+        if (ofi->url() == url)
             return ofi;
     }
-    return NULL;
+    return nullptr;
 }
 
-bool OpenFileInfoManager::changeUrl(OpenFileInfo *openFileInfo, const KUrl &url)
+bool OpenFileInfoManager::changeUrl(OpenFileInfo *openFileInfo, const QUrl &url)
 {
     OpenFileInfo *previouslyContained = contains(url);
 
     /// check if old url differs from new url and old url is valid
-    if (previouslyContained != NULL && previouslyContained->flags().testFlag(OpenFileInfo::Open) && previouslyContained != openFileInfo) {
-        kDebug() << "Open file with same URL already exists, forcefully closing it" << endl;
+    if (previouslyContained != nullptr && previouslyContained->flags().testFlag(OpenFileInfo::Open) && previouslyContained != openFileInfo) {
+        qCWarning(LOG_KBIBTEX_PROGRAM) << "Open file with same URL already exists, forcefully closing it" << endl;
         close(previouslyContained);
     }
 
-    KUrl oldUrl = openFileInfo->url();
+    QUrl oldUrl = openFileInfo->url();
     openFileInfo->setUrl(url);
 
-    if (!url.equals(oldUrl) && oldUrl.isValid()) {
+    if (url != oldUrl && oldUrl.isValid()) {
         /// current document was most probabily renamed (e.g. due to "Save As")
         /// add old URL to recently used files, but exclude the open files list
         OpenFileInfo *ofi = open(oldUrl); // krazy:exclude=syscalls
         OpenFileInfo::StatusFlags statusFlags = (openFileInfo->flags() & (~OpenFileInfo::Open)) | OpenFileInfo::RecentlyUsed;
         ofi->setFlags(statusFlags);
     }
-    if (previouslyContained != NULL) {
+    if (previouslyContained != nullptr) {
         /// keep Favorite flag if set in file that have previously same URL
         if (previouslyContained->flags().testFlag(OpenFileInfo::Favorite))
             openFileInfo->setFlags(openFileInfo->flags() | OpenFileInfo::Favorite);
@@ -524,13 +586,17 @@ bool OpenFileInfoManager::changeUrl(OpenFileInfo *openFileInfo, const KUrl &url)
 
 bool OpenFileInfoManager::close(OpenFileInfo *openFileInfo)
 {
-    Q_ASSERT_X(openFileInfo != NULL, "void OpenFileInfoManager::close(OpenFileInfo *openFileInfo)", "Cannot close openFileInfo which is NULL");
+    if (openFileInfo == nullptr) {
+        qCWarning(LOG_KBIBTEX_PROGRAM) << "void OpenFileInfoManager::close(OpenFileInfo *openFileInfo): Cannot close openFileInfo which is NULL";
+        return false;
+    }
+
     bool isClosing = false;
     openFileInfo->setLastAccess();
 
     /// remove flag "open" from file to be closed and determine which file to show instead
-    OpenFileInfo *nextCurrent = (d->currentFileInfo == openFileInfo) ? NULL : d->currentFileInfo;
-    foreach (OpenFileInfo *ofi, d->openFileInfoList) {
+    OpenFileInfo *nextCurrent = (d->currentFileInfo == openFileInfo) ? nullptr : d->currentFileInfo;
+    for (OpenFileInfo *ofi : const_cast<const OpenFileInfoManager::OpenFileInfoList &>(d->openFileInfoList)) {
         if (!isClosing && ofi == openFileInfo && openFileInfo->close()) {
             isClosing = true;
             /// Mark file as closed (i.e. not open)
@@ -538,7 +604,7 @@ bool OpenFileInfoManager::close(OpenFileInfo *openFileInfo)
             /// If file has a filename, remember as recently used
             if (openFileInfo->flags().testFlag(OpenFileInfo::HasName))
                 openFileInfo->addFlags(OpenFileInfo::RecentlyUsed);
-        } else if (nextCurrent == NULL && ofi->flags().testFlag(OpenFileInfo::Open))
+        } else if (nextCurrent == nullptr && ofi->flags().testFlag(OpenFileInfo::Open))
             nextCurrent = ofi;
     }
 
@@ -558,7 +624,7 @@ bool OpenFileInfoManager::queryCloseAll()
     OpenFileInfoList restoreLaterList;
 
     /// For each file known ...
-    foreach (OpenFileInfo *openFileInfo, d->openFileInfoList) {
+    for (OpenFileInfo *openFileInfo : const_cast<const OpenFileInfoManager::OpenFileInfoList &>(d->openFileInfoList)) {
         /// Check only open file (ignore recently used, favorites, ...)
         if (openFileInfo->flags().testFlag(OpenFileInfo::Open)) {
             if (openFileInfo->close()) {
@@ -585,7 +651,7 @@ bool OpenFileInfoManager::queryCloseAll()
         /// This makes the files to be reopened when KBibTeX is
         /// restarted again (assuming that this function was
         /// called when KBibTeX is exiting).
-        foreach (OpenFileInfo *openFileInfo, restoreLaterList) {
+        for (OpenFileInfo *openFileInfo : const_cast<const OpenFileInfoManager::OpenFileInfoList &>(restoreLaterList)) {
             openFileInfo->addFlags(OpenFileInfo::Open);
         }
 
@@ -600,20 +666,22 @@ OpenFileInfo *OpenFileInfoManager::currentFile() const
     return d->currentFileInfo;
 }
 
+/// Clazy warns: "Missing reference on non-trivial type" for argument 'servicePtr',
+/// but type 'KService::Ptr' is actually a pointer (QExplicitlySharedDataPointer).
 void OpenFileInfoManager::setCurrentFile(OpenFileInfo *openFileInfo, KService::Ptr servicePtr)
 {
     bool hasChanged = d->currentFileInfo != openFileInfo;
     OpenFileInfo *previous = d->currentFileInfo;
     d->currentFileInfo = openFileInfo;
 
-    if (d->currentFileInfo != NULL) {
+    if (d->currentFileInfo != nullptr) {
         d->currentFileInfo->addFlags(OpenFileInfo::Open);
         d->currentFileInfo->setLastAccess();
     }
     if (hasChanged) {
-        if (previous != NULL) previous->setLastAccess();
+        if (previous != nullptr) previous->setLastAccess();
         emit currentChanged(openFileInfo, servicePtr);
-    } else if (openFileInfo == NULL || servicePtr != openFileInfo->currentService())
+    } else if (openFileInfo != nullptr && servicePtr != openFileInfo->currentService())
         emit currentChanged(openFileInfo, servicePtr);
 }
 
